#!/usr/bin/perl
#configure-0.9.4-lvs.pl (C) Joseph Mack 1999-2002 jmack@wm7d.net
#
#released under GPL
#This code is part of the Linux Virtual Server (LVS) project
#http://www.linuxvirtualserver.org
#
#VERSIONS
#v0.1 6 Nov 99 for kernel 2.2.12
#v0.2 7 Nov 99 for kernel 2.2.13
#v0.3 14 Nov 99 added option for varying timeout for ppc connnection
#v0.4 17 Nov 99 fixed bug in ppc setup code
#v0.5 Jan 00 	1.Changed some names to make more readable 
#		(eg SERVER_LVS_DEVICE to SERVER_VIP_DEVICE)
#		2.rc.lvs file now has VIP (and a few other variables) as $VIP rather than values
#		(makes it easier to edit the rc.lvs file)
#		3.with VS-NAT old default gw is deleted before new one added.
#v0.6 Jun 00 reads all input before doing anything. 
#		This allows me to do things dependant on two simultaneous conditions 
#		(and makes the order of input lines irrelevant)
#		The code no longer can report errors by input line, 
#		as the code has read to the end
#		of the input file before it starts to parse.
#		added rc.horms, rc.ratz
#v0.7 Nov 00 bug fixes, mon scripts now work again
#		create an ntp.conf file for realservers and director
#		autodetects location of ipchains, route, netstat etc 
#		assumes ipvsadm is in /sbin
#		rc.lvs file restarts mon on director (if it's running)
#v0.8 Apr 2001 	1. rc.lvs file restarts ntpd on director and realserver 
#		(if it's running). ntpd cannot be restarted with kill -HUP,
#		so kills process and restarts it.
#		Tries both ps -aux and ps -ef commands to find ps of ntpd. 
#		2. tested with 2.2.19,2.4.2
#		3. better checks for input params and default gw
#		4. -i option. copies output files to real-servers with ssh 
#                    and executes them
#		5. fix icmp redirects for VS-NAT, enable one net VS-NAT
#		6. fix mon.cf file for VS-NAT
#v0.8.1 Apr 2001
#		Bug fixes (thanks to Scott Jones scott@on-sitemanager.com
#		1. executables are now classed as required (eg ifconfig) 
#		and optional (eg ntpd). 
#		If an optional executable is not found, only a notice is given.
#		Previously the script exited with "fatal error".
#		2. script now handles VIP on eth devices eg eth1 on real-servers
#		3. detects if no SERVICES have been entered
#v0.9.0 Aug 2001
#		1. Detect whether ipvs is compiled into kernel or as ip_vs.o module.
#		2. If compiled into kernel, don't look for module tools (lsmod...)
#		and don't try to load modules (thanks to Scott Jones again)
#		3. set broadcast on real-servers. defaults to xxx.xxx.xxx.255 
#		unless explicitely set, even if netmask = 255.255.255.255.
#		It should be the same as the IP, eg 192.168.1.110
#		4. create directories if needed (eg $rc_mon_home).
#		5. fixed multiple masq commands when only one is needed (thanks to Alois Treindl)
#		6. Changed names (and incremented LVSCONF_FORMAT=1.1): 
#			the IP on the director on the real-server network is the DIP (director IP).
#			(used to be called the DIIP, director inside IP.
#			This brings it into line with everyone else 
#			and removes inconsistencies in my /etc/hosts files,
#			which make it hard to setup ntp.
#			The network which the DIP and the RIPs are on 
#			is called the realserver network (used to be called the inside network).
#		7. Fixed bug which only added first persistent service (eg https, ftp)
#		   pointed out by many people.
#		8. Check contents of /proc after writing to it.
#		9. DEFAULT_GW: in previous scripts, 
#			the default gw for the real-servers (in VS-DR,VS-Tun)
#			was a router and not the director.
#			Now, priority routing is used to send packets with
#			src_addr=VIP, dst_addr=0/0 to the same machine,
#			which is now called DIRECTOR_GW or SERVER_GW.
#			All other packets are sent to the director's DIP,
#			which is now the default gw.
#			So there are now two gateways, one for packets from LVS services on the VIP,
#			and another, from the RIP to the RIP network.
#			There is no default gw anymore.
#		10. bug in grep'ing for the VIP. 
#			In the output of netstat and ifconfig I grep for $VIP 
#			rather than for "$VIP ", and so if the VIP is x.x.x.2 
#			I pick up x.x.x.255 as well. oo - bad bug :-(
#			Thanks to Oktay Akbal oktay.akbal@s-tec.de
#		11. handle configurations requiring the martian modification.
#	
#0.9.1,2,3	various trivial errors :-(
#		1. missing # from in front of a comment,
#		2. "ne" rather than "!="	;-\
#		3. replace /bin/echo with echo that's built into bash (thanks to Ratz)
#		4. replace let FOO=$FOO + 1 with ((FOO++)) (thanks to Ratz)
#		5. the remove_old_vip code triggers when it shouldn't. 
#                    Thanks to Matthew Hollander, 11 Jan 2002. (hmm, still not resolved)
#		6. new version of read_conf_file from Jon.Molin@resfeber.se Feb 2002, counts errors
#		7. this was not released.
#
#0.9.4		1. fix bug where only first realserver offering a persistent service is added.
#		(prompted by a posting by Ryan Clark clark.ryan (at) marketingtips (dot) com).
#		2. added new schedulers (lblc,sh,dh) to @schedulers
#
#Caveat - not all inputs have been tested. 
#----------------------------------------------------
#
#Note to myself: before shipping, 
#set rc_xxx_home to ./
#set poke_holes to n
#set filters_on to n
#set configure_version
#set configure_date
#set router_forwards to ""
#
#----------------------------------------------------
#
#Send bug reports to Joe jmack@wm7d.net 
#or to the lvs mailing list lvs-users@linuxvirtualserver.org
#
#----------------------------------------------------
#Description:
#
#	Generates rc.d type files for an LVS in which the 
#	director is running a Linux 2.2.x kernel.
#	(the previous version of this file, configure.lvs was for a 
#	director running a Linux 2.0.36 kernel, and was a shell script)
#
#	Input - a conf file which has
#		1. The type of LVS (VS_NAT, VS_TUN or VS_DR)
#		2. The IPs for the director and the servers
#		3. The services being served 
#
#	Outputs
#		1. rc.lvs_xxx file, which you copy (or nfs mount) to /etc/rc.d on all machines 
#		2. mon_xxx.cf file, which (if you are using mon) you copy to /etc/mon on the director 
#			where xxx is nat, tun or dr
#
#	For information about monitors see the LVS-HOWTO 
#		at http://www.linuxvirtualserver.org
#	and the 
#		mon site http://www.kernel.org/software/mon/ 
#	The fping monitor will be chosen if there is no monitor for your service.
#	Available monitors:
#		dialin, dns, fping, freespace, ftp, ftp_kernelorg, hpnp, http, http_t, 
#		imap, ldap, msql-mysql, na_quota, netappfree,
#		nntp, ping, pop3, process, rd, reboot, ssh, seq, smtp, tcp, telnet, up_rtt
#	Please read the "Big Caveat" about monitors in the LVS-HOWTO 
#
#----------------------------------------------------------
=head1 NAME

B<configure_lvs.pl> - generates the files rc.lvs, mon.cf and ntpd.conf 
using information in an input conf file.

=head1 SYNOPSYS

B<./configure> [I<conf-file> [-i]]

B<./configure> is a link to (or renamed copy of) the
configure-0.9.x-lvs.pl file. 
B<configure> expects perl to be in /usr/bin (if your perl
is in /usr/local/bin, then add a link). 

I<conf-file> (e.g. lvs_dr.conf) 
is a file containing the IP/hostnames of the machines in 
the LVS and the services that are switched (forwarded by LVS). 
I<conf-file> is in the local directory. Syntax is described below. 

=head1 FOR THE REALLY IMPATIENT

=head2 Pick a forwarding method; VS-DR, VS-NAT, VS-Tun

=head2 Pick the number of networks (1 or 2) and NICs on the director (1 or 2)

Connect your machines with the appropriate network topology
and IPs.
Make sure machines can ping adjacent nodes (don't need
default gw's, the script handles that).

=head2 pick the appropriate lvs_xxx.conf.xxx file

The conf files have private networks. 192.168.1.0/24 is
for the realservers - change this to your (presumably private)
network number. The network for the director/test_client
is set to 192.168.2.0/24 for the two network case 
(or x.x.1.0/24 for the one network case). Change this to the
network of the VIP. 

Edit IPs of the machines to suit and then run the command - 

./configure lvs_xxx.conf

run the rc.lvs file produced on the director and then 
all the realservers.

If you want this to setup your LVS automatically, read on for
USER CONFIGURABLE (for "hidden" and "router_forwards") and
COMMAND LINE OPTIONS for the -i flag.

=head1 MACHINES FOR AN LVS

director: Linux box 

	supported kernels: 
	2.4.x: all LVS modes - 
	at least 2.4.2, the self installing (ie 2nd) version
	2.2.x: all LVS modes - 
	at least 2.2.14

	unsupported kernels:
	2.0.x: 

realserver(s): almost any OS (see HOWTO). 
realservers running non-Linux unices are configured by code 
contributed by Ratz <ratz@tac.ch>. 
This part of the code has not been well tested.
Any updates, queries etc, please contact Ratz, the
LVS mailing list or myself. You will have to 
configure Microsoft realservers by hand (see the HOWTO).

=head1 INSTALL 

The B<configure> tar ball can be unzipped in any directory on
the director and the output files can be run from this directory. 
(We should probably put all the LVS configure files 
in a known location like /etc/lvs, 
but we haven't made a decision on this location yet.)
You need to run B<configure> on the director 
as it detects the director's kernel version.

For development and testing I exported the files to the same directory
on the realservers (ie to realserver:/etc/lvs).
I ran the output files on the realservers from the 
realserver console and watched for failures there.

In a production environment, for security, you won't want 
nfs running on the director. 
In this case running B<./configure> with the C<-i> option, 
will copy and execute the files using scp/ssh. 
You can still (re-)excute the rc.lvs file by hand
from the console of the realservers to monitor the realserver setup.

=head1 COMMAND LINE OPTIONS FOR B<configure> 

B<./configure> [I<conf-file> [-i]]

If you use the C<-i> option, 
the output files will be copied (using scp)
to their final locations (on director and realservers)
and the files/processes needed to setup an LVS will be
executed/read/re-HUP'ed (using ssh).
Output (STDERR only) is sent to the console of the director. 

Note: PATH for ssh - scp in openssh-2.3.0p1 calls ssh at /usr/local/bin/ssh, 
whether you make with --prefix= (when files are installed in /bin) 
or the default (when files are installed at /usr/local/bin). 
Since on the remote machine (the realservers), 
$PATH is /bin and /usr/bin (by default),
and not /usr/local/bin, 
then there must either be a link on the realservers 
from /bin/scp to /usr/local/bin/scp or you should
run

./configure --with-default-path=/bin:/usr/bin:/usr/local/bin

Other things you can do to make your life easier with ssh.

on the client end of the login (here the director, which
will be loging in to the realservers to copy files), run ssh-keygen
to create the local public and private keys. Copy the local public
key (.ssh/indentity.pub) to the file .ssh/authorized_keys on the realserver.

	#If you want to copy files without a password,
	#then you'll need this line in sshd_config
	PermitEmptyPasswords yes
	# For this to work you will also need host keys in /etc/ssh_known_hosts
	RhostsAuthentication no
	RhostsRSAAuthentication yes
	# To disable tunneled clear text passwords, change to no here!
	PasswordAuthentication yes
	PermitEmptyPasswords yes
	# Don't read ~/.rhosts and ~/.shosts files
	IgnoreRhosts yes

=head1 OUTPUT FILES

=head2 LOCATION

Defaults to local directory. From there you can test the files
(eg run them, copy/export them to the realservers and run them there).

If you run B<./configure> with the C<-i> option and if 
the location of the output files is the default (the local directory i.e. './') 
then the files will be copied to /root on the realservers. 
You probably don't want this. 
The location of the output files can be changed by editing 
$rc_lvs_home, $rc_mon_home, $rc_ntp_home (in this file).
Likely alternate locations for these directories are commented in the code.

=head2 rc.lvs

B<rc.lvs> is an executable which configures the LVS. 
It is first run on the on the director 
(B<./rc.lvs>)
and then the realservers (it knows whether it's running
on a director or a realserver).

Options for B<rc.lvs>

B<rc.lvs> [start|stop|delete|*]

	start|*: installs lvs (run on director and then realservers)
	stop: (not implemented yet)
		for director only, has no effect on realserver. 
		Changes weights of connections to 0. 
		This stops new connections, maintains current connections. 
		When current connections are dropped you can './rc.lvs delete', 
		or shutdown the nodes.
	delete: deletes ipvsadm entries aborting all connections.
		for director only, has no effect on realserver. 

If CLEAR_IPVS_TABLES=Y (in the I<conf_file>), 
then old ipvsadm settings are cleared before adding new ones.
In this case B<rc.lvs> is idempotent, i.e. you can run it multiple times
and the machine will have the same (correct) state after each run.
I've now added other state information (e.g. iproute2) and it's not easy
to reversibly add and remove this information. The result is that I have
to clear all these tables and start again each time, so your only choice
is Y here.

=head2 mon.cf 

Mon expects its conf file to be /etc/mon/mon.cf. 
The I<mon.cf> file produced by B<rc.lvs> should be a link to, 
or renamed version of /etc/mon/mon.cf.

I<mon.cf> configures monitors and alerts to check the state of services being 
offered by the LVS realservers, and is used to add/remove services to the LVS.
Mon is not needed to setup a working LVS for demonstration purposes. 
It is only used if you are using mon to monitor 
the state of the LVS for failover (see the HOWTO).
If you're unsure what mon is for, you can ignore it.

(Note: It seems that few people are using mon. 
v0.38 has problems and I haven't received replies from mon's author.
I'm using mon-0.38.21 which works fine.)

You can add your own monitor to this script. If you have a service "foo"
on port 1234, add an entry to /etc/services and add the string "foo" to 
"my @monitors".

=head2 ntp.conf

Two files are produced. One for the director (ntpd.conf.director)
and the other for the realservers (ntpd.conf.realserver). 
These will get a link from ntp.conf.
ntpd will be restarted (if it is running) 
on each machine so as to re-read the ntpd.conf file.
(ntp cannot be re-HUP'ed). 

=head1 SOFTWARE REQUIREMENTS

The configure script has been tested with perl-5.6.0.
(I expect it will work with earlier versions, but haven't tested it).
You will need the perl modules, Net::DNS, Socket.

The configure script needs a version of ping that will only
send one packet (ping -c 1). All the newer pings do this.
The script will test your ping to see if it works. 
Otherwise the script will look for fping. 
B<fping> is part of the mon package. 

Get B<fping> from

	ftp://ftp.kernel.org/pub/software/admin/mon
	http://www.kernel.org/software/mon/

You will need one of these versions of ping or B<configure> will exit.

If multiscan (http://sourceforge.net/multiscan) is on the director,
it will port scan the realservers after setup when using the C<-i> option. 
For security reasons, the only ports active on the realservers 
should be those needed for the lvs services.

=head1 NON-ENGLISH LANGUAGE UTILITIES/MACHINE READABLE OUTPUT

The script(s) parse output from netstat, ifconfig, traceroute...
in order to setup the LVS.
These executable do not produce machine readable output
(ie the format and placement of the information in the output, is not
a part of the spec of these executables). 
It seems that the script(s) are failing for people running foreign 
language versions of these executables. 

I have long been a proponent of computer output being
computer readable, but I've not had much success
(see my efforts in ham radio 
http://www.wm7d.net/az_proj/az_html/azproj_help.shtml#machine_readable_lists).
Expecting a shell script to correctly the output of ifconfig
in all languages is more than I'm prepared to handle. 
If your ifconfig produces output in a different
format than my US english version, then
you can modify my script, use the US english version
of the executables, or talk to the person who
coded up your version of the executable.

=head1 FILES PROVIDED

You get the following files 

B<configure_lvs.pl>

lvs_*.conf - sample conf files to feed to configure_lvs.pl

rc.lvs_* - sample output files produced from the conf files.

=head1 LVS VERSIONS

The code ran originally on 2.0.x directors and has been
upgraded continually since then. Presumably it should still
work on older kernels, but has not been tested on them. 
I imagine bit-rot will make it unlikely that this 
configure script will work on LVS for old kernels (<2.2.14).

Note: 2.4.x kernels are production for VS-DR and VS-Tun
but developement for VS-NAT (Jan 2000). Look on the mailing
list and docs with the patches for the latest news on 2.4.x
kernels.

=head1 DESCRIPTION

B<configure_lvs> generates rc.lvs which 

=head2 sets LVS type 

VS-NAT, VS-DR, VS-TUN

=head2 sets IPs and network on director 

including VIP. Director can accept packets on VIP
by a network device or aliase (eg eth0) or by transparent proxy (TP). 
Sets gateway for director (VS-NAT) and realservers (VS-DR, VS-Tun).


When the director is accepting packets by transparent proxy (TP),
the director does not have an IP for the VIP on a network device
and hence there will be no entry in the router/test client's arp
table for the VIP. An attempt to connect to an LVS'ed service
on the VIP will get the error.

	client:/director/src/ipchains/arch# telnet lvs
	Trying 192.168.1.110...
	telnet: Unable to connect to remote host: No route to host

You must install a route on the router/test client telling it how to get to the VIP.
You can put a host route to an IP already on the director, eg 

	route add -host 192.168.1.110 gw director 

	or
 
	put a (permanent) arp entry (arp -f /etc/ethers or arp -s) 
	pointing to a MAC address on the director.

If the realservers are accepting packets by TP, the director
already knows to route packets to the VIP (on the realservers)
through the VIP. 

=head2 sets up services 

using ipvsadm on director 

=head2 sets IPs and network on realserver(s) 

including non-arp'ing VIP (realservers accept packets on VIP
after being forwarded from the director).
Sets gateway.

=head2 tests setup

as far as is possible from inside the LVS. Many common
problems can be trapped here.

B<./configure> generates I<mon.cf> which 

=head2 sets up mon

mon will be restarted on the director (if it's running).

Failure of a service on a realserver
will be detected and this service will be removed from the LVS
list of offered services, till the service comes back on line.

The LVS can be started in the off state (INITIAL_STATE="OFF" in the
I<conf-file>) i.e. with services not being forwarded.
As the realservers bootup, 
mon will detect the services on each realserver when they come online and
will run ipvsadm to put the appropriate entries into ipvs on the director.

B<./configure> generates an ntp.conf file for the director 
and realservers which 

=head2 sets up ntp conf files 

These files can be copied/linked to /etc/ntp.conf
on all machines (done for you with the C<-i> option). 
Tested with ntp-4.0.99g ntp code from http://www.ntp.org

The files configure the director as a stratum 3 time server and the
realservers as stratum 4 time servers. Machines peer with each other
and realservers being a lower priority stratum will defer to the
director eventually becoming synchronised with the
directors CMOS clock. To synchronise the director with an external
ntp server, look up the docs on the ntp site for a public time server
near you on the net and uncomment the line in the director's ntp.conf
(it has a commented out public ntp server in North America).

The machines will take a long time to synchronise if they are offset by
more than a minute in time (this is part of the ntp protocol).
You should set the hardware clocks to within a minute (in the BIOS setup
for your machines) to prevent this.

ntpd will be killed and restarted on director and realservers 
(if it's running). (ntpd can't be re-HUP'ed).

=head1 WHICH IS THE DIRECTOR?

answer: For the moment the director is the one with the 
director_ip (DIP), the IP on the director facing the
realservers. 

reason: Although the network connectivity is different 
for the director and the realservers in an LVS, it is 
(conceptually at least) possible to promote a realserver 
to a director following a director failure. All the 
machines in an LVS then should be able to be configured 
as peers and have one machine promoted to director.

In this case, the rc.lvs has to be able to tell 
whether it's running on a director. The Linux-HA setup of LVS 
(eg ultramonkey at http://ultramonkey.sourceforge.net) 
puts flags into files in /etc/rc.d which 
results in a floating or heatbeat IP being 
assigned to the director. (This IP becomes 
the gateway for LVS packets for the realservers when 
running under VS-NAT.) 

B<configure_lvs> will read these flags 
eventually but for the moment B<configure_lvs> 
determines whether it is running on a director
by looking for the I<director_IP> (DIP). 
This DIP is set in lvs.conf and is on the 
LVS network (ie the network the realservers connect to).
(Depending on your setup, this IP may or 
may not be accessable from the internet.)

Before you run B<configure_lvs>, you must 
add this IP to the machine you want to be 
the director (on say an aliased ethernet device).

=head1 RUNNING B<rc.lvs>

If B<./configure> is run with the C<-i> parameter, B<rc.lvs> will
be executed for you on the director and then the realservers.
Only STDERR will be output to the console of the director.

You can manually run B<rc.lvs> in any directory on the director or the realservers
(it knows whether it's running on a director or realserver and
acts appropriately). 

B<rc.lvs> is verbose on STDOUT. 
Errors and Warnings are sent to STDERR.
If you only want to see the STDERR messages, you can run B<rc.lvs> with

B<./rc.lvs> >/dev/null

=head1 CHANGING VIP BETWEEN ETH DEVICES ON DIRECTOR

This section is only if

1. you have multiple ethernet devices (eth0, eth1...)
on the director

	and
	
2. you move the VIP (say 192.168.1.110) from one eth device 
(say eth0 or eth0:110, where it was from a previous installation) 
to another (say eth1 or eth1:110), when you configure a new LVS.

	and

3. You don't have iproute2 tools.
 
If this doesn't apply to you, then you can skip the rest of this section.

The script will issue a warning telling you that you can't
do this and will write a file I<rc.remove_vip> which you should execute to handle
the situation. The file I<rc.remove_vip> will require some editing to specify 

	1. the module for the ethernet devices in the LVS (eg tulip) (module for eth0, eth1) 

	2. all the ethernet devices which use this module (eg for a quad card eth0 eth1 eth2 eth3). 

	3. the command to bring up the network (e.g. /etc/rc.d/rc.inet1).

	or

You can arrange for I<rc.remove_vip> to have the 
correct contents by editing the B<configure> script in the 
subroutine I<write_remove_vip>.

e.g. director has a tulip card for eth0, eth1 and eepro for eth2.
eth0, eth1 are used in the LVS. eth2 is used for administration
of the director/LVS. The tulip module is removed by the command 

director:/etc/lvs# rmmod tulip

and the network is started with 

director:/etc/lvs# /etc/rc.d/rc.inet1

Then edit B<configure> (permanently fix), or I<rc.remove_vip> (one-off fix) 
to change the following variables. 

ETH_DEVICES="eth0 eth1"
ETH_DEVICE_MODULES="tulip"
START_NETWORK_COMMAND="/etc/rc.d/rc.inet1"

and then run I<rc.remove_vip>

director:/etc/lvs# sh ./rc.remove_vip

(Note: if the devices eth0, eth1 are on a quad card and eth2 and eth3
have an IP on them, then all four eth devices will have to be listed
in DIRECTOR_LVS_ETH_DEVICES.)

Here's an explanation. 

Before you can reconfigure the LVS, the VIP on the 
current device (usually aliased) must be removed (not just down'ed).
Down'ing an aliased device doesn't remove it.
If you were to down the aliased device (eg eth0:110), 
then when any IP on the device (eth0) is up'ed, 
then the aliased device (eth0:110) will also be up'ed again.
(Unlike Solaris you can't unplumb eth devices.
After downing eth0:110, eth0 would still be in the ifconfig table
(either marked down, or not marked UP), 
and not removed as you might have expected.
The network connections to eth0 (and its aliases)
would be gone from the routing table and 
your eth0 network will be hosed.
If you were running this script from a remote machine 
via eth0, then your connection would be lost
(possibly permanently if something went wrong)
and you'd have to come in from home and handle it from
the console, which you might not want to do at 3am."

The configure script will exit when encountering this 
situation, to prevent loosing editing sessions and possibly this connection.
You should re-execute B<configure> after removing the eth0 entry
from the ifconfig table (not just downing it).
You can either reboot :-(, or unload the driver for eth0.

To unload the module without dropping connections,

1. make sure connections/sessions to the director are idle. 
(You'll loose the session otherwise - a telnet session 
 will survive the link going down for a few minutes, 
if you don't type on the keyboard in that 
window too much, while the connection is down.) 

2. At the console, unload and reinstall the modules. 
You can do this with the I<rc.remove_vip> script
which will be written to the local directory if
needed. 

	-or-

run a set of commands like the following -
(You can swipe this with a mouse and run it from the command line.)

	# show current network. 
	netstat -rn
	# bring down all eth devices. 
	for device in `ifconfig -a | grep eth | awk '{print \$1}'| grep -v : | \$XARGS `; do ifconfig \"\$device\" down; done
	# remove ethernet module(s). 
	rmmod tulip
	#re-install default network. 
	. /etc/rc.d/rc.inet1
	# show default network (all VIPs should be removed). 
	netstat -rn

If you remove eth0 successfully, 
you will not see a notice next time you run B<configure>.

Other thoughts:

One of the consequences of unloading the NIC module is that
if you are using the same type of ethernet card for all your 
ethernet devices, then all network connections will come down. 
One solution, that will also help with security, is to have a 
separate network/NIC for administering the director.
This admin NIC should be chosen so that it will have 
a different driver module than will the NICs used for the LVS.
e.g. the LVS could use NICs with tulip drivers (one for the VIP, 
and another for the DIP), while the admin NIC is an eepro100.
Connection to the director would be by ssh and the 
sshd would only listen to the IP of the admin NIC.

You can add a list of driver modules in USER CONFIGURABLE below.
These should include all modules on both the director and realservers.
It doesn't matter if you list modules not on a host, they will
be ignored.

If your network driver is NOT a module, you will have to re-boot
the computer.

=head1 USER CONFIGURABLE variables

B<$hidden>, B<$router_forwards> and B<network_driver_modules>.

These are at the top of the perl code and have default values that
will setup an working LVS. 

B<$hidden> Y|N default Y

	Sets whether to hide the VIP device on the realserver
from arp requests (see the "arp problem" in the HOWTO).

B<$router_forwards>

	Look for I<Warning:> output when running rc.lvs. This
will tell you what to do with this variable (if anything).

	If you haven't set $router_forwards, we don't know if the VIP 
should be pingable or not from the realserver. My local
router/lvs gateway/test client is a linux-2.2.x box. By default
this does not forward between NICs and so the realserver in
a 2 network VS-DR LVS won't be able to ping the VIP on the 
director. If ip_forwarding was on in this box, then the 
VIP on the director would be pingable from the realserver.
If the router was a regular router, it might forward by
default. The LVS will work properly no matter whether
the router forwards or not. 

This ping test is just extra checking. If this test is 
properly setup, it will help diagnose problems with your LVS.

B<persistent_services>

Currently set to 

	ftp,https

If these services are found in the
SERVICE line, connections to that realserver will be
set to persistent (see PERSISTENT CONNECTIONS). 
ftp is a special case in that a helper module
(ip_masq_ftp.o) is used for LVS-NAT (for LVS-DR or
LVS-Tun ftp is set persistent). ftp is handled automatically
by the script. The SERVICE lines in the lvs.conf file
are the same for persistent and for non-persisten services.
The persistence timeout is set with PERSISTENCE_TIMEOUT.

B<network_driver_modules>

Should include all network drivers in all nodes on the LVS.
The script will have to unload the modules if changing
the VIP from one interface to another.

=head1 PERSISTENT CONNECTIONS and LINKED SERVICES

Some services need to be set up with persistence.
This is handled automatically for you. This section
explains how it works. You can skip this section
if you like.

Most services (eg telnet, http) require only a single port
(23,80 respectively) and ipvsadm can forward the connection
to any realserver. No state is developed/required/maintained
as a result of the connection and the client, 
at the conclusion of one connection (session) 
can be re-assigned to a different realserver for
another connection without any effect. 
These service do not need persistence. 

=head2 SERVICES requiring PERSISTENCE

Some services, eg https require persistence even
if not linked to other services eg http. This is
so that the client's session will always go to the 
same realserver where keys are cached.

=head2 EXAMPLES

1.
SERVICE=t telnet rr realserver1 realserver2
	client will rotate between the two realservers

2.
SERVICE=t ftp rr realserver1 realserver2
	client will connect to one realserver. 
	All subsequent connections for any service
	will go to the same realserver.

3.
SERVICE=t ftp rr realserver1 realserver2
SERVICE=t telnet rr realserver1 realserver2
SERVICE=t http rr realserver1 realserver2
	client will connect to one realserver. 
	All subsequent connections for any service
	(except telnet, http) will go to the same realserver.
	Connections for telnet and http will rotate between
	the two realservers
	
4.
SERVICE=t ftp rr realserver1 realserver2
SERVICE=t telnet rr realserver1 realserver2
SERVICE=t http rr realserver1 realserver2
SERVICE=t https rr realserver1 realserver2
	client will connect to one realserver. 
	All subsequent connections for any service
	(except telnet) will go to the same realserver.
	http, https are linked and http will not
	be configured by ipvsadm. Client will connect
	to http through the persistence of all ports.
	http service will however still be monitored by
	the monitoring service. Connections for telnet 
	will rotate between the two realservers.
	There will be no entry seen for http 
	in the output of the ipvsadm command.

Don't do this.	
5.
SERVICE=t http rr realserver1 realserver2 realserver3 realserver4
SERVICE=t https rr realserver1 realserver2
	connections to http will go to 1 of the 4
	realservers. Only if you connect to the
	first 2 will you then be able to maintain
	session data when hopping to https.
	If realserver3,4 are unrelated websites,
	not connected to the e-commerce https site,
	then you should put them on a different VIP.
	
=head1 DEBUGGING PROBLEMS

Expected problems will give warnings or errors and hints
as to the problem.

Unexpected problems running rc.lvs_* files will give 
error messages from shell commands in rc.lvs*. To
find which command is causing the problem run

sh -x ./rc.lvs 2>&1 | more

and look for the line causing the error. You'll have
to track down the line in the rc.lvs file and run it 
by hand. Hopefully this will give you an idea as to 
the problem.

=head1 EXAMPLE FILES FOR THE IMPATIENT

1. 
The file B<lvs_dr.conf.simple_telnet> will setup a one realserver 
LVS. You need a client with telnet (any OS), a director (Linux) and
a realserver (any OS but for a test Linux will be faster to setup)
all on the same network. Edit B<lvs_dr.conf.simple_telnet> changing
the names of the machines to your setup. Make the client the gateway for 
LVS packets from the director (VS-NAT) and the realservers (VS-DR, VS-Tun).

Alternately, change telnet to http to connect from the browser on the
client to an httpd on the realserver.

2. 
The file B<lvs_nat.conf.simple_telnet> will setup a one realserver 
LVS as above using VS-NAT. The network for the realserver must be private
(eg 192.168.1.0/24) and the realserver must not be able to route to the
client.  

=head1 /etc/hosts

ipvsadm can use hostnames or IPs. 
The example lvs.conf files use generic names (eg RS1 for
realserver_1) allowing you to use the same I<conf-file> almost
anywhere with minimum modification, 
provided that you add the appropriate aliases to the 
/etc/hosts file. (You can substitute the IPs in place of the
names if you like.)

If IPs
are resolvable (eg you have an /etc/hosts file on the LVS machines),
then hostnames will be output to the rc.lvs file.

If you want to use the lvs.conf file in this manner, add 
the following aliases to your /etc/hosts file on all machines 
in the LVS: VIP, DIP, RS1, RS2..RSn, client.
(You may not need all of these names). 

Here's part of my /etc/hosts. 

192.168.1.9	dip.mack.net 		dip
192.168.1.11	sneezy.mack.net 	sneezy		RS1
192.168.1.12	bashfull.mack.net	bashfull	RS2
192.168.1.13	dopey.mack.net		dopey		RS3
192.168.1.14	doc.mack.net		doc		RS4

#when setup to test a one network LVS
192.168.1.110	lvs.mack.net		lvs		vip1
192.168.1.254	client.mack.net		client

#when setup to test a two network LVS
192.168.2.110	lvs2.mack.net		lvs2		vip2
192.168.2.254	client2.mack.net	client2

=head1 /etc/services

The I<conf-file> can use services as a name (eg telnet) or port number (eg 23).
The file /etc/services will be used to translate one to another. 
If the /etc/services file has a port->name translation, then
the name of the service rather than the number
will be written to the output files.

Likely additions needed to your /etc/services will be

	ftp-data        20/tcp
	ssh             22/tcp
	domain          53/tcp  nameserver      dns #add "dns" here
	domain          53/udp  nameserver      dns #and here
	shell           514/tcp cmd             rsh #add rsh here
	https           443/tcp
	https           443/udp			    #hmm, does https use udp?
	printer         515/tcp spooler         lpd #add "lpd" here
	nfs             2049/udp        nfs
	mysql           3306/tcp
	mysql           3306/udp
	netpipe         5002/tcp

=head1 CLASSLESS NETWORKS

All networks/netmasks are set explicitely in the I<conf-file>, 
so presumably you can set networks on any boundary. 
I have only tested this with a class C setup. 

=head1 DESCRIPTION OF CONF FILE

=head2 LVSCONF_FORMAT

The I<conf-file> described here has B<LVSCONF_FORMAT=1.1> .
Presumably the VERSION for B<./configure> and 
LVSCONF_FORMAT for I<conf-file> will change somewhat independantly.

=head2 CONTENTS OF CONF FILES

The I<conf-file> consists of 

	comments (any part of a line starting at '#'),
 	blank lines. 
	variable names with arguements, 

Hostnames can be a resolvable name or a dotted quad IP. 
Resolvable IPs will be output as hostnames in the output files.
The variables can be in any order.
For my sanity I keep them in the following sections and order

	lvs type
	director setup
	services setup
	realserver setup

Devices for all IPs are explicitely set (eg eth0, eth1, lo:0, dummy0). 
This allows the script to work with directors 
and realservers which have any number of NICs and any OS.

Here are the variables required, with explanation, examples and a working value.

	#-----------------
	# conf file format
	# hopefully will prevent grief in the future
LVSCONF_FORMAT=1.1

	#-----------------
	#lvs type
	#

	#LVS_TYPE: VS_TUN|VS_NAT|VS_DR 
	#(string is matched with tun/nat/dr, 
	# any string containing tun, nat or dr will do. eg tunnel, natural, drive)
LVS_TYPE=VS_DR

	#-----------------
	#initial state
	#

	#INITIAL_STATE: ON|OFF (case insensitive)  
	#Only useful if you are using mon 
	#(or some tool to determine the state of servers and services)
	#otherwise ignore (leave "on" or ommit, or comment out)
	#
	#suggested: for testing - "on"
	#           for production systems - "off", 
	#if you don't understand the following then choose "on"
	#
	#when testing an LVS, you normally have the services 
	#(eg http, telnet) already running on the real servers, 
	#before you run ipvsadm (to add servers and services)
	#on the director, and before you make a request from the client. 
	#
	#In a production system, if ipvsadm adds servers:services to
	#the director as soon as the director boots, the servers 
	#could still be booting/fsck'ing. 
	#A client could be routed to a server which is still coming up.
	#If you are running mon (and mon has been tested by pulling 
	#and reinserting cables etc), you can handle this by configuring
	#LVS with no entries (or entries with weight = 0).
	#When the director comes on line, the ipvsadm table will be empty.
	#Mon will also be running and as each server comes on line, 
	#mon will detect an on-line server and add entries to the LVS
	#table by running ipvsadm, bringing the server into the LVS.
	#
	#If you have INITIAL_STATE="OFF", then the configure script
	#will setup the LVS with no entries, and allow mon to add the
	#entries as the services come up on the realservers.
	#
	#If you choose "off" here, mon must be (re)started after 
	#running the rc.lvs scripts. This is incase the realservers
	#are already up. Mon upalerts only occur on a transition 
	#of the services from down to up. If mon is already running 
	#and the realservers are offering services and the rc.lvs 
	#script sets up an LVS with services off, then the LVS will
	#just sit there in the off state. You have to re-HUP
	#mon (to get it to re-read its config file and test the services).
	#
	#I've only got mon-3.71 to work. The 3.8 series have problems for
	#me and Jim Trocki, the author of mon doesn't reply to e-mails.
	#
INITIAL_STATE=on  

	#-----------------
	#clear previous LVS entries
	#

	#CLEAR_IPVS_TABLES: YES|Y|NO|N (case insensitive)
	#yes; clear any current ipvs tables by running ipvsadm -C.
	#(A running LVS will be cleared.)
	#
	#no; leave current ipvsadm commands/tables untouched.
	#this will allow you to run several rc.lvs_* files in turn
	#which will allow you to have an LVS with several IPs.
	#If you do this you must make sure that subsequent commands
	#do not override previous entries in the ipvs tables.
	#
	#If you don't clear the ipvsadm tables, you can generate 
	#two rc.lvs files, the first which clears the tables, and
	#the second which doesn't and runs extra ipvsadm commands.
	#To test, run each file separately with CLEAR_IPVS_TABLES=Y
	#to make sure that each LVS works separately, before you
	#run them together.
	#
CLEAR_IPVS_TABLES=yes

	#-----------------
	#director setup 
	#

	#Virtual IP (VIP)
	#
	#The LVS has its own IP (and name in DNS, presumably that of 
	#the LVS, eg lvs.foobar.org).
	#This IP is called the VIP and is on a regular 
	#(or ip_aliased) ethernet device on the director.
	#Packets sent to the VIP are forwarded on to the 
	#servers according to the ipvsadm table.
	#Clients must be able to route to/ping/connect with the VIP.
	#
	#The director will need a separate IP/machine name for #logging in/admin. 
	#For security this should be on a different NIC/IP to the VIP. 
	#(eg if the LVS forwards telnet packets, you will not be able
	#to telnet to the VIP on the director). 
	#
	#For testing I use VIP=x.x.x.110 (where x.x.x.0 is your domain).
	#
	#After setup, clients must be about to route/ping/connect with this VIP.
	#
	#syntax: 
	#Note the host mask - 255.255.255.255
	#and the broadcast address - it's the IP, it is NOT x.x.x.255
	#The LVS can accept packets by a regular network device (eg eth0)
	#or by transparent proxy (see HOWTO)
	#I use an ethernet alias with the same number 
	#as the last number in the IP (here 110)
	#to reduce the chance of name space collisions
	#
	#VirtualIP: device, IP (or name), netmask, broadcast 
	#VIP=eth1:110 192.168.2.110 255.255.255.255 192.168.2.110
	#VIP=eth0:110 192.168.1.110 255.255.255.255 192.168.1.110
	#VIP=eth0:110 lvs 255.255.255.255 192.168.1.110
	#the director can accept packets by TP (tested for VS-DR so far)
	#VIP=TP 192.168.1.110 255.255.255.255 192.168.1.110
VIP=eth1:110 lvs 255.255.255.255 lvs

	#DIP
	#DIP: device IP(or name) network netmask broadcast
	#DIP=eth0:9 192.168.1.9 192.168.1.0 255.255.255.0 192.168.1.255
	#DIP=eth0:9 director 192.168.1.0 255.255.255.0 192.168.1.255
DIP=eth0:9 192.168.1.9 192.168.1.0 255.255.255.0 192.168.1.255

	# DIRECTOR_GW: hostname|ip
DIRECTOR_GW=192.168.2.254

	#-----------------
	#services setup
	#

	#SERVICE:
	#syntax:
	#service proto (t|u) port service scheduling_algorithm (wlc|rr|wrr|pcc) server_IP[,weight (default 1)] [server IP[,weight]]
	#
	#For TUN, the realserver network can be anything, including the client's network.
	#For DR, the VIP must be part of the realserver network.
	#
	#scheduling algorithms 
	#wlc: weighted least connection
	#rr: round robin
	#wrr: weighted round robin
	#pcc: persistent client connection
	#
	#
	#telnet is LVS'ed using round robin scheduling to realserver x.x.x.10..15 and by localnode to the director
	#SERVICE=t telnet rr 192.168.1.10 192.168.1.11 192.168.1.12 192.168.1.13 192.168.1.14 192.168.1.15 127.0.0.1 
	#
	#dns requires 2 entries, one for udp and another for tcp
	#realserver x.x.x.12 gets twice the number of requests of realserver x.x.x.11
	#SERVICE=t dns rr 192.168.1.11,1 192.168.1.12,2  
	#SERVICE=u dns rr 192.168.1.11,1 192.168.1.12,2  
	#
SERVICE=t telnet rr RS1 

SERVICE=t telnet rr RS2 

SERVICE=t netpipe rr 192.168.1.11 

SERVICE=u nfs rr 192.168.1.11 

SERVICE=t rsh rr 192.168.1.11 


	#-----------------
	#realserver setup 
	#
	#the realserver IP's are on eth devices (or aliases) 
	#and must be accessable (ping etc) from the director.
	#realservers can accept packets by transparent proxy, 
	#in which case there will be no ethernet device for the RIP
	#and the RIP won't be pingable.
	#
	#
	#SERVER_VIP_DEVICE: TP|lo:x|tunl0:x|dummy0:x|eth0:1|eth1|...
	#TP on realservers is for VS-DR and VS-NAT only
	#lo:0 for VS-DR, tunl0:0 for VS-TUN
	#there is no VIP on the realservers for VS-NAT.
	#This input will be ignored for VS-NAT 
	#(you could also comment it out, or leave it out for VS-NAT) 
	#SERVER_VIP_DEVICE=TP
	#SERVER_VIP_DEVICE=tunl0
	#SERVER_VIP_DEVICE=dummy0
	#SERVER_VIP_DEVICE=eth0:1
SERVER_VIP_DEVICE=lo:0

	#SERVER_NET_DEVICE: eth0, eth1, hme0, le0:1
	#network device on the real servers (the device that has the IP's in SERVICE)
	#this device will have been setup by rc.inet1 before configuring LVS 
	#since all servers for convenience are configured identically, 
	#this will be the same for all servers
	#on linux eth0, eth1, eth0:1..., on Solaris le0, le0:1, hme0
	#SERVER_NET_DEVICE=hme0
	#SERVER_NET_DEVICE=le0:1
SERVER_NET_DEVICE=eth0

	#SERVER_GW: name|ip
	#for vs-nat, must be on director
	#for vs_nat, must NOT be on director
	#for vs_tun, must NOT be on director,
	#but can be different for each realserver 
	#(B<configure> makes them all the same)
	#to set the gateway for packets with src_addr=VIP
SERVER_GW=192.168.1.254 	

#----------end lvs_dr.conf-------------

=head1 EXAMPLE I<conf-files>

lvs_dr.conf

	LVS_TYPE=VS_DR
	INITIAL_STATE=ON 
	CLEAR_IPVS_TABLES=y
	
	VIP=eth1:110 192.168.1.110 255.255.255.255 192.168.1.110
	DIP=eth0 director 192.168.1.0 255.255.255.0 192.168.1.255 
	DIRECTOR_GW=client2
	SERVICE=t telnet rr 192.168.1.11 
	SERVICE=t telnet rr RS2
	SERVICE=t netpipe rr 192.168.1.11 
	SERVICE=u nfs rr 192.168.1.11 
	SERVICE=t rsh rr RS1 
	
	SERVER_VIP_DEVICE=lo:0
	SERVER_NET_DEVICE=eth0
	SERVER_GW=client


lvs_nat.conf

	LVS_TYPE=VS_NAT
	INITIAL_STATE=ON  
	VIP=eth1:110 lvs2 255.255.255.0 192.168.2.255
	DIP=eth0 director 192.168.1.0 255.255.255.0 192.168.1.255
	DIRECTOR_GW=client2
	SERVICE=t telnet rr 192.168.1.11:telnet RS2:telnet 
	SERVICE=t rsh rr RS1:rsh 
	SERVICE=t netpipe rr 192.168.1.11:netpipe
	SERVICE=u nfs rr 192.168.1.11:nfs

	SERVER_NET_DEVICE=eth0
	SERVER_GW=director

=head1 PROBLEMS

1. configure hangs. 

It isn't hanging - It's waiting to timeout doing a lookup on a hostname in the conf file.
You don't have the hostname in /etc/hosts or DNS. 

To see the problem hostname, run

director:/etc/lvs# perl -w configure conf_file

and watch for the failure message after hostname resolution timeout.

=head1 MODULES

The setup/running of LVS requires some modules (for 2.2.x; ip_masq_ftp:
for 2.4.x; ip_tables, ip_vs_ftp).
If not already loaded, these modules are loaded by the configure script.

If these functions are in the kernel, the script will look
for them in the kernel using /System.map /boot/System.map or
/usr/src/linux/System.map as the index.

=head1 BUGS

Someone has reported a bug setting up VS-Tun. Haven't looked into it yet.

=head1 TODO

merge configure-lvs with HA scripts (eg Horms stuff).
Setup router (if it runs sh that is).

display multiscan output for services not lvs'ed (currently
displays all services on realserver).

=head1 HISTORY

B<early version>, shell script Jul 99, for 2.0.36 kernels

B<later version>, perl script  Dec 99, for 2.2.x kernels, 
generates sh script output. Action is taken as conf file 
is read in. Requires entries to be in correct order.
Information from previous entries in conf file is discarded
so you can't take action based on two of the input lines.
Works OK enough.

B<v0.6 Jun 2000>, perl script, entries in configure file 
are read in before any output is constructed. 
Entries in conf file can be in any order 
(this is not particularly useful) and (more importantly)
decisions can now be made using two (or more) of input data. 

B<v0.7 Dec 2000>
Bug fixes and make script compatible with LVS-mini-HOWTO.
add creation of ntp.conf files.

B<0.8 Apr 2001>
See top of this file under VERSIONS. 

B<0.8.1 Apr 2001>
See top of this file under VERSIONS. 

=head1 THANKS TO

discussions with Rob McCauley and Tommy Cathey at work.

=head1 LICENSE

B<configure>is distributed under the GPL. 
You should have several hundred copies of it by now. 
If not please contact
 
	Free Software Foundation, Inc.
	675 Mass Ave, Cambridge, MA 02139, USA

=head1 AUTHOR

This code is (C) 2000-2002 Joseph Mack jmack@wm7d.net
and is part of the Linux Virtual Server project.

=head1 LVS PROJECT

The Linux Virtual Server (LVS) is a project started by 
Wensong Zhang, which is writing a highly available layer 4 switch. 
All information (code, documentation, HOWTO, mailing lists) is
at http://www.linuxvirtualserver.org

=cut
#----------------------------------------------------
#
# General setup
# perl modules
use strict;
use Net::DNS;
use Socket;
$| = 1; #flush buffers

#------------------------------
# user configurable stuff
# (default values will do something sensible, 
# so don't be too concerned if you don't know what's going on yet)
my $hidden="Y"; # Y = default LVS behaviour
		# "Y"|"N" - Y - lo:0, tunl0 etc on 2.2.x real-servers _do_not_ reply to arp requests 
                #           N - devices _reply_ to arp requests (default 2.2.x behaviour)
my $router_forwards="";	#no default, you're going to have to test your setup to see what you have, 
			#or else know how the router is setup.
			#"y"|"n"|"Y"|"N"
			#in a VS-DR or VS-Tun LVS, the gateway for packets with src_addr=VIP 
			#from for the real-servers can forward (or not forward) packets 
			#between the director and the real-servers.
			#In either case the LVS will work.
			#A ping test between the real-server and the VIP 
			#on the director will only work in the forwarding case.
			#
			#This variable tells the test what result to expect.
			#With this variable null, you'll get the results of the ping test
			#without any decision as to whether this is an error or normal situation.
			#
			#If you set the variable to "y"|"N",
			#then the result of the ping test will be interpreted as a succeed/fail.
			#If the LVS works, and if the VIP is pingable, then set this variable to "Y"
			#
			#If the gateway for packets with src_addr=VIP for the real-server is a test client, 
			#a linux-2.2.x kernel with two ethernet cards 
			#(one for the real-server network, one for the director/VIP)
			#and 
			#cat /proc/sys/net/ipv4/ip_forward = 0 
			#(ie no forwarding, the 2.2.x default)
			#
			#

my @persistent_services = ("ftp","https");
			#these services will be set up as persistant
my $persistent_service; #an entry in @persistent_services

my @network_driver_modules = ("tulip", "eepro100","wd");
			# modules for all NICs in LVS. 
			# Drivers for cards not in machines are ignored.
			# i.e. you can list every driver known if you like.

my $filters_on = "n";	#y|n
			#writes|does_not_write filter code.
			#Code not finished.
			#Don't turn on or you'll lock up your machines.

my $poke_holes = "y";	#y|n, poke hole in filters.
			#In my setup, I nfs export directories to the client. 
			#These holes won't be needed in a production system, but
			#you might need holes for your setup and can adapt the code.
			#Hopefully I'll remember to turn this off in the version I make public.
			#
#------------------------------

#DIRECTORIES
#location of output files; rc.lvs, mon.cf, ntpd.conf (note trailing '/' in directory name)
#
#The default locations for mon.cf and ntp.conf are those expected by mon and ntpd respectively.
#The default location for rc.lvs is ./ - you should change this to wherever you put your inits
#eg /etc/rc.d/ or /etc/rc.d/init.d/
my $rc_lvs_home="./";
#my $rc_lvs_home="/etc/rc.d/"; #a likely default location
#my $rc_lvs_home="/etc/rc.d/init.d/"; #another likely default location

my $rc_mon_home="/etc/mon/"; #the default location for mon.cf
#my $rc_mon_home="./";
my $rc_ntp_home="/etc/"; #the default location for ntp.conf
#my $rc_ntp_home="./";

#CONF_FILE
my ($lvs_type, $connection_type, $connection_parameter);
my $initial_state;
my $clear_ipvs_tables;
my $line; #input line
my $input_errors = 0;

my $command_line_argument;

#MONITORS
my @monitors = ("dialin", "dns", "fping", "freespace", "ftp", "ftp_kernelorg", "hpnp", "http", "http_t", "imap", "ldap", "msql-mysql", "na_quota", "netappfree", "nntp", "ping", "pop3", "process", "seq", "smtp", "ssh", "rd", "reboot", "tcp", "telnet", "up_rtt");

#FILES
my ($conf_file, $rc_file, $mon_file, $director_ntp_file, $realserver_ntp_file);

my @params;

#DIRECTOR NETWORK
my ($director_vip_device, $short_director_vip_device, $vip, $vip_netmask, $vip_broadcast); #VIP setup for VS_TUN, VS_DR
my ($dip_device, $short_dip_device, $dip, $realserver_network, $realserver_netmask, $realserver_broadcast,$dot_quad, $classless, $log_2); #server network for VS_NAT
my $dip_name;

#SERVICES
my $default_scheduler = "wlc";
my ($proto, $service, $scheduler, $server, $weight, $weight_lvsadm); 
my ($service_name, $port, $protocol, $server_name, $server_ip);
my ($server_port, $server_ipport, $server_service_name);
my (@service_lines, $element, $line_reference); #the input lines containing SERVICES
						#each element of the line,
						#and the reference to the line inside @service_lines
my $number_service_lines=0;
my (@redirect_lines, @services);#for running rc.horms
my %persistence_set;	#vip, Y|N
my %persistence_found;	#vip, Y|N
my @high_ports_masqueraded; 
my %proto_service;	#holds services, proto

#SERVERS
my ($server_vip_device, $short_server_vip_device, $server_net_device, $server_counter, $monitor); 
my @servers; #list of server IPs
my @server_names; #list of server names
my $is_server_vip_device_local = "N";	#ie is it lo, tunl, dummy?
my @persistent_servers;	#if a server has even 1 persistent service on it,
			#then all services will be persistent
			#and there is no point in adding more services to that server,
			#since they're already handled.

#SERVER/SERVICE data
#hashes to contain server_name, service, weight
my %servers_and_services;	#key=server_name (eg RS1)
				#data=reference to %services_and_params
my %services_and_params;	#key=service_name (eg ftp)
				#data=reference to array containing $proto, $scheduler, $weight
my $num_tabs = 0; #for printing array/hash contents

#kernel version
my $uname_r_known = "N";
my $uname_r_item; 
my $kernel_series; #will be "2.0","2.2","2.4"
my $minor_rev;

#list director kernels that this script is thought to work on
#pre kernels are named like 2.2.15pre5. 
#this will be rev'ed down to 2.2.14.
my @uname_r_ok = ("2.2.15","2.2.16","2.2.17","2.2.18","2.2.19","2.4.0","2.4.1","2.4.2","2.4.3","2.4.4","2.4.5","2.4.6","2.4.7","2.4.8","2.4.9","2.5.10","2.4.11","2.4.12");
my $uname_r;

#GATEWAYS, set to null incase no input data for them 
#now handled in verify_inputs. 
my ($server_gw, $director_gw);
$server_gw="";
$director_gw="";

#OTHER

my $line_counter;
$line_counter=0;
$server_counter=0;

#SCHEDULERS list of known schedulers (ppc is only supported to 2.2.10)
my @schedulers = ("rr","wrr","lc","wlc","pcc");

#conf file formats 
my ($lvsconf_format, $this_format, $known_format, $unknown_format, $lvsconf_format_is_ok);
my @formats_ok = ("1.1");

#configure version
my $configure_version="0.9.4";
my $configure_date="Sep 2002";

#--------------------------------------------
#shell boilerplate
#Perl Cookbook 1998, p23
my $general_boilerplate;
($general_boilerplate = <<'BOILERPLATE') =~ s/^@//gm;
@
@hello_notice()
@	{
@	MAINTAINER="jmack@wm7d.net"
@	LVS_USERS="lvs-users@linuxvirtualserver.org"
@	#find_standard_executables 
@	echo "rc.lvs version $VERSION $DATE"
@	echo "(C) 2000-2002 Joseph Mack $MAINTAINER, distributed under GPL license"
@	echo "This file is part of the LVS project http://www.linuxvirtualserver.org"
@	echo "setting up $HOSTNAME"
@	echo ""
@	HOSTNAME=`$HOSTNAME_CMD` 
@	UNAME=`$UNAME_CMD`
@	UNAME_R=`$UNAME_CMD -r`
@	if [ $UNAME = "Linux" ]
@	then
@		LINUX_KERNEL_SERIES=${UNAME_R%.*}
@	fi  
@	}
@#
@#was rc.ratz--------------------------------------
@#
@#adds VIP on non-linux realservers operating in a VS-DR LVS
@#Info from Roberto Nibali (ratz) Ratz <ratz@tac.ch>
@#
@realserver_osversion_unknown()
@	{
@	echo "$0 error: unknown version ${UNAME_R} of OS $UNAME"  1>&2
@	echo "" 1>&2
@	echo "You may be able to configure the direct routing interface" 1>&2
@	echo "on this realserver using the known commands for " 1>&2
@	echo "$UNAME contained in this file." 1>&2
@	echo " " 1>&2
@	echo "if you are successfull, please send the output of the following commands" 1>&2
@	echo "" 1>&2
@	echo "$UNAME_CMD " 1>&2
@	echo "$UNAME_CMD -r" 1>&2
@	echo "the successful ifconfig command" 1>&2
@	echo "" 1>&2
@	echo "to $MAINTAINER or $LVS_USERS - thanks Joe" 1>&2
@	exit 1
@	}
@
@realserver_os_unknown()
@	{
@	echo "$0 error: unknown OS $UNAME" 1>&2
@	echo "" 1>&2
@	echo "cannot configure non arping lo device" 1>&2
@	echo " " 1>&2
@	echo "if you can figure out how to do this, please send the output of the following commands" 1>&2
@	echo "" 1>&2
@	echo "$UNAME_CMD" 1>&2 
@	echo "$UNAME_CMD -r" 1>&2
@	echo "the successful ifconfig command" 1>&2
@	echo "" 1>&2
@	echo "to $MAINTAINER or $LVS_USERS - thanks Joe" 1>&2
@	exit 1
@	}
@
@install_realserver_vip()
@	{
@	#if device is tunl and the ipip module isn't loaded, then ifconfig will fail.
@	#see if ipip is in kernel
@	if [ "${SHORT_SERVER_VIP_DEVICE#tunl}" != "$SHORT_SERVER_VIP_DEVICE" ]
@	then
@		#$SHORT_SERVER_VIP_DEVICE has the string "tunl" in it
@		echo "$SHORT_SERVER_VIP_DEVICE is a tunl device"
@		if [ `$GREP -c " ipip_init$" $SYSTEM_MAP` = "0" ]
@		then
@			#hopefully it's a module"
@			echo "ipip is not in kernel, see if it's a module"
@			if [ `$LSMOD | $GREP -c "ipip"` = "0" ]
@			then
@				#not loaded
@				$INSMOD ipip
@				if [ "$?" = "0" ]
@				then
@					echo "ipip module loaded OK"
@				else
@					echo "Error: cannot load ipip module"
@					((NUM_ERRORS++))
@				fi
@			elif [ `$LSMOD | $GREP -c "ipip"` = "1" ]
@			then
@				echo "ipip module already loaded"
@			else
@				NUM_IPIP=`$LSMOD | grep -c "ipip"`
@				echo "Error: expected 0 or 1 ipip modules, found $NUM_IPIP"
@				((NUM_ERRORS++))
@			fi
@		elif [ `$GREP -c " ipip_init$" $SYSTEM_MAP` = "1" ]
@		then
@			#ipip is in the kernel
@			echo "ipip is in the kernel, OK"
@		else
@			NUM_IPIP=`$GREP -c " ipip_init$" $SYSTEM_MAP`
@			echo "Error: expected 0 or 1 ipip init functions in kernel, found $NUM_IPIP"
@			((NUM_ERRORS++))
@		fi
@	else
@		echo "$SHORT_SERVER_VIP_DEVICE is not a tunl device, OK"
@	fi
@	#if the device is tunl0:0 then you need to up tunl0 
@	#or else tunl0:0 isn't up and you can't add a route to it.
@	#presumably dummy is the same, but I haven't tested it
@	if [ "$IS_SERVER_VIP_DEVICE_LOCAL" = "Y" ]
@	then
@		echo "$SERVER_VIP_DEVICE is local, up'ing $SHORT_SERVER_VIP_DEVICE."
@		$IFCONFIG $SHORT_SERVER_VIP_DEVICE up
@	fi
@	echo "install_realserver_vip: configuring $UNAME $UNAME_R "
@	$IFCONFIG $DEV $ALIAS $VIP broadcast $VIP_BROADCAST netmask $NETMASK $ARP_STATUS up 
@	echo "ifconfig output "
@	#$IFCONFIG -a
@	$IFCONFIG $DEV
@	echo "installing route for VIP $VIP on device $DEV"
@	$ROUTE add -host $VIP dev $DEV
@	echo "listing routing info for VIP $VIP "
@	$NETSTAT -rn | $GREP "$VIP "
@	}
@
@hide_all()
@	{
@	#
@	HIDE_ERRORS=0
@	if [ $HIDDEN = "Y" -a $IS_SERVER_VIP_DEVICE_LOCAL = "Y" ]
@	then
@		echo "hiding interface $DEV, will not arp"
@		echo 1 > /proc/sys/net/ipv4/conf/all/hidden
@		#check in case don't have hidden patch applied 
@		#(for 2.2 is in distribution now, will only be a problem for 2.4)
@		if [ $? = 1 ]
@		then 
@			echo "Error: hide all failed" 1>&2
@			((NUM_ERRORS++))
@			let HIDE_ERRORS=$HIDE_ERRORS+1
@		fi
@		echo 1 > /proc/sys/net/ipv4/conf/${SHORT_DEV}/hidden
@		if [ $? = 1 ]
@		then 
@			echo "Error: hide $SHORT_DEV failed" 1>&2
@			((NUM_ERRORS++))
@			let HIDE_ERRORS=$HIDE_ERRORS+1
@		fi
@	else
@		echo "un-hiding interface $DEV, will arp"
@		echo 0 > /proc/sys/net/ipv4/conf/all/hidden
@		if [ $? = 1 ]
@		then 
@			echo "Error: unhide all failed" 1>&2
@			((NUM_ERRORS++))
@			let HIDE_ERRORS=$HIDE_ERRORS+1
@		fi
@		echo 0 > /proc/sys/net/ipv4/conf/${SHORT_DEV}/hidden
@		if [ $? = 1 ]
@		then 
@			echo "Error: unhide $SHORT_DEV failed" 1>&2
@			((NUM_ERRORS++))
@			let HIDE_ERRORS=$HIDE_ERRORS+1
@		fi
@	fi
@	if [ "$HIDE_ERRORS" != "0" ]
@	then
@		echo "Error: you probably don't have the hidden patch applied"
@	fi
@	}
@
@detect_realserver_os()
@	{
@	#despite the name of this routine, it installs the VIP for known OS's 
@	DEV=$SERVER_VIP_DEVICE
@	#hopeless doing regexp in shell, do it in perl and put the result here
@	#SHORT_DEV=${DEV%:*}	#lo:0 -> lo, tunl0:0 -> tunl0
@	SHORT_DEV=$SHORT_SERVER_VIP_DEVICE	#lo:0 -> lo, tunl0:0 -> tunl0
@	#echo "SHORT_DEV=$SHORT_DEV"
@	ARP_STATUS="-arp"
@	case $UNAME in 
@		Linux )
@			case $UNAME_R in
@				2.0.* )
@				#DEV="lo:0"
@				NETMASK="0xffffffff"
@				ALIAS=""
@				install_realserver_vip
@				;;
@				2.2.*|2.4.* )
@				NETMASK="0xffffffff"
@				ALIAS=""
@				ARP_STATUS=""
@				if [ "$SERVER_VIP_DEVICE" = "TP" ]
@				then
@					#detect OS version in the subroutine
@					setup_realserver_TP
@				else
@					#bash has terrible string/regex handling - erk
@					#test for "pre","-pre","-test" at the end of uname_r eg 2.2.15pre10
@					#you can't test for a string (like "pre") 
@					#but you can remove it and see if the string is changed.
@					#if [ ${UNAME_R%pre*} != "$UNAME_R" ]
@					#UNAME_R="2.2.1-9"	#for testing
@					#echo "UNAME_R $UNAME_R"		
@					#echo "UNAME_R no suffix ${UNAME_R%%[\-A-Za-z]*}"		
@					if [ ${UNAME_R%%[\-A-Za-z]*} != "$UNAME_R" ]
@					then
@						echo -n "OS verion ${UNAME}-${UNAME_R} being treated as minor version "
@						UNAME_R=${UNAME_R%%[\-A-Za-z]*}
@						if [ $LINUX_KERNEL_SERIES = "2.2" ]
@						then
@							MINOR_VERSION_NUMBER=${UNAME_R#2\.2\.} 
@						elif [ $LINUX_KERNEL_SERIES = "2.4" ]
@						then
@							MINOR_VERSION_NUMBER=${UNAME_R#2\.4\.} 
@						else
@							echo "Error: detect_realserver_os: unknown kernel series $LINUX_KERNEL_SERIES" 1>&2
@							((NUM_ERRORS++))
@						fi
@						#decrement MINOR_VERSION_NUMBER 
@						if [ $MINOR_VERSION_NUMBER != "0" ]
@						then
@							MINOR_VERSION_NUMBER=$(($MINOR_VERSION_NUMBER - 1))
@						fi
@						echo "$MINOR_VERSION_NUMBER"
@						UNAME_R="${LINUX_KERNEL_SERIES}.${MINOR_VERSION_NUMBER}"
@					else
@						#MINOR_VERSION_NUMBER=${UNAME_R#2\.2\.}
@						if [ $LINUX_KERNEL_SERIES = "2.2" ]
@						then
@							MINOR_VERSION_NUMBER=${UNAME_R#2\.2\.} 
@						elif [ $LINUX_KERNEL_SERIES = "2.4" ]
@						then
@							MINOR_VERSION_NUMBER=${UNAME_R#2\.4\.} 
@						else
@							echo "Error: detect_realserver_os: unknown kernel series $LINUX_KERNEL_SERIES" 1>&2
@							((NUM_ERRORS++))
@						fi
@					fi 
@					#
@                                       # echo "MINOR_VERSION_NUMBER $MINOR_VERSION_NUMBER"
@					if [ $CONNECTION_TYPE = $DR ] || [ $CONNECTION_TYPE = $TUN ]
@					then
@						#Kyle Sparger ksparger@dialtoneinternet.net pointed out 
@						#that there is a race condition setting up a hidden device 
@						#in Linux 2.2.x, 2.4.x, 
@						#if the device is ifconfig'ed with arp on and then hidden. 
@						#Between these two steps, the device can reply to an arp request.
@						#handle it here	FIXME
@
@						#DEV="lo:0" or "tunl0:0"
@						install_realserver_vip
@						#can't do this (ie (A -a B) -o C) in bash :-(
@						#if [ ( $MINOR_VERSION_NUMBER -ge 14 -a $LINUX_KERNEL_SERIES =  "2.2" ) -o $LINUX_KERNEL_SERIES = "2.4" ]
@						#do it in parts
@						if [  $MINOR_VERSION_NUMBER -ge 14 -a $LINUX_KERNEL_SERIES =  "2.2" ]
@						then
@							#hide VIP, OS minor_version >=14
@							hide_all
@						elif [ $LINUX_KERNEL_SERIES = "2.4" ]
@						then
@							hide_all
@						elif [ $MINOR_VERSION_NUMBER -lt 14 -a $LINUX_KERNEL_SERIES =  "2.2" ]
@						then
@							echo "" 
@							echo "Warning:${UNAME}-${UNAME_R} does not support hiding devices" 1>&2
@							echo "upgrade to at least 2.2.14 or else read the HOWTO for solving the arp problem"
@							echo "the VIP device on the realserver will arp and the LVS will not work in its current state"
@							echo "" 
@							((NUM_WARNINGS++))
@						else
@							echo "detect_realserver_os: unknown kernel series $LINUX_KERNEL_SERIES minor version $MINOR_VERSION_NUMBER"
@					
@						fi 
@					else
@						echo "unknown connection type $CONNECTION_TYPE"
@					fi
@				fi
@				;;
@				* )
@				realserver_osversion_unknown
@				;;
@			esac
@			;;
@	
@	#uname      : FreeBSD
@	#uname -r   : 3.2-RELEASE (tested with 2.2.5, 3.2, 3.3, 4.2)
@	#<command>  : ifconfig lo0 alias <VIP> netmask 0xffffffff -arp up 
@	#ifconfig -a: lo0: flags=80c9<UP,LOOPBACK,RUNNING,NOARP,MULTICAST>mtu 16837
@	#                  inet 127.0.0.1 netmask 0xff000000
@	#                  inet <VIP> netmask 0xffffffff
@		FreeBSD )
@			case $UNAME_R in
@				4.2* | 3.2* | 3.3* | 2.2.5* )
@				#DEV="lo0"
@				NETMASK="0xffffffff"
@				ALIAS="alias"
@				if [ $CONNECTION_TYPE = $TUN ]
@				then
@					echo "Error:tunneling not supported with ${UNAME}-${UNAME_R}" 1>&2
@					echo "VIP device not installed, no action taken" 
@					((NUM_ERRORS++))
@				else
@					install_realserver_vip
@				fi
@				;;
@				* )
@				realserver_osversion_unknown
@				;;
@			esac
@			;;
@	#
@	#uname      : IRIX
@	#uname -r   : 6.5
@	#<command>  : ifconfig lo0 alias <VIP> netmask 0xffffffff -arp up
@	#ifconfig -a: lo0: flags=18c9<UP,LOOPBACK,RUNNING,NOARP,MULTICAST,CKSUM>
@	#                  inet 127.0.0.1 netmask 0xff000000
@	#                  inet <VIP> netmask 0xffffffff
@		IRIX )
@			case $UNAME_R in
@				6.5 )
@				#DEV="lo0"
@				NETMASK="0xffffffff"
@				ALIAS="alias"
@				if [ $CONNECTION_TYPE = $TUN ]
@				then
@					echo "Error:tunneling not supported with ${UNAME}-${UNAME_R}" 1>&2
@					echo "VIP device not installed, no action taken" 
@					((NUM_ERRORS++))
@				else
@					install_realserver_vip
@				fi
@				;;
@				* )
@				realserver_osversion_unknown
@				;;
@			esac
@			;;
@	
@	#uname      : SunOS
@	#uname -r   : 5.7
@	#<command>  : ifconfig lo0:1 <VIP> netmask 255.255.255.255 up
@	#ifconfig -a: lo0:  flags=849<UP,LOOPBACK,RUNNING,MULTICAST>mtu 8232
@	#                   inet 127.0.0.1 netmask ff000000
@	#             lo0:1 flags=849<UP,LOOPBACK,RUNNING,MULTICAST>mtu 8232
@	#                   inet <VIP> netmask ffffffff
@	#from sraja@mediasc.com for Solaris 8
@	#           : 5.8
@	#           : ifconfig lo0 addif $VIP up
@		SunOS )
@			case $UNAME_R in
@				5.7 )
@				#DEV="lo0"
@				NETMASK="0xffffffff"
@				ALIAS="alias"
@				if [ $CONNECTION_TYPE = $TUN ]
@				then
@					echo "Error:tunneling not supported with ${UNAME}-${UNAME_R}" 1>&2
@					echo "VIP device not installed, no action taken" 
@					((NUM_ERRORS++))
@				else
@					install_realserver_vip
@				fi
@				;;
@				* )
@				realserver_osversion_unknown
@				;;
@			esac
@			;;
@	
@	#
@	#uname      : HP-UX
@	#uname -r   : B.11.00
@	#<command>  : ifconfig lan1:1 10.10.10.10 netmask 0xffffff00 -arp up
@	#ifconfig -a: lan0:   flags=842<BROADCAST,RUNNING,MULTICAST>
@	#                     inet <some IP> netmask ffffff00
@	#             lan0:1: flags=8c2<BROADCAST,RUNNING,NOARP,MULTICAST>
@	#                     inet <VIP> netmask ffffff00
@	#
@		HP-UX )
@			case $UNAME_R in
@			B.11.00 )
@				#DEV="lan1:1"
@				NETMASK="0xffffff00"
@				ALIAS=""
@				if [ $CONNECTION_TYPE = $TUN ]
@				then
@					echo "Error:tunneling not supported with ${UNAME}-${UNAME_R}" 1>&2
@					echo "VIP device not installed, no action taken" 
@					((NUM_ERRORS++))
@				else
@					install_realserver_vip
@					echo "warning HP-UX lo replies to arp requests."
@					echo "This will work if the VIP is on a separate ethernet device."
@					echo "See the HOWTO about solutions to the arp problem."
@				fi
@				;;
@				* )
@				realserver_osversion_unknown
@				echo "Warning: HP-UX lo replies to arp requests."
@				echo "Have to handle arp problem by hand."
@				((NUM_WARNINGS++))
@				;;
@			esac
@	
@		;;
@
@		#AIX, from Julian, 5 Feb 2002
@		#ifconfig lo0 alias VIP netmask 255.255.255.255 -arp up
@		AIX )
@			case $UNAME_R in
@				* )
@				#DEV="lo0"
@				NETMASK="0xffffffff"
@				ALIAS="alias"
@				if [ $CONNECTION_TYPE = $TUN ]
@				then
@					echo "Error:tunneling not supported with ${UNAME}-${UNAME_R}" 1>&2
@					echo "VIP device not installed, no action taken" 
@					((NUM_ERRORS++))
@				else
@					install_realserver_vip
@				fi
@				;;
@				#* )
@				#realserver_osversion_unknown
@				#;;
@			esac
@			;;
@		* )
@		realserver_os_unknown
@		;;
@	esac
@	}
@#end rc.ratz-------------------------------------------
@
@#--------was rc.horms--------------------------
@setup_realserver_TP()
@	{
@	#realserver and director TP setup are different
@	#on director accept all services
@	#on realserver only accept services destined for that realserver
@	#
@	#test if TP on real-server and real-server is not linux
@	if [ \$SERVER_VIP_DEVICE = "TP" -a $UNAME != "Linux" ]
@	then
@		echo "Error: Transparent proxy not supported on $UNAME $UNAME_R " 1>&2 
@		((NUM_ERRORS++))  
@	fi
@
@	#turn on IP forwarding (off by default in 2.2.x kernels)
@	echo ""
@	echo "Turn on ip_forward on real-server (1 on, 0 off), required for TP."
@	echo "1" >/proc/sys/net/ipv4/ip_forward
@	echo -n "proc/sys/net/ipv4/ip_forward "  
@	$CAT /proc/sys/net/ipv4/ip_forward
@	if [ `$CAT /proc/sys/net/ipv4/ip_forward` != "1" ]
@	then
@		echo "Error: turn on ip_forward failed." 1>&2
@		((NUM_ERRORS++))
@	fi 
@	echo ""
@	
@	#flush ipchains table
@	#$IPCHAINS -F input
@	print_redirect_services
@	#list ipchain rules
@	#echo ""
@	#echo "listing ipchains rules"
@	#$IPCHAINS -L input
@	}
@#end rc.horms------------------------------------
@
@vip_warning()
@	{
@	echo "Warning: " 1>&2
@	#new version
@	echo "You are moving the VIP from one device" 1>&2
@	echo "(presumably from a previous LVS installation)," 1>&2
@	echo "to a different ethernet device." 1>&2
@	echo "This is difficult to do with ifconfig." 1>&2
@	echo "It's easy to do with iproute2." 1>&2
@	echo "If the iproute2 tools were installed," 1>&2
@	echo "you would not have seen this message." 1>&2
@	echo "Till then you'll have to do the changeover yourself with this script." 1>&2
@	echo "" 1>&2
@	echo "1. Read (with perldoc) the documentation on " 1>&2
@	echo "CHANGING VIP BETWEEN ETH DEVICES ON DIRECTOR." 1>&2
@	echo "" 1>&2
@	echo "2. (After editing) run rc.remove_vip " 1>&2
@	echo "(it's in the local directory), with the command." 1>&2
@	echo "" 1>&2
@	echo "director:/etc/lvs# ./rc.remove_vip" 1>&2
@	echo "" 1>&2
@	echo "If there are no errors, re-run the rc.lvs file" 1>&2
@	echo "and you should not see this notice next time." 1>&2
@	echo "" 1>&2
@	echo "NOTE: this node is not setup." 1>&2
@	echo "" 1>&2
@	write_remove_vip
@	}
@
@find_vip_devices()
@	{
@	#which (ifconfig) devices are on this machine
@	DEVICES=`$NETSTAT -rn | $GREP "$VIP " | $AWK '{print $8}' | $XARGS` 
@	}
@
@remove_old_vip()
@	{
@	#don't remove device with VIP if it's going to be the replacement
@	#
@	#much later - oooh - big blunder. 
@	#If the VIP is still on the real-server when you do a 
@	#ping check of the VIP (looking for it on the director),
@	#you'll find the local VIP.
@	#Handle this now with check_for_vip_on_director and ping_test_vip_on_director	
@	#
@	VIP_FOUND="N"
@	DEVICES=`$IFCONFIG -a | $CUT -c0-10 | $XARGS` 
@	for DEVICE in $DEVICES
@	do
@		#echo "checking device $DEVICE for VIP"
@		NUMBER_VIPS=`$IFCONFIG $DEVICE | $GREP -c "$VIP "`
@		let NUMBER_VIPS="$NUMBER_VIPS"
@		if [ ${NUMBER_VIPS} -ne 0 ]
@		then
@			echo "device $DEVICE has VIP $VIP"
@			#if this is an ethernet alias (eg eth0:110)
@			#then bringing it down will also bring down 
@			#all eth devices and 
@			#remove the entries from the routing table.
@			#
@			#However if we are just deinstalling and reinstalling
@			#the same aliased device (eg eth0:110) then we don't
@			#need to bother anyone with alarming messages.
@			#
@			if [ "$DEVICE" = "$NEW_VIP_DEVICE" ]
@			then
@				#do nothing, new device == old device
@				echo "new VIP device == old VIP device, don't reinstall"
@				#echo "old VIP device = new VIP device"
@			else
@				#do we have to remove an eth device?
@				echo "DEBUG: moving VIP $VIP from $DEVICE to $NEW_VIP_DEVICE"
@				SHORT_DEV=${DEVICE%:*}	#eth0:0 -> eth0, tunl0 -> tunl0
@				echo "DEBUG: $DEVICE maybe an alias for $SHORT_DEV. Is it an ethernet device?"
@				THIS_DEVICE=`$EXPR "$DEVICE" : '\(...\).*' ` #first 3 chars of device, eth0:110 -> eth
@				#looking for eth devices only
@				if [ "$THIS_DEVICE" = "eth" ]
@				then
@					echo "DEBUG: $DEVICE is an ethernet device"
@					#Apr 2002, v0.9.4
@					#if have iproute2, can remove ip
@					if [ "$IP" = "" ]
@					then
@						vip_warning
@						exit 1
@					else
@						echo "DEBUG: using $IP to remove $VIP from $SHORT_DEV"
@						$IP addr del $VIP dev $SHORT_DEV
@						IPROUTE2_USED="Y"
@					fi
@				fi # eth device   
@			fi # replacing VIP onto device different to that found with ifconfig -a
@			#echo "is it up?"
@			#I don't know why I do this next bit.
@			#maybe it's a way of removing local aliases 
@			#eg lo:0 dummy0:0 from the ifconfig table.
@			#hmm, buggy code. 
@			#if you have eth0:1 but no eth0:110
@			#and you ask $IFCONFIG eth0:1 :-(
@			#you'll get the output for ethand you 
@			if [ `$IFCONFIG $DEVICE | $GREP -c "UP "` != "0" ]
@			then
@				VIP_FOUND="Y"
@				echo "DEBUG: device $DEVICE has VIP $VIP and is UP"
@				echo "DEBUG: removing $VIP from $DEVICE"
@				$IFCONFIG $DEVICE down 2>&1
@				if [ $? != "0" ]; then echo "DEBUG: error ignored" ; fi
@			else
@				#echo "it's not up"
@				echo -n "DEBUG: VIP device $DEVICE is DOWN"
@			fi 
@		fi
@	done
@
@	#DEVICES=`$NETSTAT -rn | $GREP "$VIP " | $AWK '{print $8}' | $XARGS` 
@	find_vip_devices
@	for DEVICE in $DEVICES
@	do
@		echo "removing route to VIP through $DEVICE"
@		$ROUTE del -host $VIP $DEVICE
@	done
@
@	if [ $VIP_FOUND = "N" ]
@	then
@		echo "$VIP not found on any network devices, good"
@		#echo -n ""
@	fi
@	echo ""
@	}
@
@get_current_gw(){
@	# routing table can have multiple default gw
@	NUMBER_DEFAULT_GW=`$NETSTAT -rn| $GREP "^0.0.0.0" | $AWK '{print $2}' | $WC -l | $XARGS `
@	#turn it into a number
@	let NUMBER_DEFAULT_GW="$NUMBER_DEFAULT_GW"+0
@	#get first default gw
@	CURRENT_GW=`$NETSTAT -rn| $GREP "^0.0.0.0" | $AWK '{print $2}'| $XARGS | $AWK '{print $1}'`
@	let CURRENT_DEFAULT_GW="$CURRENT_DEFAULT_GW"+0
@	echo "number of default gw $NUMBER_DEFAULT_GW, first gw $CURRENT_GW "
@	}
@
@install_default_gw(){
@	#Note: Aug 2001. default gw's are being removed from the code.
@	#For the realservers, a default is set here, 
@	#then if the iproute2 utils are installed, 
@	#routes are set for packets from the VIP and from the RIP and the default gw is deleted.
@	#I'll be doing the same thing for the director sometime.
@	#
@	#This code installs the default gw for both the director and the realservers.
@	#While academically this is the way to go (using the same code again),
@	#it is difficult to maintain.
@	#I may split this into two routines eventually.
@	#
@	echo "installing default gw $DEFAULT_GW for $LVS_TYPE"
@	get_current_gw
@	
@	#it's a little tricky to test if only one default gw is installed
@	#and if it's IP you want to keep for the setup. 
@	#Just strip out all the gw's and re-install the correct one.
@	
@	#echo " "
@	while [ "$NUMBER_DEFAULT_GW" != "0" ]
@	do
@		echo "deleting current default gw $CURRENT_GW"
@		$ROUTE del default gw $CURRENT_GW
@		if [ "$?" != "0" ]
@		then
@			echo "Error: cannot remove default gw $CURRENT_GW"
@			echo "Have $NUMBER_DEFAULT_GW gateways."
@			((NUM_ERRORS++))
@			break
@		fi
@		get_current_gw
@	done
@
@	#install a default gw if we have one.
@	#echo "DEFAULT_GW $DEFAULT_GW."
@	#quote $DEFAULT_GW as can be null
@	if [ "$DEFAULT_GW" = "" ]
@	then	
@		echo "not installing a default gw for LVS_TYPE $LVS_TYPE"
@	else
@		echo "setting default gw to $DEFAULT_GW"
@		$ROUTE add default gw $DEFAULT_GW
@		echo "showing routing table"
@		echo ""
@		$NETSTAT -rn
@		echo ""
@		echo -n "checking if DEFAULT_GW $DEFAULT_GW is reachable - "
@		#the tr removes the \n, 
@		#but combined command gives the return code for tr.
@		#if fping fails, then I won't see the failure. 
@		#this resulted in a bug in 0.7, which is hopefully fixed here.
@		#$FPING $DEFAULT_GW | $TR -d '\n'	#remove \n from fping
@		$FPING $DEFAULT_GW 
@		if [ "$?" = "0" ]
@		then 
@			echo "good"
@		else
@			echo "Error: cannot connect to default gw $DEFAULT_GW." 1>&2
@			((NUM_ERRORS++)) 
@		fi
@	fi 
@	} #install_default_gw
@
@
@check_for_DIP(){
@	#since this is a realserver, there should be no DIP here 
@	#and it should have been installed on the director by now. 
@	echo "looking for DIP $DIP "
@	$FPING $DIP 
@	if [ $? = "0" ] 
@	then 
@		echo "found, good"
@		#check that it's not local.
@		#(we shouldn\'t have got here if it is)
@		if [ `$IFCONFIG -a | $GREP -c "$DIP "` = "0" ] 
@		then 
@			echo "not local, good "
@		else 
@			echo "Error: DIP is local - " 1>&2
@			echo "we shouldn't even have got here!"
@			((NUM_ERRORS++))
@		fi 
@	 else 
@	 	echo "DIP not found, director has not been setup yet." 
@	 	echo "rerun this script after setting up director" 
@	 	echo "so that all tests can be run." 
@	 fi 
@	 } #check_for_DIP 
@
@ping_test_VIP_on_director(){
@	echo "pinging VIP $VIP from RIP" 
@	#with the addition of Julian's martian modification to the LVS options
@	#a different scheme for testing the reachability of the VIP
@	#from the real-servers was devised.
@	#The SERVER_GW is either 
@	#the DIP (VS-NAT, or VS-DR with Julian's martian modification)
@	#or
@	#a router which is NOT the director.
@
@	#For a particular forwarding method (VS-NAT,DR, Tun)
@	#if
@	#	1. the director is setup (ipvadm commands run)
@	#	and
@	#	2. the real-server has the default gw set
@	#	and
@	#	3. the real-server doesn't have the VIP installed yet
@	#then	
@	#	1. the real-server can ping the VIP on the director 
@	#	   if the VIP is in the real-server network 
@	#	   (ie a one network LVS)
@	#	   no matter which forwarding method is used. (duh)
@	#	2. the real-server can ping the VIP on a VS-NAT director, 
@	#	   if it's on a different network to the real-server,
@	#	   as the VIP is on the default route from the real-server,
@	#	   which goes through the director, and the director 
@	#	   forwards packets from the real-server network,
@	#	   which is neccessary for the VS-NAT director to return
@	#	   packets to the internet/client/router.
@	#	3. For a VS-DR/VS-Tun director, the default gw for the
@	#	   real-server is the router (or test client). 
@	#	   If the VIP and the real-server network are different,
@	#	   then the connection to the VIP depends on whether
@	#	   the router forwards packets. 
@	#	   Presumably forwarding packets is what routers 
@	#	   do and the VIP should be pingable.
@	#	   In my setup, the place of the router is taken by 
@	#	   my client, a linux 2.2.x box, with ip_forward off
@	#	   by default..
@	#	   If ip_forward is on in the router, then the real-server 
@	#	   can ping the VIP on the director.
@	#
@	# Is SERVER_GW on the director
@	if [ "$SERVER_GW" = "$DIP" ]
@	then
@		echo "server gw on director."
@		if [ "$LVS_TYPE" = "vs-nat" ]
@		then
@			echo "OK is ${LVS_TYPE}"
@			ping_vip_server_gw_on_director
@		elif [ "$LVS_TYPE" = "vs-dr" ]
@		then
@			echo "is $LVS_TYPE with Julian's martian modification."
@			ping_vip_server_gw_on_director
@		elif [ "$LVS_TYPE" = "vs-tun" ]
@		then
@			echo "Error: cannot have server gw on director for $LVS_TYPE" 1>&2
@			((NUM_ERRORS++))
@		else
@			echo "Error: you shouldn't get here. LVS type $LVS_TYPE" 1>&2
@			((NUM_ERRORS++))
@		fi
@	else
@		echo "server gw not on director"
@		if [ "$LVS_TYPE" = "vs-nat" ]
@		then
@			echo "Error: for $LVS_TYPE server gw must be on director." 
@			((NUM_ERRORS++))
@		elif [ "$LVS_TYPE" = "vs-dr" ]
@		then
@			echo "OK is $LVS_TYPE"
@			ping_vip_server_gw_not_on_director
@		elif [ "$LVS_TYPE" = "vs-tun" ]
@		then
@			echo "OK is $LVS_TYPE"
@			ping_vip_server_gw_not_on_director
@		else
@			echo "Error: you shouldn't get here. LVS type $LVS_TYPE" 1>&2
@			((NUM_ERRORS++))
@		fi
@	fi
@	} #ping_test_VIP_on_director
@
@ping_vip_server_gw_on_director(){
@	echo "LVS_TYPE  $LVS_TYPE"
@	echo "VIP $VIP should be pingable "
@	$FPING $VIP	
@	if [ "$?" = "0" ] 
@	then 
@		#echo "$VIP pingable, good. "
@		echo "good. "
@		#echo "It's not on this real-server, assume it's on the director. "
@	else
@		echo "Error:$VIP not pingable." 1>&2
@		echo "check routing, forwarding, cabling, " 1>&2
@		echo "values for default gws " 1>&2
@		echo "and errors from running rc.lvs on the director." 1>&2
@		echo "Director, real-servers and their default gw(s)" 1>&2
@		echo "should be connected before running this script." 1>&2
@		((NUM_ERRORS++)) 
@	fi
@	} #ping_vip_server_gw_on_director
@
@ping_vip_server_gw_not_on_director(){
@	if [ $VIP_RIP_SAME_NETWORK = "Y" ]
@	then
@		echo "VIP is in the same network as the RIP "
@		echo "VIP $VIP should be pingable "
@		$FPING $VIP	
@		if [ "$?" = "0" ] 
@		then 
@			#echo "$VIP pingable, good. "
@			echo "good. "
@			#echo "It's not on this real-server, assume it's on the director. "
@		else
@			echo "Error:$VIP not pingable." 1>&2
@			echo "check routing, forwarding, cabling, " 1>&2
@			echo "values for default gws " 1>&2
@			echo "and errors from running rc.lvs on the director." 1>&2
@			echo "Director, real-servers and their default gw(s)" 1>&2
@			echo "should be connected before running this script." 1>&2
@			((NUM_ERRORS++)) 
@		fi
@	#this is now handled in ping_vip_server_gw_on_director
@	#elif [ $LVS_TYPE = "vs-nat" ]
@	#then
@	#	#The real-server should be able to ping the VIP with VS-NAT
@	#	#in both the single and two network case as the director 
@	#	#is the default gw for the real-servers in both cases.
@	#	#packets from the real-server through the director.
@	#	#This is the same code as for VIP_RIP_SAME_NETWORK = Y.
@	#	#The logic for the subrouting is a bit clearer 
@	#	#if I duplicate the code here, as the echo output is different.
@	#	echo "LVS_TYPE  $LVS_TYPE"
@	#	echo "VIP $VIP should be pingable "
@	#	$FPING $VIP	
@	#	if [ "$?" = "0" ] 
@	#	then 
@	#		#echo "$VIP pingable, good. "
@	#		echo "good. "
@	#		#echo "It's not on this real-server, assume it's on the director. "
@	#	else
@	#		echo "Error:$VIP not pingable." 1>&2
@	#		echo "check routing, forwarding, cabling, " 1>&2
@	#		echo "values for default gws " 1>&2
@	#		echo "and errors from running rc.lvs on the director." 1>&2
@	#		echo "Director, real-servers and their default gw(s)" 1>&2
@	#		echo "should be connected before running this script." 1>&2
@	#		((NUM_ERRORS++)) 
@	#	fi
@	else
@		#Handling a two network VS-DR or VS-Tun LVS
@		#flushing the arp table sounded like a good idea initially,
@		#but if the arp table doesn't have an entry for $VIP
@		#and the network of the $VIP is uncontactable,
@		#you'll get the error "SIOCDARP(priv): Network is unreachable".
@		#This will just confuse people, so leave out flushing and checking arp table.
@		#echo "flush VIP $VIP from arp table"
@		#$ARP -d $VIP
@		echo "LVS_TYPE is $LVS_TYPE and VIP $VIP not in same network as RIP "
@		#have to handle 3 cases
@		#1. ROUTER_FORWARDS=""	#default in distribution, ie have no idea about router
@		#	Issue warnings whether VIP pingable or not.
@		#2. ROUTER_FORWARDS="Y"
@		#	Issue error if VIP not pingable
@		#3. ROUTER_FORWARDS="N"
@		#	Issue error if VIP pingable
@		#
@		if [ "$ROUTER_FORWARDS" = "" ]
@		then
@			echo "Warning: You haven't set \"\$ROUTER_FORWARDS\"  " 1>&2
@			echo "in the User configurable section of configure.pl, " 1>&2
@			echo "so we don't know if the VIP should be pingable or not." 1>&2
@			echo "This ping test is just extra checking and the LVS " 1>&2
@			echo "will work whether or not this ping test is properly setup." 1>&2
@			echo "If this test is properly setup and fails, " 1>&2
@			echo "it will help diagnose problems with your LVS." 1>&2
@			echo "" 1>&2
@			echo "VIP should not be pingable if forwarding is off in the box router/test_client"
@			echo "VIP should     be pingable if forwarding is on  in the box router/test_client"
@			echo "" 1>&2
@			$FPING $VIP	
@			if [ "$?" = "0" ] 
@			then 
@				echo "" 1>&2
@				echo "VIP $VIP pingable." 1>&2
@				echo "Output from route -Cn" 1>&2
@				echo "source              dest             gw"  1>&2
@				route -Cn | $GREP "$SERVER_GW " | $GREP "$VIP " 1>&2
@				echo "" 1>&2
@				echo "If you find that your LVS works " 1>&2
@				echo "AND you aren't going to be changing the routing " 1>&2
@				echo "on the real-server's default gw $SERVER_GW, " 1>&2
@				echo "then set \"\$ROUTER_FORWARDS\"=\"Y\" " 1>&2
@			else
@				echo "$VIP not pingable. " 1>&2
@				echo "If you find that your LVS works " 1>&2
@				echo "AND you aren't going to be changing the routing " 1>&2
@				echo "on the real-server's default gw $SERVER_GW, " 1>&2
@				echo "then set \"\$ROUTER_FORWARDS\"=\"N\" " 1>&2
@			fi
@			((NUM_WARNINGS++)) 
@		fi
@
@		if [ "$ROUTER_FORWARDS" = "N" ]
@		then
@			$FPING $VIP	
@			if [ "$?" = "0" ] 
@			then 
@				echo "" 1>&2
@				echo "Error:VIP $VIP pingable" 1>&2
@				echo "Here's output from ifconfig -a" 1>&2
@				$IFCONFIG -a 1>&2
@				echo "Here's output from netstat -rn" 1>&2
@				$NETSTAT -rn $VIP 1>&2
@				echo "Here's output from traceroute." 1>&2
@				$TRACEROUTE $VIP 1>&2
@				echo "and from arp -a" 1>&2
@				$ARP -a 1>&2 
@				echo "" 1>&2
@				echo "Output from route -Cn" 1>&2
@				echo "source              dest             gw" 1>&2
@				$ROUTE -Cn | $GREP "$SERVER_GW " | $GREP "$VIP " 1>&2
@				echo "" 1>&2
@				echo "check routing, forwarding, cabling, " 1>&2
@				echo "values for default gws " 1>&2
@				echo "and errors from running rc.lvs on the director." 1>&2
@				echo "Director, real-servers and their default gw(s)" 1>&2
@				echo "should be connected before running this script." 1>&2
@				((NUM_ERRORS++)) 
@			else
@				echo "VIP $VIP not pingable, good. "
@				#echo "Assume it's been installed on the director. "
@			fi
@		fi
@
@		if [ "$ROUTER_FORWARDS" = "Y" ]
@		then
@			$FPING $VIP	
@			if [ "$?" = "0" ] 
@			then 
@				echo "VIP $VIP pingable, good. "
@				#echo "Assume it's been installed on the director. "
@			else
@				echo "" 1>&2
@				echo "Error:VIP $VIP not pingable" 1>&2
@				echo "check routing, forwarding, cabling, " 1>&2
@				echo "values for default gws " 1>&2
@				echo "and errors from running rc.lvs on the director." 1>&2
@				echo "Director, real-servers and their default gw(s)" 1>&2
@				echo "should be connected before running this script." 1>&2
@				((NUM_ERRORS++)) 
@			fi
@		fi
@	fi 
@	} #ping_vip_server_gw_not_on_director
@
@old_ping_test_VIP_on_director(){
@	echo "pinging VIP $VIP from RIP" 
@	#For a particular forwarding method (VS-NAT,DR, Tun)
@	#if
@	#	1. the director is setup (ipvadm commands run)
@	#	and
@	#	2. the real-server has the default gw set
@	#	and
@	#	3. the real-server doesn't have the VIP installed yet
@	#then	
@	#	1. the real-server can ping the VIP on the director 
@	#	   if the VIP is in the real-server network 
@	#	   (ie a one network LVS)
@	#	   no matter which forwarding method is used. (duh)
@	#	2. the real-server can ping the VIP on a VS-NAT director, 
@	#	   if it's on a different network to the real-server,
@	#	   as the VIP is on the default route from the real-server,
@	#	   which goes through the director, and the director 
@	#	   forwards packets from the real-server network,
@	#	   which is neccessary for the VS-NAT director to return
@	#	   packets to the internet/client/router.
@	#	3. For a VS-DR/VS-Tun director, the default gw for the
@	#	   real-server is the router (or test client). 
@	#	   If the VIP and the real-server network are different,
@	#	   then the connection to the VIP depends on whether
@	#	   the router forwards packets. 
@	#	   Presumably forwarding packets is what routers 
@	#	   do and the VIP should be pingable.
@	#	   In my setup, the place of the router is taken by 
@	#	   my client, a linux 2.2.x box, with ip_forward off
@	#	   by default..
@	#	   If ip_forward is on in the router, then the real-server 
@	#	   can ping the VIP on the director.
@	#	
@#	if [ $VIP_RIP_SAME_NETWORK = "Y" -o $LVS_TYPE = "vs-nat" ]
@#	then
@#		if [ $VIP_RIP_SAME_NETWORK = "Y" ]
@#		then
@#			echo "VIP is in the same network as the RIP "
@#		fi
@#		if [ $LVS_TYPE = "vs-nat" ]
@#		then
@#			echo "LVS_TYPE  $LVS_TYPE"
@#		fi
@#		echo "VIP $VIP should be pingable "
@#		$FPING $VIP	
@#		if [ "$?" = "0" ] 
@#		then 
@#			#echo "$VIP pingable, good. "
@#			echo "good. "
@#			#echo "It's not on this real-server, assume it's on the director. "
@#		else
@#			echo "Error:$VIP not pingable." 1>&2
@#			echo "check routing, forwarding, cabling, " 1>&2
@#			echo "values for default gws " 1>&2
@#			echo "and errors from running rc.lvs on the director." 1>&2
@#			echo "Director, real-servers and their default gw(s)" 1>&2
@#			echo "should be connected before running this script." 1>&2
@#			((NUM_ERRORS++)) 
@#		fi
@	if [ $VIP_RIP_SAME_NETWORK = "Y" ]
@	then
@		echo "VIP is in the same network as the RIP "
@		echo "VIP $VIP should be pingable "
@		$FPING $VIP	
@		if [ "$?" = "0" ] 
@		then 
@			#echo "$VIP pingable, good. "
@			echo "good. "
@			#echo "It's not on this real-server, assume it's on the director. "
@		else
@			echo "Error:$VIP not pingable." 1>&2
@			echo "check routing, forwarding, cabling, " 1>&2
@			echo "values for default gws " 1>&2
@			echo "and errors from running rc.lvs on the director." 1>&2
@			echo "Director, real-servers and their default gw(s)" 1>&2
@			echo "should be connected before running this script." 1>&2
@			((NUM_ERRORS++)) 
@		fi
@	elif [ $LVS_TYPE = "vs-nat" ]
@	then
@		#The real-server should be able to ping the VIP with VS-NAT
@		#in both the single and two network case as the director 
@		#is the default gw for the real-servers in both cases.
@		#packets from the real-server through the director.
@		#This is the same code as for VIP_RIP_SAME_NETWORK = Y.
@		#The logic for the subrouting is a bit clearer 
@		#if I duplicate the code here, as the echo output is different.
@		echo "LVS_TYPE  $LVS_TYPE"
@		echo "VIP $VIP should be pingable "
@		$FPING $VIP	
@		if [ "$?" = "0" ] 
@		then 
@			#echo "$VIP pingable, good. "
@			echo "good. "
@			#echo "It's not on this real-server, assume it's on the director. "
@		else
@			echo "Error:$VIP not pingable." 1>&2
@			echo "check routing, forwarding, cabling, " 1>&2
@			echo "values for default gws " 1>&2
@			echo "and errors from running rc.lvs on the director." 1>&2
@			echo "Director, real-servers and their default gw(s)" 1>&2
@			echo "should be connected before running this script." 1>&2
@			((NUM_ERRORS++)) 
@		fi
@	else
@		#Handling a two network VS-DR or VS-Tun LVS
@		#flushing the arp table sounded like a good idea initially,
@		#but if the arp table doesn't have an entry for $VIP
@		#and the network of the $VIP is uncontactable,
@		#you'll get the error "SIOCDARP(priv): Network is unreachable".
@		#This will just confuse people, so leave out flushing and checking arp table.
@		#echo "flush VIP $VIP from arp table"
@		#$ARP -d $VIP
@		echo "LVS_TYPE is $LVS_TYPE and VIP $VIP not in same network as RIP "
@		#have to handle 3 cases
@		#1. ROUTER_FORWARDS=""	#default in distribution, ie have no idea about router
@		#	Issue warnings whether VIP pingable or not.
@		#2. ROUTER_FORWARDS="Y"
@		#	Issue error if VIP not pingable
@		#3. ROUTER_FORWARDS="N"
@		#	Issue error if VIP pingable
@		#
@		#Jun 2001, have to handle VS-DR with Julian's martian modification
@		if [ "$ROUTER_FORWARDS" = "" ]
@		then
@			echo "Warning: You haven't set \"\$ROUTER_FORWARDS\"  " 1>&2
@			echo "in the User configurable section of configure.pl, " 1>&2
@			echo "so we don't know if the VIP should be pingable or not." 1>&2
@			echo "This ping test is just extra checking and the LVS " 1>&2
@			echo "will work whether or not this ping test is properly setup." 1>&2
@			echo "If this test is properly setup and fails, " 1>&2
@			echo "it will help diagnose problems with your LVS." 1>&2
@			echo "" 1>&2
@			echo "VIP should not be pingable if forwarding is off in the box router/test_client"
@			echo "VIP should     be pingable if forwarding is on  in the box router/test_client"
@			echo "" 1>&2
@			$FPING $VIP	
@			if [ "$?" = "0" ] 
@			then 
@				echo "" 1>&2
@				echo "VIP $VIP pingable." 1>&2
@				echo "Output from route -Cn" 1>&2
@				echo "source              dest             gw"  1>&2
@				route -Cn | $GREP "$SERVER_GW " | $GREP "$VIP " 1>&2
@				echo "" 1>&2
@				echo "If you find that your LVS works " 1>&2
@				echo "AND you aren't going to be changing the routing " 1>&2
@				echo "on the real-server's default gw $SERVER_GW, " 1>&2
@				echo "then set \"\$ROUTER_FORWARDS\"=\"Y\" " 1>&2
@			else
@				echo "$VIP not pingable. " 1>&2
@				echo "If you find that your LVS works " 1>&2
@				echo "AND you aren't going to be changing the routing " 1>&2
@				echo "on the real-server's default gw $SERVER_GW, " 1>&2
@				echo "then set \"\$ROUTER_FORWARDS\"=\"N\" " 1>&2
@			fi
@			((NUM_WARNINGS++)) 
@		fi
@
@		if [ "$ROUTER_FORWARDS" = "N" ]
@		then
@			$FPING $VIP	
@			if [ "$?" = "0" ] 
@			then 
@				echo "" 1>&2
@				echo "Error:VIP $VIP pingable" 1>&2
@				echo "Here's output from ifconfig -a" 1>&2
@				$IFCONFIG -a 1>&2
@				echo "Here's output from netstat -rn" 1>&2
@				$NETSTAT -rn $VIP 1>&2
@				echo "Here's output from traceroute." 1>&2
@				$TRACEROUTE $VIP 1>&2
@				echo "and from arp -a" 1>&2
@				$ARP -a 1>&2 
@				echo "" 1>&2
@				echo "Output from route -Cn" 1>&2
@				echo "source              dest             gw" 1>&2
@				$ROUTE -Cn | $GREP "$SERVER_GW " | $GREP "$VIP " 1>&2
@				echo "" 1>&2
@				echo "check routing, forwarding, cabling, " 1>&2
@				echo "values for default gws " 1>&2
@				echo "and errors from running rc.lvs on the director." 1>&2
@				echo "Director, real-servers and their default gw(s)" 1>&2
@				echo "should be connected before running this script." 1>&2
@				((NUM_ERRORS++)) 
@			else
@				echo "VIP $VIP not pingable, good. "
@				#echo "Assume it's been installed on the director. "
@			fi
@		fi
@
@		if [ "$ROUTER_FORWARDS" = "Y" ]
@		then
@			$FPING $VIP	
@			if [ "$?" = "0" ] 
@			then 
@				echo "VIP $VIP pingable, good. "
@				#echo "Assume it's been installed on the director. "
@			else
@				echo "" 1>&2
@				echo "Error:VIP $VIP not pingable" 1>&2
@				echo "check routing, forwarding, cabling, " 1>&2
@				echo "values for default gws " 1>&2
@				echo "and errors from running rc.lvs on the director." 1>&2
@				echo "Director, real-servers and their default gw(s)" 1>&2
@				echo "should be connected before running this script." 1>&2
@				((NUM_ERRORS++)) 
@			fi
@		fi
@	fi 
@	} #ping_test_VIP_on_director
@
@check_for_vip_on_director(){
@	echo ""
@	echo "looking for VIP on director from realserver"
@	#The cases to handle:
@	#1. The VIP is not on the real-server. 
@	#   then 
@	#	a. ping test for VIP which you expect will be on director 
@	#   	(which should have been installed by the time you get here).
@	#	b. install VIP on real-server
@	#   fi
@	#This will only happen if there has been no previous LVS install.
@	#If you are in the midst of a few rounds of setting up and testing, 
@	#you will be unlikely to find this situation.
@
@	#2. The VIP is already on the real-server.
@	#This is handled as 2 cases
@	#The problem is that you can't remove an IP from a device.
@	#You can only "down" the device,
@	#which only means that it won't accept packets from outside the machine.
@	#It will happily reply to pings on the local machine.
@	#For ethernet devices, you have to exit the install and rmmod the ethernet driver.
@	#Local devices appear benign and you can just ignore them. 
@	#i.e. you can have one already on tunl0 and install the same IP on lo:0 
@	#and get a functioning VS-DR LVS.
@	#I don't know how to remove these devices apart from rebooting :-(
@	#
@	#2a. VIP is on an ethernet device
@	#	If the VIP is on the correct ethernet device
@	#	then
@	#		do not run the ping test for the VIP (on director), 
@	#		as replies will come from the VIP on the real-server.
@	#		do not install the VIP again.
@	#	else
@	#		VIP on not on correct eth device.
@	#		This will have already been handled by remove_old_vip
@	#		By the time we get here.
@	#		remove_old_vip will - 	
@	#		write out script "rc.remove_vip" (to local directory), 
@	#		which when run will remove drivers for all eth devices.
@	#		write notice to screen.
@	#		exit (ie from rc.lvs)
@	#		From shell prompt, run rc.remove_ip by hand,
@	#		rerun rc.lvs, which should now install the LVS
@	#	fi
@	#		
@	#2b. VIP is on a local (lo) device alias eg lo:0 or tunl0:0, dummy0:0...
@	#	This will have already been removed by down'ing the lo alias,
@	#	which removes the VIP from the ifconfig output. 
@	#	ping test VIP. 
@	#	install the VIP on the real-server.
@	#
@	#I haven't done much with the VIP being accepted by TP
@	#(apart from getting it to work), i.e. I haven't followed
@	#what happens if I've done previous installs.
@	# 
@	#There is no VIP device installed yet on realserver.
@	#(If VS-NAT, will not be installing a VIP device on the realserver.)
@	echo ""
@	echo "looking for VIP on director from realserver"
@	if [ $DIRECTOR_VIP_DEVICE = "TP" ] 
@	then
@		echo "director is accepting packets on VIP $VIP by TP " 
@		echo "(i.e. there is no VIP device on director)." 
@	 	echo "bypassing ping check of VIP $VIP." 
@	else
@	 	echo "director is accepting packets on VIP $VIP, device $DIRECTOR_VIP_DEVICE" 
@		#you can remove VIP from a previous installation on 
@		#the wrong ethernet device by 
@		#downing the device and rmmod the driver.
@		#For VIP on lo, tunl0, you have to reboot to removed them.
@		#However for LVS you can ignore VIP 
@		#on the wrong lo device (the LVS works anyhow).
@		#will not be able to ping test VIP on director in these cases
@		NUM_VIP=`$IFCONFIG -a | $GREP -c "$VIP "`
@		let NUM_VIP="$NUM_VIP"
@		if [ $NUM_VIP -eq 0 ]
@		then 
@			#case 1.
@			echo "No VIP on real-server, VIP will be on director." 
@			#$IFCONFIG -a
@			echo ""
@			ping_test_VIP_on_director
@		else
@			#case 2.
@			echo "VIP on real-server, bypassing ping check for VIP $VIP on director."
@			#VIP can be on eth1 and will have to bypass test
@			#echo "Error: there should be no VIP on the real-server at this stage." 1>&2
@			#echo "supposedly the script has removed it." 1>&2
@			#echo "Either the script has a bug, or your setup has something unexpected." 1>&2
@			#echo "Here's the output of ifconfig. Look for the device with $VIP on it." 1>&2
@			#$IFCONFIG -a 1>&2
@			#echo ""
@			#((NUM_ERRORS++))
@		fi
@	fi
@	} #check_for_vip_on_director 
@
@restart_mon(){
@	if [ -e /var/run/mon.pid ]
@	then
@		echo " "
@		echo "mon is running - restarting "
@		$KILL -HUP `$CAT /var/run/mon.pid`
@		#presumably this can fail. mon.pid might be from a previous running of mon.
@		#unfortunately if the mon.cf file is invalid, 
@		#mon won't be restarted but kill will return "0"
@		#FIXME
@		#look to see if mon is running
@		if [ "$?" != "0" ]
@		then
@			#the system will give a message that the call failed
@			#here's another
@			echo "Error: mon restart failed" 1>&2
@			((NUM_ERRORS++))
@		fi
@	fi
@	}
@
@restart_ntpd(){
@	#if two ntpds are running (I assume by mistake), 
@	#then NTPD_PID found by a command like 
@	#NTPD_PID=`ps -auxw | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'`
@	#will have a value like "1234 1235" 
@	#and the shell "if" command will fail with too many arguements.
@	#need to kill all ntpd's and then start only one.
@
@	NTPD_FOUND="N"
@
@	#assume ps command is BSD like
@	NTPD_EXTRA_PID=`$PS auxw | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'| $XARGS | $AWK '{print $2}'`
@	while [ "$NTPD_EXTRA_PID" != "" ]
@	do 
@		echo "BSD like ps "
@		echo "killing extra ntpd process $NTPD_EXTRA_PID."
@		$KILL -TERM $NTPD_EXTRA_PID
@		NTPD_EXTRA_PID=`$PS auxw | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'| $XARGS | $AWK '{print $2}'`
@	done
@	NTPD_PID=`$PS auxw | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'`
@	if [ "$NTPD_PID" = "" ]
@	then
@		#maybe ps command is AT&T like
@		NTPD_EXTRA_PID=`$PS ef | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'| $XARGS | $AWK '{print $2}'`
@		while [ "$NTPD_EXTRA_PID" != "" ]
@		do 
@			echo "AT&T like ps "
@			echo "killing extra ntpd process $NTPD_EXTRA_PID."
@			$KILL -TERM $NTPD_EXTRA_PID
@			NTPD_EXTRA_PID=`$PS ef | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'| $XARGS | $AWK '{print $2}'`
@		done
@
@		NTPD_PID=`$PS ef | $GREP -v grep | $GREP ntpd | $AWK '{print $2}'`
@		if [ "$NTPD_PID" = "" ]
@		then
@			#ntpd is not running
@			echo "ntpd not running, won't be restarted"
@		else	
@			NTPD_PID_FOUND='Y'
@		fi
@	else
@	NTPD_PID_FOUND='Y'
@	fi
@	if [ "$NTPD_PID_FOUND" = 'Y' ]
@	then
@		echo "killing ntpd process $NTPD_PID."
@		$KILL -TERM $NTPD_PID
@		#echo "restarting ntpd."
@		$NTPD
@		if [ $? = 0 ]
@		then
@			echo "ntpd restarted"
@		else
@			echo "Error: ntpd restart failed"
@			((NUM_ERRORS++))
@		fi
@	fi
@	}
@
@test_ping()
@	{
@	#Need a ping that will only ping once and will return success or fail for the target.
@	#The newer pings do this with "ping -c 1".
@	#In earlier versions of this script, fping was used.
@	#fping is a version of ping designed for mon, which only sends one ping, 
@	#returns the string "hostname is alive" and sets $? = 0.
@	#For fping, we were only using the single ping feature of fping, 
@	#and we aren't looking for the returned string.
@
@	#Is ping OK?
@	#the old ping barfs with the "-c 1" option.
@	echo "testing ping"
@	#for testing: fake a failure in ping (give it an option it won't take)
@	#PING="/usr/local/bin/ping -x"
@
@	$PING localhost >/dev/null 2>&1
@	if [ $? = "0" ]
@	then
@		echo "ping can send one packet. is OK."
@		FPING=$PING
@		echo "\$FPING=$FPING"
@	else
@		#echo "Notice: ping -c 1 fails, looking for fping" 1>&2
@		OPTIONAL_CMDS="fping"
@		for CMD in $OPTIONAL_CMDS
@		do
@			EXECUTABLE_FOUND="N"
@			for DIRECTORY in $DIRECTORIES
@			do
@				if [ -x ${DIRECTORY}/$CMD ]
@				then
@					case "$CMD"
@					in
@						fping) 		#echo "fping"
@								FPING=${DIRECTORY}/$CMD
@								echo "\$FPING=${FPING}"
@								EXECUTABLE_FOUND="Y"
@								;;
@					esac
@				fi
@			done
@		done
@		#see if FPING set
@		if [ "$FPING" = "" ]
@		then
@			#FPING not found
@			echo "Error: ping cannot send only one packet. " 1>&2
@			echo "No fping replacement found." 1>&2
@			echo "Either get a newer version of ping or get fping." 1>&2
@			echo "fping is available from ftp://ftp.kernel.org/pub/software/admin/mon and http://www.kernel.org/software/mon/ " 
@			echo "Any recent copy of ping should be a functional replacement for fping." 
@			((NUM_ERRORS++))
@		else
@			#FPING found
@			echo "using $FPING for ping "
@		fi
@	fi
@	} #test_ping
@
@find_standard_executables(){
@	#find_standard_executables 
@	#You could hope they'd all be in the path, 
@	#or at a standard location, but they never are.
@	#I'm just going to find them.
@	#This script is meant to run on any OS so the files could be anywhere.
@	
@	#ipvsadm is going to have to be in /sbin
@	IPVSADM="/sbin/ipvsadm"
@	
@	#all directories are searched, even if executable already found.
@	#if have two versions of executable, in say /usr/bin and /usr/local/bin
@	#the version you'll get will be the last one found.
@ 
@	DIRECTORIES="/bin /usr/bin /sbin /usr/sbin /usr/local/bin /usr/local/sbin"
@
@	#need to find `echo` first, as it's needed to echo all the other variables.	
@	#Sept 2001, now using the echo that's built into bash
@	#/bin/echo "looking for standard utilities"
@	echo "looking for standard utilities"
@
@	#although finding standard executables works perfectly for me, 
@	#when run on other people's machines, things seem to go wrong.
@	#having "" as the default value for the variables means that if
@	#$IP="" instead of /usr/local/bin/ip
@	#and the command 
@	#$IP route add 
@	#is issued, then the user will see and error from the command
@	#route add 
@	#rather than an error for "ip route add"
@	#If I give IP the default value of "/bin/false", 
@	#then the user will see 
@	#false route add being run and will at least get a clue as to what's happening.
@	#The return value will be false :-)
@	
@	REQUIRED_CMDS="false"
@
@	for CMD in $REQUIRED_CMDS
@	do
@		EXECUTABLE_FOUND="N"
@		for DIRECTORY in $DIRECTORIES
@		do
@			if [ -x ${DIRECTORY}/$CMD ]
@			then
@				case "$CMD"
@				in
@					false)	#echo "false"
@						FALSE=${DIRECTORY}/$CMD
@						echo "\$FALSE=${FALSE}"
@						EXECUTABLE_FOUND="Y"
@						;;
@				esac
@			fi
@		done
@		if [ $EXECUTABLE_FOUND = "N" ]
@		then
@			/bin/echo "Notice: optional executable $CMD not found "
@			case "$CMD" in
@				false )	echo "false should be a standard executable."
@					;;
@			esac
@		fi
@	done
@
@	PING=$FALSE
@	IFCONFIG=$FALSE
@	NETSTAT=$FALSE
@	ROUTE=$FALSE
@	AWK=$FALSE
@	GREP=$FALSE
@	HOSTNAME_CMD=$FALSE
@	UNAME_CMD=$FALSE
@	CAT=$FALSE
@	TAIL=$FALSE
@	CUT=$FALSE
@	XARGS=$FALSE
@	PS=$FALSE
@	KILL=$FALSE
@	WC=$FALSE
@	TRACEROUTE=$FALSE
@	ARP=$FALSE
@	ROUTE=$FALSE
@	INSMOD=$FALSE
@	TR=$FALSE
@	EXPR=$FALSE
@	CHMOD=$FALSE
@	MV=$FALSE
@	RM=$FALSE
@	LN=$FALSE
@	MKDIR=$FALSE
@	SSH=$FALSE
@	NTPD=$FALSE
@	IP=$FALSE
@
@	REQUIRED_CMDS="ping ifconfig netstat route awk grep hostname uname cat cut tail xargs ps kill wc traceroute arp tr expr chmod mv ln rm mkdir ssh"
@
@	for CMD in $REQUIRED_CMDS
@	do
@		EXECUTABLE_FOUND="N"
@		for DIRECTORY in $DIRECTORIES
@		do
@			if [ -x ${DIRECTORY}/$CMD ]
@			then
@				case "$CMD" in
@					#echo)		#echo "echo"
@					#		ECHO=${DIRECTORY}/$CMD
@					#		$ECHO "\$ECHO=${ECHO}"
@					#		EXECUTABLE_FOUND="Y"
@					#		;;
@					ping) 		#echo "ping"
@							#"-U" causes failure on older pings 
@							#ping doesn't give you the version either :-(
@							#PING="${DIRECTORY}/$CMD -U -c 1"
@							PING="${DIRECTORY}/$CMD -c 1"
@							echo "\$PING=${PING}"
@							test_ping
@							EXECUTABLE_FOUND="Y"
@							;;
@					ifconfig)	#echo "ifconfig"
@							IFCONFIG=${DIRECTORY}/$CMD
@							echo "\$IFCONFIG=${IFCONFIG}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					netstat) 	#echo "netstat"
@							NETSTAT=${DIRECTORY}/$CMD
@							echo "\$NETSTAT=${NETSTAT}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					route) 		#echo "route"
@							ROUTE=${DIRECTORY}/$CMD
@							echo "\$ROUTE=${ROUTE}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					awk)		#echo "awk"
@							AWK=${DIRECTORY}/$CMD
@							echo "\$AWK=${AWK}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					grep)		#echo "grep"
@							GREP=${DIRECTORY}/$CMD
@							echo "\$GREP=${GREP}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					hostname)	#echo "hostname"
@							HOSTNAME_CMD=${DIRECTORY}/$CMD
@							echo "\$HOSTNAME_CMD=${HOSTNAME_CMD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					uname)		#echo "uname"
@							UNAME_CMD=${DIRECTORY}/$CMD
@							echo "\$UNAME_CMD=${UNAME_CMD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					cat)		#echo "cat"
@							CAT=${DIRECTORY}/$CMD
@							echo "\$CAT=${CAT}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					tail)		#echo "tail"
@							TAIL=${DIRECTORY}/$CMD
@							echo "\$TAIL=${TAIL}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					cut)		#echo "cut"
@							CUT=${DIRECTORY}/$CMD
@							echo "\$CUT=${CUT}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					xargs)		#echo "xargs"
@							XARGS=${DIRECTORY}/$CMD
@							echo "\$XARGS=${XARGS}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					ps)		#echo "ps"
@							PS=${DIRECTORY}/$CMD
@							echo "\$PS=${PS}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					kill)		#echo "kill"
@							KILL=${DIRECTORY}/$CMD
@							echo "\$KILL=${KILL}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					wc)		#echo "wc"
@							WC=${DIRECTORY}/$CMD
@							echo "\$WC=${WC}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					traceroute)	#echo "traceroute"
@							TRACEROUTE=${DIRECTORY}/$CMD
@							echo "\$TRACEROUTE=${TRACEROUTE}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					arp)		#echo "arp"
@							ARP=${DIRECTORY}/$CMD
@							echo "\$ARP=${ARP}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					insmod)		#echo "insmod"
@							INSMOD=${DIRECTORY}/$CMD
@							echo "\$INSMOD=${INSMOD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					tr)		#echo "tr"
@							TR=${DIRECTORY}/$CMD
@							echo "\$TR=${TR}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					expr)		#echo "expr"
@							EXPR=${DIRECTORY}/$CMD
@							echo "\$EXPR=${EXPR}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					chmod)		#echo "chmod"
@							CHMOD=${DIRECTORY}/$CMD
@							echo "\$CHMOD=${CHMOD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					mv)		#echo "mv"
@							MV=${DIRECTORY}/$CMD
@							echo "\$MV=${MV}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					rm)		#echo "rm"
@							RM=${DIRECTORY}/$CMD
@							echo "\$RM=${RM}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					ln)		#echo "ln"
@							LN=${DIRECTORY}/$CMD
@							echo "\$LM=${LN}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					mkdir)		#echo "mkdir"
@							MKDIR=${DIRECTORY}/$CMD
@							echo "\$MKDIR=${MKDIR}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					ssh)		#echo "ssh"
@							SSH=${DIRECTORY}/$CMD
@							echo "\$SSH=${SSH}"
@							EXECUTABLE_FOUND="Y"
@							;;
@				esac
@			fi
@		done
@		if [ $EXECUTABLE_FOUND = "N" ]
@		then
@			/bin/echo "Error: executable $CMD not found " 1>&2
@			((NUM_ERRORS++))
@			case "$CMD" in
@				ssh ) 	
@					echo "you need ssh if you use the -i option with configure."
@					echo "ssh is available from http://www.openssh.org " 
@					echo "you'll also need ssl from http://www.openssl.org" 
@					;;
@				* )	echo "$CMD should be in your distribution."
@					;;
@			esac
@		fi
@	done
@	
@	OPTIONAL_CMDS="ntpd ip"
@
@	for CMD in $OPTIONAL_CMDS
@	do
@		EXECUTABLE_FOUND="N"
@		for DIRECTORY in $DIRECTORIES
@		do
@			if [ -x ${DIRECTORY}/$CMD ]
@			then
@				case "$CMD"
@				in
@					ntpd)	#echo "ntpd"
@						NTPD=${DIRECTORY}/$CMD
@						echo "\$NTPD=${NTPD}"
@						EXECUTABLE_FOUND="Y"
@						;;
@					ip)	#echo "ip"
@						IP=${DIRECTORY}/$CMD
@						echo "\$IP=${IP}"
@						EXECUTABLE_FOUND="Y"
@						;;
@				esac
@			fi
@		done
@		if [ $EXECUTABLE_FOUND = "N" ]
@		then
@			/bin/echo "Notice: optional executable $CMD not found "
@			case "$CMD" in
@				ip ) 	echo "ip is a part of iproute2"
@				 	echo "at ftp://ftp.inr.ac.ru/ip_routing"
@					echo "future versions of config will require ip"
@							;;
@				ntpd ) 	
@					echo "ntp is available from http://www.ntp.org"
@					echo "ntp will keep your servers and director synchronised to within 3msec."
@					;;
@			esac
@		fi
@	done
@	} #find_standard_executables
@
@
@find_kernel_dependant_executables()
@	{
@	DIRECTORIES="/bin /usr/bin /sbin /usr/sbin /usr/local/bin /usr/local/sbin"
@	if [ $LINUX_KERNEL_SERIES = "2.2" ]
@	then
@
@		CMD="ipchains"
@		EXECUTABLE_FOUND="N"
@		for DIRECTORY in $DIRECTORIES
@		do
@			if [ -x ${DIRECTORY}/$CMD ]
@			then
@				#echo "ipchains"
@				IPCHAINS=${DIRECTORY}/$CMD
@				echo "\$IPCHAINS=${IPCHAINS}"
@				EXECUTABLE_FOUND="Y"
@			fi
@		done
@		if [ $EXECUTABLE_FOUND = "N" ]
@		then
@			echo "Notice: kernel version dependant executable $CMD not found "
@			case "$CMD" in
@				ipchains ) echo "$CMD is available from http://netfilter.samba.org"
@					;;
@			esac
@		fi
@	elif [ $LINUX_KERNEL_SERIES = "2.4" ]
@	then
@		CMD="iptables"
@		EXECUTABLE_FOUND="N"
@		for DIRECTORY in $DIRECTORIES
@		do
@			if [ -x ${DIRECTORY}/$CMD ]
@			then
@				#echo "iptables"
@				IPTABLES=${DIRECTORY}/$CMD
@				echo "\$IPTABLES=${IPTABLES}"
@				EXECUTABLE_FOUND="Y"
@			fi
@		done
@		if [ $EXECUTABLE_FOUND = "N" ]
@		then
@			echo "Notice: kernel version dependant executable $CMD not found "
@			case "$CMD" in
@				ipchains ) echo "$CMD is available from http://netfilter.samba.org"
@					;;
@			esac
@		fi
@	else
@		echo "Warning: kernel version $LINUX_KERNEL_VERSION not 2.2. or 2.4"
@		((NUM_WARNINGS++)) 
@	fi
@	} #find_kernel_dependant_executables
@
@find_module_executables()
@	{
@	#Looking for these may be rediculous, 
@	#since I hope they'll always be in one place
@	DIRECTORIES="/bin /usr/bin /sbin /usr/sbin /usr/local/bin /usr/local/sbin"
@
@	MODULE_CMDS="lsmod rmmod insmod"
@	for CMD in $MODULE_CMDS
@	do
@		EXECUTABLE_FOUND="N"
@		for DIRECTORY in $DIRECTORIES
@		do
@			if [ -x ${DIRECTORY}/$CMD ]
@			then
@				case "$CMD"
@				in
@					lsmod)		#echo "lsmod"
@							LSMOD=${DIRECTORY}/$CMD
@							echo "\$LSMOD=${LSMOD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					rmmod)		#echo "rmmod"
@							RMMOD=${DIRECTORY}/$CMD
@							echo "\$RMMOD=${RMMOD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@					insmod)		#echo "insmod"
@							INSMOD=${DIRECTORY}/$CMD
@							echo "\$INSMOD=${INSMOD}"
@							EXECUTABLE_FOUND="Y"
@							;;
@				esac
@			fi
@		done
@
@		if [ $EXECUTABLE_FOUND = "N" ]
@		then
@			echo "Error: executable $CMD not found " 1>&2
@			((NUM_ERRORS++))
@			case "$CMD" in
@				lsmod|rmmod|insmod ) echo "$CMD is available from http://www.kernel.org/pub/linux/utils/kernel/modutils"
@					;;
@			esac
@		fi
@	done
@	}
@
@find_System_map()
@	{
@	echo "find_System_map"
@	#I don't know where System.map is on most machines
@	#if System.map is in multiple locations, the last is the one used.
@	#if /usr/src/linux is the last, 
@	#then rc.system_map will have to be run first, to match it with the kernel.
@	DIRECTORIES="/ /boot /usr/src/linux"
@	FILE="System.map"
@
@	SYSTEM_MAP_FOUND="N"
@	for DIRECTORY in $DIRECTORIES
@	do
@		if [ -f ${DIRECTORY}/$FILE ]
@		then
@			echo "System.map"
@			SYSTEM_MAP=${DIRECTORY}/$FILE
@			echo "\$SYSTEM_MAP=${SYSTEM_MAP}"
@			SYSTEM_MAP_FOUND="Y"
@		fi
@	done
@
@	if [ $SYSTEM_MAP_FOUND = "N" ]
@	then
@		echo -n "Warning: System.map not found in " 1>&2
@			for DIRECTORY in $DIRECTORIES
@			do
@				echo -n "$DIRECTORY " 1>&2
@			done
@		echo "" 1>&2
@		echo "Will not be able to determine if functions are in kernel."
@		echo "Will hope they are loadable in modules."
@		((NUM_WARNINGS++))
@	fi
@	}
@
@report_errors(){
@	
@	if [ $NUM_ERRORS -ne 0 ]
@	then
@		echo "" 1>&2
@		echo "Errors: $NUM_ERRORS" 1>&2
@		echo "Some of these errors are from tests that failed." 1>&2
@		echo "If you're experimenting, it's possible that the LVS will work." 1>&2
@		echo "If you're doing production, you can't assume that the LVS will work." 1>&2
@		echo ""
@		echo "Your LVS may not be setup properly." 1>&2
@	fi
@	
@	if [ $NUM_WARNINGS -ne 0 ]
@	then
@		echo "" 1>&2
@		echo "Warnings: $NUM_WARNINGS" 1>&2
@		echo "The configure script has encountered an unexpected situation." 1>&2
@		echo "Your LVS will probably run (it may not do what you want)." 1>&2
@		echo "It would be reasonable to try your LVS first, " 1>&2
@		echo "since it will likely work at least partially. " 1>&2
@		echo "" 1>&2
@		echo "It is possible that the conf file has invalid information" 1>&2
@		echo "(or the configure script has a bug)." 1>&2
@		echo "If you suspect a bug in the script, please contact $MAINTAINER" 1>&2
@		echo "or post to the LVS mailing list $LVS_USERS" 1>&2
@		echo "(You can always run rc.lvs manually on director/real-servers.)" 1>&2
@	fi
@	let TOTAL_ERRORS="$NUM_ERRORS"+"$NUM_WARNINGS"
@	if [ "$WHOAMI" = "DIRECTOR" -a $NUM_ERRORS = "0" ]
@	then 
@		echo "Remember to run this rc.lvs script on the real-servers too."
@		echo ""
@		echo "This script will use ssh to do this for you if you run it with"
@		echo "./configure lvs_xx.conf -i"
@	fi
@	exit $TOTAL_ERRORS
@	} #report_errors
@
@install_realserver_vip_device(){
@	#I now remove_old_vip elsewhere
@	#echo "install non-arping network device on Server "
@	#echo "don't bother removing device with VIP if it's going to be the replacement "
@	#NEW_VIP_DEVICE=$SERVER_VIP_DEVICE 
@	#remove_old_vip
@	#hmm, don't know why I empty NEW_VIP_DEVICE here
@	#NEW_VIP_DEVICE=""
@
@	detect_realserver_os #installs VIP
@	echo "" 
@	}
@
@setup_director_vip(){
@	#setup VIP
@	echo "adding ethernet device and routing for VIP $VIP "
@	$IFCONFIG $DIRECTOR_VIP_DEVICE $VIP broadcast $VIP_BROADCAST netmask $VIP_NETMASK 
@	$ROUTE add -host $VIP dev $DIRECTOR_VIP_DEVICE 
@	echo "listing ifconfig info for VIP $VIP "
@	$IFCONFIG $DIRECTOR_VIP_DEVICE 
@	echo -n "checking VIP $VIP is reachable from self (director): "
@	$FPING $VIP 
@	echo "listing routing info for VIP $VIP "
@	$NETSTAT -rn | $GREP "$VIP " 
@	echo ""
@	}
@
@write_remove_vip(){
@	echo "#!/bin/bash" >rc.remove_vip
@	echo "#rc.remove_vip" >>rc.remove_vip
@	echo "#(C) 2000-1 Joseph Mack, released under GPL" >>rc.remove_vip
@	echo "#Part of the Linux Virtual Server project" >>rc.remove_vip
@	echo "#http://www.linuxvirtualserver.org/" >>rc.remove_vip
@	echo "#-----------------------------------------------------" >>rc.remove_vip
@	echo "#edit these variables," >>rc.remove_vip
@	echo "#using the perl documentation in the configure script," >>rc.remove_vip
@	echo "#(in the section CHANGING VIP BETWEEN ETH DEVICES ON DIRECTOR)" >>rc.remove_vip
@	echo "#" >>rc.remove_vip
@	echo "ETH_DEVICES=\"eth0 eth1 eth2 eth3\"" >>rc.remove_vip
@	echo "ETH_DEVICE_MODULES=\"$ETH_DEVICE_MODULES\"" >>rc.remove_vip
@	echo "START_NETWORK_COMMAND=\"/etc/rc.d/rc.inet1\"" >>rc.remove_vip
@	echo "#-----------------------------------------------------" >>rc.remove_vip
@	echo "echo \"show current network\"" >>rc.remove_vip
@	echo "$NETSTAT -rn" >>rc.remove_vip
@	echo "MODULE_FOUND=\"N\"" >>rc.remove_vip
@	echo "echo \"bring down all eth devices.\"" >>rc.remove_vip
@	echo "for DEVICE in \$ETH_DEVICES; do $IFCONFIG \$DEVICE down; done" >>rc.remove_vip
@	echo "echo \"remove ethernet module(s).\"" >>rc.remove_vip
@	echo "for MODULE in \$ETH_DEVICE_MODULES" >>rc.remove_vip
@	echo "do" >>rc.remove_vip
@	echo "        rmmod \$MODULE" >>rc.remove_vip
@	echo "        RETURN_CODE=\$?" >>rc.remove_vip
@	echo "        if [ \"\$RETURN_CODE\" = \"0\" ]" >>rc.remove_vip
@	echo "        then" >>rc.remove_vip
@	echo "                echo \"module \$MODULE removed\"" >>rc.remove_vip
@	echo "                MODULE_FOUND=\"Y\"" >>rc.remove_vip
@	echo "        elif [ \"\$RETURN_CODE\" = \"1\" ]" >>rc.remove_vip
@	echo "        then" >>rc.remove_vip
@	echo "                echo \"ignored\"" >>rc.remove_vip
@	echo "        fi" >>rc.remove_vip
@	echo "done" >>rc.remove_vip
@	echo "if [ \"\$MODULE_FOUND\" = \"N\" ]" >>rc.remove_vip
@	echo "then" >>rc.remove_vip
@	echo "        echo \"Error: cannot remove network driver modules\"" >>rc.remove_vip
@	echo "        echo \"either you have not listed the modules for your hosts in @network_driver_modules\"" >>rc.remove_vip
@	echo "        echo \"or your drivers are part of the kernel (in which case you'll have to reboot).\"" >>rc.remove_vip
@	echo "else" >>rc.remove_vip
@	echo "        echo \"re-install default network.\"" >>rc.remove_vip
@	echo "        \$START_NETWORK_COMMAND" >>rc.remove_vip
@	echo "        echo \"show default network (check that all VIPs are removed).\"" >>rc.remove_vip
@	echo "        /bin/netstat -rn" >>rc.remove_vip
@	echo "fi" >>rc.remove_vip
@	echo "" >>rc.remove_vip
@	echo "#-----------------------------------------------------" >>rc.remove_vip
@	$CHMOD 755 rc.remove_vip
@	} #write_remove_vip
@
@setup_ipvsadm_table(){
@	echo " "
@	if [ $CLEAR_IPVS_TABLES = "YES" ] 
@	then 
@		echo "clearing ipvsadm table "
@		$IPVSADM -C 
@	else 
@		echo "leaving ipvsadm table as is "
@		#$IPVSADM 
@	fi 
@	echo "installing LVS services with ipvsadm"
@	print_ipvsadm_lines 
@	#display current settings
@	echo " "
@	echo "displaying ipvsadm settings "
@	$IPVSADM 
@	echo ""
@	} 
@
@install_director_gw(){
@	#VS-NAT: 
@	#the director needs a default gw, 
@	#so that it can reply to incoming packets
@
@	#Classic VS-Tun, VS-DR 
@	#ie VIP on ethernet device, router default gw for real-servers:
@	# 
@	#reply packets to the client come from the real-servers.
@	#The real-servers have their own default gw 
@	#(some router local to them, usually the same router as for the director).
@	#Thus the director does not need a default gw.
@	#Since the VIP on the real-servers cannot reply to pings 
@	#(it's either hidden from arp requests, or blocked by the router),
@	#an outside machine shouldn't be able to connect to the VIP on the real-servers
@	#and shouldn't be able to get replies from the VIP on the director.
@	#This is a security feature.
@	#To put a default gw on the director for VS-Tun or VS-DR 
@	#will only expose the VIP to port scans.
@
@	#Non-classic VS-DR. 
@	#director is default gw for real-servers 
@	#(VIP on TP or using Julian's martian modification):
@	#
@	#For VS-DR with director as default gw for the real-servers:
@	#VS-DR can be setup accepting packets by TP or with Julian's martian modification 
@	#in this case the director needs a default gw
@	#(no-one has started messing with VS-Tun in this way yet 
@	#so don't include code to handle Julian's martian modification with VS-Tun.)
@
@	#What we do here:
@	#remove current default gw(s)
@	#VS-NAT - install default gw
@	#VS-Tun, VS-DR - do nothing
@
@	#Aug 2001, used to have logic here to detect whether to set a default gw for the director
@	#the logic is now in from_director_check_target_of_server_gw
@	#if [ $LVS_TYPE = "vs-nat" ] 
@	#then 
@	#	#note blank at the end of the IP 
@	#	DEFAULT_GW=$DIRECTOR_GW 
@	#	echo "setting director default gw to $DIRECTOR_GW for $LVS_TYPE LVS"
@	#	#echo "Setting director default gw. " 
@	#	install_default_gw 
@	#	echo " " 
@	#else 
@	#	#don't install a default gw for VS-DR, VS-Tun.
@	#	#Just remove old ones. 
@	#	echo "no director default gw installed for $LVS_TYPE LVS"
@	#	DEFAULT_GW="" 
@	#	install_default_gw 
@	#fi
@
@	install_default_gw 
@
@	} #install_director_gw
@
@del_rule_from()
@	{
@	#unfortunately you can add the same rule many times 
@	#and it will appear each time. You only need each rule once ;-(
@	#delete all rules acting on packets from $FROM_RULE_TARGET
@	#unfortunately the output of ip rule isn't the same as the input.
@	#if you're looking (with grep) for a network,
@	#you have to look for 192.168.1.0/0 (grep with 192.168.1.0)
@	#and when you're inserting it, you have to use 192.168.1.0/24
@	#
@	#as well if you have rules from SRC_IP to two DST_IPs and one of the DST_IPs=0/0
@	#these will show up with ip rule show as
@	#from src to dst
@	#and from src
@	#you have to be able to differentiate these two outputs so that you can delete both of them.
@	#the last one has no dst, so you can't grep for dst.
@	#this is worse than writing postscript :-(
@	#
@	#
@	#delete lookup 253
@	if [ `$IP rule show | $GREP -c " from all lookup 253"` = "1" ]
@	then
@		echo "deleting rule to table 253"
@		$IP rule del lookup 253	
@	fi 
@
@	if [ "$TO_RULE_TARGET" = "0/0" ]
@	then
@		#echo "deleting rule from $FROM_RULE_TARGET to default"
@		#note "lookup" at end of string
@		#output can look like
@		#100: from 192.168.1.110 lookup VIP
@		#100: from 192.168.1.11 to 192.168.1.0/24 lookup RIP
@		#need to differentiat the two "192.168.1.11" strings 
@		while [ `$IP rule show | $GREP -c "from ${FROM_RULE_TARGET%/*} lookup"` != 0 ]
@		do 
@			echo "deleting ip rule from $FROM_RULE_TARGET"
@			$IP rule del from $FROM_RULE_TARGET 
@			if [ "$?" != "0" ]
@			then
@				echo "Error: couldn't delete rule from $FROM_RULE_TARGET" 	
@				((NUM_ERRORS++))
@				exit 1
@			fi
@		done
@	else
@		#echo "deleting rule from $FROM_RULE_TARGET to $TO_RULE_TARGET"
@		while [ `$IP rule show | $GREP "to ${TO_RULE_TARGET%/*}" | $GREP -c "from ${FROM_RULE_TARGET%/*} "` != 0 ]
@		do 
@			echo "deleting ip rule from $FROM_RULE_TARGET to $TO_RULE_TARGET"
@			$IP rule del from $FROM_RULE_TARGET to $TO_RULE_TARGET
@			if [ "$?" != "0" ]
@			then
@				echo "Error: couldn't delete rule from $FROM_RULE_TARGET to $TO_RULE_TARGET"	
@				((NUM_ERRORS++))
@				exit 1
@			fi
@		done
@	fi
@	}
@
@check_rt_table_entries()
@	{
@	if [ `$GREP -c $RT_TABLE_LABEL $RT_TABLES_FILE` = "1" -a `$GREP -c "$RT_TABLE_NUMBER $RT_TABLE_LABEL" $RT_TABLES_FILE` = "1" ]
@	then
@		#there is only one entry for $RT_TABLE_LABEL and it's the correct one
@		echo "correct $RT_TABLE_LABEL entry in $RT_TABLES_FILE: `$GREP $RT_TABLE_LABEL $RT_TABLES_FILE`"
@	else
@		#wrong number of $RT_TABLE_LABEL entries, or wrong entry
@		#remove all $RT_TABLE_LABEL entries
@		echo "removing $RT_TABLE_LABEL entries from $RT_TABLES_FILE"
@		$MV $RT_TABLES_FILE $RT_TABLES_TEMP_FILE
@		#grab everything except (ie using grep -v) $TABLE_LABEL
@		#and blank lines
@		$GREP -v $RT_TABLE_LABEL $RT_TABLES_TEMP_FILE | $GREP -v "^ *$" > $RT_TABLES_FILE 
@		$RM $RT_TABLES_TEMP_FILE
@		#add correct entry
@		echo "$RT_TABLE_NUMBER $RT_TABLE_LABEL" >> $RT_TABLES_FILE
@	fi	
@	}
@
@install_priority_route()
@	{
@	#link (ie to RIP network) route or default route?
@	echo "ip route show table $TABLE_LABEL (should be empty): "
@	$IP route show table $TABLE_LABEL
@	if [ "$GW" = "" ]
@	then
@		echo "no gateway - is link route: ip route add table $TABLE_LABEL to $DST_IP dev $SRC_DEVICE from $SRC_IP"
@		$IP route add table $TABLE_LABEL to $DST_IP dev $SRC_DEVICE src $SRC_IP
@	else
@		echo "adding route to gateway $GW in table $TABLE_LABEL"
@		echo "ip route add table $TABLE_LABEL to $DST_IP via $GW dev $SRC_DEVICE from $SRC_IP"
@		$IP route add table $TABLE_LABEL to $DST_IP via $GW dev $SRC_DEVICE from $SRC_IP
@		#$IP route add table $TABLE_LABEL to $DST_IP via $GW dev $SRC_DEVICE src $SRC_IP
@	fi
@	RETURN_CODE=$?
@	echo "route add: return_code $RETURN_CODE"
@	if [ "$RETURN_CODE" = "0" ]
@	then
@		echo "ip route for table $TABLE_LABEL"
@		$IP route list table $TABLE_LABEL 
@	elif [ "$RETURN_CODE" = "2" ]
@	then
@		echo "Error: table $TABLE_LABEL already has entry: via $GW dev $SRC_DEVICE" 1>&2
@		echo "cannot overwrite entry, you'll have to delete the old one first." 1>&2
@		echo -n "current entry: "
@		$IP route list table $TABLE_LABEL 
@		((NUM_ERRORS++))
@	else
@		echo "Error: couldn't add gateway $GW for packets from $SRC_IP" 1>&2
@		((NUM_ERRORS++))
@	fi
@	}
@
@set_src_device()
@	{
@	echo "set src device for packets from VIP"
@	#if the VIP device is lo etc, 
@	#then packets with src=VIP will go out $SERVER_NET_DEVICE
@	if [ "$IS_SERVER_VIP_DEVICE_LOCAL" = "Y" ]
@	then
@		echo "packets with src=VIP are sent out $SERVER_NET_DEVICE"
@		VIP_SRC_DEVICE=$SERVER_NET_DEVICE 
@	else
@		#This give the same result as above, except the notice is different
@		#Will look into handling this some other time.
@		#eg if VIP device is eth1
@		echo "packets with src=VIP are sent out $SERVER_NET_DEVICE rather than $SERVER_VIP_DEVICE"
@		VIP_SRC_DEVICE=$SERVER_NET_DEVICE 
@	fi
@	}
@
@clear_priority_tables()
@	{
@	setup_priority_label_names
@	setup_lvs_priority_table
@	#clear route entries in tables setup for LVS
@	#
@	if [ "$CLEAR_IPVS_TABLES" = "YES" ]
@	then
@		echo "clearing priority routing table"
@		#I don't know how to delete entries one at a time,
@		#so delete everything :-(
@		#If you flush an empty table, you get an error message.
@		#Since no action is needed by the user to fix this,
@		#it doesn't seem to be an error to me ;-\
@		#The user shouldn't be presented with annoying messages when everything is OK, 
@		#this is unix after all.
@		#So check for empty table, to prevent user getting "informative" message.
@		for TABLE in $RT_TABLE_LABEL_VIP $RT_TABLE_LABEL_RIP
@		do
@			# if the table is empty then 
@			#`$IP route show table $TABLE` = "" 
@			# and it's easy to test for an empty table
@			# if however the table has contents, the output will be something like
@			# default via 192.168.1.254 dev eth0  src 192.168.2.110 
@			# make sure the LHS is inside ""
@			echo "deleting table $TABLE"
@			if [ "`$IP route show table $TABLE`" = "" ]
@			then
@				echo "ip route table $TABLE already empty"
@			else
@				echo "ip route flush table $TABLE"
@				$IP route flush table $TABLE
@			fi
@			echo "routing table $TABLE should be empty: "
@			$IP route show table $TABLE
@		done
@	else
@		echo "not clearing priority routing tables"
@	fi
@	}
@
@setup_lvs_priority_table()
@	{
@	#filename for file that holds mapping between table name and number
@	#we don't need this table if we're prepared to remember that table 200 is the VIP table etc.
@	RT_TABLES_DIR="/etc/iproute2"
@	RT_TABLES_FILE="${RT_TABLES_DIR}/rt_tables"
@	RT_TABLES_TEMP_FILE="${RT_TABLES_DIR}/temp"
@	if [ ! -d $RT_TABLES_DIR ]; then echo "creating directory $RT_TABLES_DIR"; $MKDIR $RT_TABLES_DIR; fi
@	#does the file exist?
@	if [ ! -f $RT_TABLES_FILE ]
@	then
@		#create rt_tables
@		RT_TABLE_LABEL=$RT_TABLE_LABEL_VIP
@		RT_TABLE_NUMBER=$RT_TABLE_NUMBER_VIP
@		echo "$RT_TABLE_NUMBER $RT_TABLE_LABEL" >> $RT_TABLES_FILE
@		
@		RT_TABLE_LABEL=$RT_TABLE_LABEL_RIP
@		RT_TABLE_NUMBER=$RT_TABLE_NUMBER_RIP
@		echo "$RT_TABLE_NUMBER $RT_TABLE_LABEL" >> $RT_TABLES_FILE
@	else
@		#check rt_tables
@		RT_TABLE_LABEL=$RT_TABLE_LABEL_VIP
@		RT_TABLE_NUMBER=$RT_TABLE_NUMBER_VIP
@		check_rt_table_entries
@	
@		#delete rules	
@		#because of the way iproute2 works 
@		#(you have to grep the output of the command, you just can't delete all rules in a table)
@		#you have to delete the specific rules first.
@		#if DST_IP="0/0", the dest doesn't show up in the output, so you have to use different grep rules.
@		
@		FROM_RULE_TARGET=$VIP
@		TO_RULE_TARGET="0/0"
@		#echo "deleting rule from $FROM_RULE_TARGET to $TO_RULE_TARGET"
@		del_rule_from
@	
@		RT_TABLE_LABEL=$RT_TABLE_LABEL_RIP
@		RT_TABLE_NUMBER=$RT_TABLE_NUMBER_RIP
@		check_rt_table_entries
@
@		FROM_RULE_TARGET=$RIP
@		TO_RULE_TARGET=${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS}
@		#echo "deleting rule from $FROM_RULE_TARGET to $TO_RULE_TARGET"
@		del_rule_from
@		FROM_RULE_TARGET=$RIP
@		TO_RULE_TARGET="0/0"
@		#echo "deleting rule from $FROM_RULE_TARGET to $TO_RULE_TARGET"
@		del_rule_from
@
@		echo "ip rules (RIP and VIP tables should be empty) "
@		$IP rule show
@	fi 
@	echo "route table name<->id translation table:"
@	$CAT $RT_TABLES_FILE
@	echo ""
@	}
@
@setup_priority_label_names()
@	{
@	MAIN_PRIORITY="50" 
@	VIP_PRIORITY="99"
@	RT_TABLE_LABEL_VIP="VIP"
@	RT_TABLE_NUMBER_VIP="200"
@	#RIP priority has to be the same as the entry 
@	#route add -net 192.168.1.0/24 eth0 
@	#and this is in main.
@	#originally I had this at prio=100 and all packets from the real-server
@	#were sent to the DIP (including packets to other real-servers, eg for telnet sessions)
@	#ie I couldn't telnet from real-server to real-server anymore.
@	RIP_PRIORITY="100"
@	RT_TABLE_LABEL_RIP="RIP"
@	RT_TABLE_NUMBER_RIP="201"
@	}
@
@install_rip_priority_route()
@	{
@	#In my initial setup, 
@	#In my initial setup, 
@	#thanks to Julian for help here.
@	#table main has the 2nd lowest priority, 32766.
@	#It contains the route info from $route add -net 192.168.1.0/24 dev eth0
@	#and it contains the default route.
@	#add to table main.
@	#send packets with any source directly.
@	#$IP rule add prio $MAIN_PRIORITY from 0/0 table main
@
@	#table main has the default rule, which now has a high priority.
@	#we don't want anything to have a higher priority than the LVS stuff.
@	#the default route will need to have a low priority, if we're going to have one.
@	#in this case we don't even want a default route.
@	#$IP route flush default	
@
@	#now for RIP
@	#send packets from RIP to RIP network
@	SRC_IP=$RIP
@	DST_IP="${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS}"
@	PRIORITY=$RIP_PRIORITY
@	TABLE_LABEL=$RT_TABLE_LABEL_RIP
@	echo "adding rules to $TABLE_LABEL with priority $PRIORITY from $SRC_IP to $DST_IP"
@	$IP rule add prio $PRIORITY from $SRC_IP to $DST_IP table $TABLE_LABEL
@
@	#send packets from RIP to outside world
@	#send packets from the ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} to $DEST_IP
@	SRC_IP=$RIP
@	DST_IP="0/0"
@	PRIORITY=$RIP_PRIORITY
@	TABLE_LABEL=$RT_TABLE_LABEL_RIP
@	echo "adding routing rules to $TABLE_LABEL with priority $PRIORITY from $SRC_IP to $DST_IP"
@	$IP rule add prio $PRIORITY from $SRC_IP to $DST_IP table $TABLE_LABEL
@
@	echo "showing ip rules for priority $PRIORITY"
@	$IP rule show | $GREP $PRIORITY
@
@	#setup routing rules
@	#set gw null since are link entries for local network
@	GW=""
@	set_src_device
@	SRC_DEVICE=$SERVER_NET_DEVICE
@	SRC_IP=$RIP
@	DST_IP=${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS}
@	install_priority_route
@
@	#setup routing rules
@	#set gw for default packets from RIP
@	GW=$DIP
@	set_src_device
@	SRC_DEVICE=$SERVER_NET_DEVICE
@	SRC_IP=$RIP
@	DST_IP="0/0"
@	install_priority_route
@	}
@
@install_vip_priority_route()
@	{
@	PRIORITY=$VIP_PRIORITY
@	SRC_IP=$VIP
@	DST_IP="0/0"
@	TABLE_LABEL=$RT_TABLE_LABEL_VIP
@
@	echo "adding $TABLE_LABEL routing rules with priority $PRIORITY to realserver."
@	$IP rule add prio $PRIORITY from $SRC_IP to $DST_IP table $TABLE_LABEL
@	echo "showing ip rules for prio $PRIORITY"
@	$IP rule show | $GREP $PRIORITY
@	
@	#setup routing rules
@	GW=$DEFAULT_GW
@	set_src_device
@	SRC_DEVICE=$VIP_SRC_DEVICE
@	SRC_IP=$VIP
@	install_priority_route
@	}
@
@install_realserver_priority_routing()
@	{
@	#at this stage all packets will go to the default gw=$SERVER_GW.
@	#For VS-DR, VS-Tun, it would be better would 
@	#if all packets from the RIP network went to the DIP
@	#and all packets from the VIP go to the SERVER_GW.
@	#This will make sure that only packets from the LVS services (ie the VIP)
@	#are sent to the outside world. 
@	#Packets from the RIP network should not be allowed out.
@	#These presumably will be regular network traffic between the real-servers,
@	#but as well bogus packets, like calls from the identd client on the real-servers
@	#will try to get out too. 
@	#These bogus packets should be trapped someway.
@	#Sending them to the director will stop them getting to the outside world,
@	#and the director can be setup as a sink for bogus packets.
@	#Calls from the identd client on the real-server 
@	#will need to be blocked by the director.
@	#
@	#After this is run, packets from the RIP will go to the DIP.
@	#
@	#Table labelling:
@	#ip has tables default,local and main labelled by strings
@	#Your own tables have to be numbered.
@	#The numbering doesn't have any ordering associated with it.
@	#The numbers appear to be like strings.
@	#If you want to use strings in the same way that default, main and local are used, 
@	#then you have to make a table in a file with the name /etc/iproute2/rt_table.
@	#I don't know why this can't be in memory like is done for default, main and local.
@	#I don't know why you just can't use strings to label your own tables.
@	#
@	#ip route show, doesn't show all tables, just default, main and local.
@	#if you want to show your own tables you have to give the table name too.
@	#If you are the sysadmin and don't know the names of the tables, 
@	#you may not know where the entries are coming from.
@	#
@	#Will not setup priority routing unless is VS-DR,VS-Tun
@	#However if are setting up VS-NAT when the last configuration was VS-DR
@	#then will have to erase previous entries.
@	echo "Installing priority routing"
@	echo "lvs type $LVS_TYPE, SERVER_GW $DEFAULT_GW"
@	#the following are now run earlier in the program.
@	#ie before any pinging etc is done.
@	#at the moment is only done for real-servers (in setup_realserver)
@	#setup_priority_label_names
@	#setup_lvs_priority_table
@	#clear_priority_tables
@
@	if [ "$LVS_TYPE" = "vs-nat" ]
@	then
@		echo "LVS_TYPE $LVS_TYPE, priority routing not needed"
@	elif [ "$LVS_TYPE" = "vs-dr" -o "$LVS_TYPE" = "vs-tun" ]
@	then 
@		echo "priority rules"
@		$IP rule show
@		echo "routes VIP"
@		$IP route show table VIP
@		echo "routes RIP"
@		$IP route show table RIP
@		install_rip_priority_route
@		install_vip_priority_route
@		#undo default gw since default packets are now handled by priority routing
@		echo "removing default gw"
@		$ROUTE del -net 0.0.0.0
@	
@		#Your new entries are active now.
@		#However the cache contains routes from the old tables.
@		#These routes take a while (minutes) to expire.
@		#You can help things along 
@		$IP route flush cache
@
@		echo "showing priority routing"
@		$IP rule show
@		$IP route show
@		for TABLE in $RT_TABLE_LABEL_VIP $RT_TABLE_LABEL_RIP
@		do
@			echo "routing for table $TABLE"
@			$IP route list table $TABLE
@		done
@	else
@		echo "Error: LVS_TYPE $LVS_TYPE not vs-nat, vs-tun or vs-dr"
@		((NUM_ERRORS++))
@	fi
@	}
@
@install_realserver_gw(){
@	#we should have a default gw now no matter what. 
@	DEFAULT_GW="$SERVER_GW" 
@	if [ $VIP_RIP_SAME_NETWORK = "Y" -a $LVS_TYPE = "vs-nat" ] 
@	then 
@		install_realserver_gw_for_one_net_director_is_server_gw 
@		check_realserver_default_route_nat 
@	else 
@		#VS-NAT with 2 networks, or VS-DR, VS-Tun with server_gw as a route (and not the dip).
@		#if we're setting up a real-server 
@		#and the real-server was previously configured as a 1 network setup  
@		#then it won't have a network route to the director. 
@		#When the default gw's are removed, the director connection will drop 
@		NUM_REALSERVER_NETWORK_ENTRIES=`$NETSTAT -rn | $GREP "^$REALSERVER_NETWORK " | $AWK '{print $2}' | $GREP "0.0.0.0" | $WC -l | $XARGS` 
@		let NUM_REALSERVER_NETWORK_ENTRIES="$NUM_REALSERVER_NETWORK_ENTRIES" 
@		if [ $NUM_REALSERVER_NETWORK_ENTRIES -eq 0 ]
@		then  
@			echo "adding route to real-server network $REALSERVER_NETWORK " 
@			$ROUTE add -net $REALSERVER_NETWORK netmask $REALSERVER_NETMASK dev $SERVER_NET_DEVICE 
@		fi 
@		install_default_gw 
@	fi 
@	} #install_realserver_gw 
@
@setup_realserver(){
@	echo "LVS realserver type $LVS_TYPE "
@	#original code did ping tests with the default gw/routing 
@	#as already setup at the time. This could be anything.
@	#Now setup routing and gw for the LVS and then do ping tests.
@	install_realserver_gw 
@	set_realserver_ip_forwarding
@	clear_ip_tables_chains
@	clear_priority_tables
@	#don't bother removing device with VIP, 
@	#if it's just going to be the replacement. 
@	NEW_VIP_DEVICE=$SERVER_VIP_DEVICE 
@	remove_old_vip 
@	#hmm, don't know why I null out NEW_VIP_DEVICE
@	NEW_VIP_DEVICE="" 
@	echo " "
@	check_for_DIP 
@	check_for_vip_on_director 
@	if [ $LVS_TYPE = "vs-nat" ] 
@		then 
@		echo "LVS_TYPE = $LVS_TYPE, no VIP device" 
@	else 
@		echo "LVS_TYPE = $LVS_TYPE, VIP device installed " 
@		install_realserver_vip_device 
@	fi 
@	if [ "$IP" = "" ]
@	then
@		echo "iproute2 not available: not installing priority routing"
@	else
@		#echo "not installing priority routing"
@		install_realserver_priority_routing
@	fi
@	restart_ntpd 
@	}
@	
@set_realserver_gw(){	
@	#if VS-NAT, real-server SERVER_GW is not set in conf file.
@	#
@	#if VS-DR or VS-Tun, SERVER_GW is set in conf file.
@	#It should not be the director, 
@	#(unless you're accepting packets by TP, or doing something wierd, 
@	#like using Julian's martian modification).
@	#
@	if [ $LVS_TYPE = "vs-nat" ] 
@	then 
@		#no VIP device for VS-NAT (real-servers don't have VIP for VS-NAT)
@		#ignore SERVER_VIP_DEVICE incase has been set,
@		#this should really be handled earlier 
@		SERVER_VIP_DEVICE="" 
@
@		#default gw for VS-NAT:
@		#for VS-NAT, default gw for real-server is $DIP
@		#expect conf file to have no entry for SERVER_GW 
@		#in which case $SERVER_GW will have its default='' (set at the top of this file). 
@		#It is possible that someone will find a good reason to have a different default gw.
@		#If SERVER_GW is set, then its value will be used and a warning issued.
@		if [ $SERVER_GW = "" ] 
@		then 
@			echo "server_gw not set, will set to director $DIP " 
@			SERVER_GW=$DIP 
@			echo "server default gw = $SERVER_GW " 
@		else 
@			SERVER_GW="$SERVER_GW" 
@			echo "Warning: default gw of $SERVER_GW is set in conf file for real-server. " 1>&2 
@			echo "Normally the DIP $DIP would be used and no value would be specified in the conf file. " 1>&2 
@			echo "The value set in the conf file $SERVER_GW will be used on the assumption that you know what you're doing. " 1>&2 
@			echo "If you did not mean to set a value in the conf file, please edit it and rerun the scripts. " 1>&2 
@			((NUM_WARNINGS++)) 
@		fi 
@	else 
@		#$LVS_TYPE ne nat, presumably is VS-DR or VS-Tun 
@		if [ $SERVER_GW = "" ] 
@		then 
@			echo "Error: no default gw information available for real-server. " 1>&2 
@			echo "Please specify a SERVER_GW in the conf file." 1>&2 
@			((NUM_ERRORS++)) 
@		fi 
@	fi 
@	#we should have a default gw now no matter what, 
@	#SERVER_GW="$SERVER_GW" 
@	echo "we should have a SERVER_GW=${SERVER_GW} no matter what"
@	}
@
@install_realserver_gw_for_one_net_director_is_server_gw(){
@	#install hostroute (this matches code at bottom of this part of the "if")
@	# 
@	#If you are changing the default gw for the real-server(s),  
@	#you assume that communication between the director and real-server(s) 
@	#is not dependant on the default gw. 
@	#For most setups, there will be a route of the type (here is netstat -rn output) 
@	#192.168.1.0     *               255.255.255.0   U         0 0          0 eth0 
@	#on the real-server (192.168.1.0/24 is the real-server network),  
@	#which connects the DIP to the RIPs. 
@	#In the case of a 1 network VS-NAT, this route is removed from the real-server(s), 
@	#so that the only route out will be via the default gw (i.e. the director). 
@	#In this case you have to have another route to the director, 
@	#so that the connection between real-server and director will not drop 
@	#when the real-server executes this script  
@	#and finds itself without a route to the director. 
@	#Make this other route a hostroute to the director,  
@	#which you remove after installing the working default gw. 
@
@	echo "adding temporary hostroute to DIP $DIP" 
@	$ROUTE add -host $DIP dev $SERVER_NET_DEVICE 
@	#Now it's safe to remove net route to real-server network. 
@	#If the same entry multiple times (by mistake), remove them. 
@	NUM_REALSERVER_NETWORK_ENTRIES=`$NETSTAT -rn | $GREP "^$REALSERVER_NETWORK " | $AWK '{print $2}' | $GREP "^0.0.0.0" | $WC -l | $XARGS` 
@	while [ $NUM_REALSERVER_NETWORK_ENTRIES != "0" ] 
@	do 
@		echo "removing route to real-server network $REALSERVER_NETWORK " 
@		$ROUTE del -net $REALSERVER_NETWORK netmask $REALSERVER_NETMASK dev $SERVER_NET_DEVICE 
@		NUM_REALSERVER_NETWORK_ENTRIES=`$NETSTAT -rn | $GREP "^$REALSERVER_NETWORK " | $AWK '{print $2}' | $GREP "^0.0.0.0" | $WC -l | $XARGS` 
@	done 
@		
@	install_default_gw 
@	#remove hostroute (this matches code at top of this routine) 
@	#remove hostroute to director 
@	echo "removing temporary hostroute to $DIP " 
@	$ROUTE del -host $DIP dev $SERVER_NET_DEVICE 
@	}
@
@check_for_martian_modification_patch(){
@	#returns MARTIAN_PATCH_FOUND=Y|N
@	#if Y, sets /proc flags
@	
@	echo "Checking for martian modification patch to kernel..."
@	#set flag
@	MARTIAN_PATCH_FOUND="N"
@	if [ -e /proc/sys/net/ipv4/conf/all/forward_shared ]
@	then
@		echo "kernel is patched."
@		MARTIAN_PATCH_FOUND="Y"
@		#don't know if need SHORT_DIRECTOR_VIP_DEVICE
@		for file in all $SHORT_DIP_DEVICE $SHORT_DIRECTOR_VIP_DEVICE
@		do
@			echo "1" >/proc/sys/net/ipv4/conf/${file}/forward_shared
@			if [ `$CAT /proc/sys/net/ipv4/conf/${file}/forward_shared` != "1" ]
@			then 
@				echo "Error: write to ${file}/forward_shared failed " 1>&2
@				((NUM_ERRORS++)) 
@			else
@				echo "write to $file OK"
@			fi
@		done
@	else
@		echo "Error: kernel does not have Julian's martian modification patch." 1>&2
@		echo "It's available at http://www.linuxvirtualserver.org/~julian" 1>&2
@		echo "UNCOMMENT NEXT LINE BEFORE SHIPPING OFF"
@		#((NUM_ERRORS++)) 
@		#DELETE NEXT LINE BEFORE SHIPPING
@		MARTIAN_PATCH_FOUND="Y"
@	fi
@	} #check_for_martian_modification_patch
@
@from_director_check_target_of_server_gw(){ 
@	echo "checking for valid server_gw for $LVS_TYPE LVS."
@	if [ "$LVS_TYPE" = "vs-nat" ] 
@	then 
@		#this has to be ifconfig if running from director and netstat if from real-server 
@		NUM_SERVER_GW=`${IFCONFIG} -a | $GREP "$SERVER_GW " | $AWK '{print $2}' | $WC -l | $XARGS` 
@		let NUM_SERVER_GW="$NUM_SERVER_GW" 
@		echo "NUM_SERVER_GW = $NUM_SERVER_GW" 
@		if [ $NUM_SERVER_GW -eq 1 ] 
@		then 
@			#echo "default gw "$SERVER_GW" for the $LVS_TYPE servers is on director, good " 
@			echo "default gw $SERVER_GW for the $LVS_TYPE servers is on director, good " 
@		else 
@			#echo "Warning: the default gw "$SERVER_GW" for the $LVS_TYPE servers is NOT on the director. " 1>&2 
@			echo "Warning: the default gw $SERVER_GW for the $LVS_TYPE servers is NOT on the director. " 1>&2 
@			echo "You probably do not want this - " 1>&2 
@			echo "a standard vs-$LVS_TYPE LVS will not work this way. " 1>&2 
@			((NUM_WARNINGS++)) 
@		fi 
@
@	elif [ $LVS_TYPE = "vs-tun" ] 
@	then 
@		#separate code for tun incase we start using this in a different manner 
@		if [ `$IFCONFIG -a | $GREP -c "$SERVER_GW "` = "0" ] 
@		then 
@			echo "default gw for vs-$LVS_TYPE servers is NOT on director, good " 
@		else 
@			echo "Warning: the SERVER_GW for the $LVS_TYPE servers is on the director. " 1>&2 
@			echo "You probably do not want this - " 1>&2 
@			echo "a standard $LVS_TYPE LVS will not work this way. " 1>&2 
@			((NUM_WARNINGS++)) 
@		fi
@
@	elif [ "$LVS_TYPE" = "vs-dr" ] 
@	then
@		#have 3 possibilities
@		#standard: 
@		#	packets from VIP on realserver must be sent to another host 
@		#	ie not the director, eg a router.
@		#	In this case the director will not have a default route.
@		#TP: 
@		#	the director is accepting packets for the VIP by transparent proxy.
@		#	Packets from the VIP on the realserver can be sent to the outside world
@		#	via the director, or via a router. 
@		#	If the packets go via the director, then the director needs a default route.
@		#	TP is explicitely configured in the lvs_dr.conf file 
@		#	so you already know if your setup is TP.
@		#Julian's martian modification:
@		#	For routing, this is the same as for TP, 
@		#	ie packets from the VIP on the realservers
@		#	can go via the director, which will need a default route.
@		#	The original martian modification patches ("fib" patches)
@		#	required that the director have 2 NICs.
@		#	These patches are not supported anymore (Jul 2001)
@		#	and can't be used with this configure script.
@		#	The "shared media" implementation of the martian modification
@		#	allows the director to have only 1 NIC.
@		#	The "shared media" patches are used with this configure script.	
@		#	The martian modification is not explicitely stated in the conf file.
@		#	It appears to be a normal vs_dr setup with the server_gw=dip.
@		#	You don't know if the user has made a mistake or if they have
@		#	a kernel patched for the martian modification.
@		#
@ 
@		#see which possibility we have.
@		#check whether the SERVER_GW is on director.
@		echo "director_vip_device=${DIRECTOR_VIP_DEVICE}, server_gw=${SERVER_GW}" 
@		if [ `$IFCONFIG -a | $GREP -c "$SERVER_GW "` = "0" ] 
@		then 
@			echo "default gw for the $LVS_TYPE servers is NOT on director, good "
@			#make sure there is no route from director for packets from VIP 
@			if [ "$DIRECTOR_GW" = "" ]
@			then
@				echo "DIRECTOR_GW not set, good."
@			else
@				echo "Error: DIRECTOR_GW is set to $DIRECTOR_GW" 1>&2
@				echo "It should not be set for $LVS_TYPE" 1>&2
@				echo "setting DIRECTOR_GW to null" 1>&2
@				DIRECTOR_GW=""
@				DEFAULT_GW=$DIRECTOR_GW
@				((NUM_ERRORS++)) 
@			fi
@		else 
@			echo "SERVER_GW $SERVER_GW for the $LVS_TYPE servers is on the director. " 
@			echo "This is OK when the director accepts packets on the VIP by TP " 
@			echo "or when the director has the martian modification patch. " 
@			#check that the director has a default gw
@			#and that either the director is accepting packets on the VIP by TP, 
@			#or that the director has the martian modification patch.
@
@			#set flag
@			SERVER_GW_ON_DIRECTOR_OK="N"	
@	
@			if [ $DIRECTOR_VIP_DEVICE = "TP" ] 
@			then
@				echo "director accepting packets by TP"
@			fi
@
@			#sets MARTIAN_PATCH_FOUND
@			check_for_martian_modification_patch
@
@			if [ $DIRECTOR_VIP_DEVICE = "TP" -o  "$MARTIAN_PATCH_FOUND" = "Y" ] 
@			then 
@				#is there a director_gw
@				if [ "$DIRECTOR_GW" = "" ]
@				then
@					echo "Error: DIRECTOR_GW not set." 1>&2
@					echo "needed if SERVER_GW is on director." 1>&2
@					((NUM_ERRORS++)) 
@				else
@					echo "are using TP or martian patch with SERVER_GW on director, OK"
@					SERVER_GW_ON_DIRECTOR_OK="Y"
@					DEFAULT_GW=$DIRECTOR_GW
@
@					#set ip_forward on.
@					#It will already by on if using TP
@					echo "1" >/proc/sys/net/ipv4/ip_forward
@					#check
@					if [ `$CAT /proc/sys/net/ipv4/ip_forward` = "1" ]
@					then
@						echo "ip_forward turned on for TP or martian modification"
@					else
@						echo "Error: could not turn on ip_forward for TP, martian modification"
@						((NUM_ERRORS++)) 
@					fi
@				fi
@			fi		
@		fi 
@
@	else
@		echo "Error: LVS_TYPE $LVS_TYPE not vs-nat, vs-dr or vs-tun" 1>&2
@		((NUM_ERRORS++)) 
@	fi 
@ 	} #from_director_check_target_of_server_gw 
@
@test_realserver_gw(){ 
@	#hmm, this isn't being used anymore.
@	echo "checking for valid server_gw "
@	if [ $LVS_TYPE eq "vs-nat" ] 
@	then 
@		#this has to be ifconfig for director and netstat for real-server 
@		NUM_SERVER_GW=`${NETSTAT} -rn | $GREP "$SERVER_GW " | $AWK '{print $2}' | $WC -l | $XARGS` 
@		let NUM_SERVER_GW="$NUM_SERVER_GW" 
@		echo "NUM_SERVER_GW = $NUM_SERVER_GW" 
@		if [ $NUM_SERVER_GW -eq 1 ] 
@		then 
@			echo "default gw "$SERVER_GW" for the $LVS_TYPE servers is on director, good " 
@		else 
@			echo "Warning: the default gw "$SERVER_GW" for the $LVS_TYPE servers is NOT on the director. " 1>&2 
@			echo "You probably do not want this - " 1>&2 
@			echo "a standard vs-$LVS_TYPE LVS will not work this way. " 1>&2 
@			((NUM_WARNINGS++)) 
@		fi 
@	fi 
@	if [ $LVS_TYPE = "vs-dr" ] 
@	then 
@		if [ $DIRECTOR_VIP_DEVICE = "TP" ] 
@		then 
@			#if calls to VIP on director are being accepted by transparent proxy (TP), 
@			#then the director can be the default gw for the real-servers. 
@			echo "director_vip_device=${DIRECTOR_VIP_DEVICE}, server_gw=${SERVER_GW}" 
@			if [ `$IFCONFIG -a | $GREP -c "$SERVER_GW "` = "0" ] 
@			then 
@				echo "default gw for the $LVS_TYPE servers is on the director. " 
@				echo "this is OK when accepting packets on the VIP by transparent proxy. " 
@			else 
@				echo "default gw for the $LVS_TYPE servers is NOT on director, good "
@			fi 
@		fi 
@	else 
@		#$DIRECTOR_VIP_DEVICE is a hardware device (eg ethernet card) 
@		echo "director_vip_device = $DIRECTOR_VIP_DEVICE, server_gw = $SERVER_GW " 
@		if [ `$IFCONFIG -a | $GREP -c "$SERVER_GW "` = "0" ] 
@		then 
@			echo "default gw for $LVS_TYPE servers is NOT on director, good " 
@		else 
@			echo "Notice: gw for $LVS_TYPE servers is on director."
@			echo "This is Julian's martian modification."
@			echo "and is _NOT_ the usual configuration for VS-DR."
@			echo "A standard $LVS_TYPE LVS will not work this way. " 
@			if [ "$NUMBER_DIRECTOR_NICS" = "2" ]
@			then
@				echo "director has $NUMBER_DIRECTOR_NICS NICS, OK."
@				echo "Checking for martian modification patch to kernel..."
@				echo ""
@				MARTIAN_PATCH_FOUND="Y"
@				if [ -e /proc/sys/net/ipv4/conf/all/forward_shared ]
@				then
@					for file in "all $SHORT_DIP_DEVICE"
@					do
@						echo "1" >/proc/sys/net/ipv4/${file}/forward_shared
@						if [ `$CAT /proc/sys/net/ipv4/conf/${file}/forward_shared` != "1" ]
@						then 
@							echo "Error: write to ${file}/forward_shared failed " 1>&2
@							((NUM_ERRORS++)) 
@							MARTIAN_PATCH_FOUND="N"
@						else
@							echo "write to $file OK"
@						fi
@					done
@					if [ "$MARTIAN_PATCH_FOUND" = "Y" ]
@					then
@						echo "martian modification setup OK."
@					else
@						echo "Error: setup of martian modification failed." 1>&2
@						((NUM_ERRORS++)) 
@					fi
@				else
@					echo "Error: kernel does not have Julian's forward_shared patch." 1>&2
@					echo "It's available at http://www.linuxvirtualserver.org/~julian" 1>&2
@					((NUM_ERRORS++)) 
@				fi
@			elif [ "$NUMBER_DIRECTOR_NICS" = "1" ]
@			then
@				echo "Error: director has $NUMBER_DIRECTOR_NICS NICS." 1>&2
@				echo "need 2 NICs for Julians's martian modification." 1>&2
@				((NUM_ERRORS++)) 
@			else
@				echo "Error: shouldn't get here. Director has $NUMBER_DIRECTOR_NICS NICS, should be 1 or 2" 1>&2
@				((NUM_ERRORS++)) 
@			fi
@		fi 
@	fi 
@	if [ $LVS_TYPE = "vs-tun" ] 
@	then 
@		#separate code for tun incase we start using this in a different manner 
@		if [ `$IFCONFIG -a | $GREP -c "$SERVER_GW "` = "0" ] 
@		then 
@			echo "default gw for vs-$LVS_TYPE servers is NOT on director, good " 
@		else 
@			echo "Warning: the default gw for the $LVS_TYPE servers is on the director. " 1>&2 
@			echo "You probably do not want this - " 1>&2 
@			echo "a standard $LVS_TYPE LVS will not work this way. " 1>&2 
@			echo "To work, this setup will need Julian's martian kernel modification. " 1>&2 
@			echo "or accepting packets on the VIP by transparent proxy. " 1>&2 
@			echo "Even so, this is an unusual way to run a $LVS_TYPE  LVS. " 1>&2 
@			echo "If you think you have a valid reason for doing this," 1>&2 
@			echo "let me know and I'll modify the script to accept your case. " 1>&2 
@			echo "mail me at $MAINTAINER or $LVS_USERS - thanks Joe " 1>&2 
@			((NUM_WARNINGS++)) 
@		fi 
@	fi 
@ 	} #test_realserver_gw 
@
@set_director_ip_forwarding(){
@	echo "set_director_ip_forwarding on $LVS_TYPE director (1 on, 0 off)."
@	if [ $LVS_TYPE = "vs-nat" ] 
@	then 
@		echo "set ip_forward ON for $LVS_TYPE director (1 on, 0 off)."
@		echo "1" >/proc/sys/net/ipv4/ip_forward 
@		echo -n "proc/sys/net/ipv4/ip_forward "
@		$CAT /proc/sys/net/ipv4/ip_forward 
@		if [ `$CAT /proc/sys/net/ipv4/ip_forward` != "1" ]
@		then
@			echo "Error: turn on ip_forward failed." 1>&2
@			((NUM_ERRORS++))
@		fi 
@		echo ""
@	else 
@		#note: latter will turn on ip_forward for TP and martian modification if VS-DR
@		echo "set ip_forward OFF for $LVS_TYPE director (1 on, 0 off)."
@		echo "0" >/proc/sys/net/ipv4/ip_forward 
@		echo -n "proc/sys/net/ipv4/ip_forward " 
@		$CAT /proc/sys/net/ipv4/ip_forward 
@		if [ `$CAT /proc/sys/net/ipv4/ip_forward` != "0" ]
@		then
@			echo "Error: turn off ip_forward failed." 1>&2
@			((NUM_ERRORS++))
@		fi 
@		echo ""
@	fi 
@	}
@
@set_realserver_ip_forwarding(){
@	echo "set_realserver_ip_forwarding to OFF (1 on, 0 off)."
@	echo "0" >/proc/sys/net/ipv4/ip_forward 
@	echo -n "proc/sys/net/ipv4/ip_forward " 
@	$CAT /proc/sys/net/ipv4/ip_forward 
@	if [ `$CAT /proc/sys/net/ipv4/ip_forward` != "0" ]
@	then
@		echo "Error: turn off ip_forward failed." 1>&2
@		((NUM_ERRORS++))
@	fi 
@	echo ""
@	}
@
@check_realserver_default_route_nat(){
@	echo "checking default routing for $LVS_TYPE realserver " 
@	echo "packets to director's default gw should go through director." 
@	echo "(This test will return quickly if the routing is wrong," 
@	echo "will return in about 5 secs if setup correctly," 
@	echo "and will hang if the routing is deranged.)" 
@	echo "Is director's default gw 2 hops away " 
@	echo "and is director 1 hop away on the path to the director's gw?" 
@	#flush kernel route cache 
@	$IP route flush cache 
@	# do this once to allow icmp redirects to reroute packets
@	# (if they are going to be rerouted)
@	$TRACEROUTE $DIRECTOR_GW >/dev/null 2>&1 
@	echo "traceroute output to DIRECDIRECTOR_GW $DIRECTOR_GW" 
@	$TRACEROUTE $DIRECTOR_GW 2>&1
@	# do it again, only parse the output 
@	HOPS_TO_DIRECTORS_GW=`$TRACEROUTE $DIRECTOR_GW 2>/dev/null | $WC -l` 
@	#HOPS_TO_DIRECTORS_GW is a string like "     2" 
@	#turn it into a number, operand has to be a string, so put in double quotes
@	let HOPS_TO_DIRECTORS_GW="$HOPS_TO_DIRECTORS_GW" 
@	HOPS_TO_DIRECTOR=`$TRACEROUTE $SERVER_GW 2>/dev/null | $GREP "($SERVER_GW)" | $CUT -c0-4` 
@	#HOPS_TO_DIRECTOR is a string, but doesn't have leading blanks 
@	#turn it into a number anyway 
@	let HOPS_TO_DIRECTOR="$HOPS_TO_DIRECTOR" 
@	if [ $HOPS_TO_DIRECTORS_GW -eq 2 ] && [ $HOPS_TO_DIRECTOR -eq 1 ] 
@	then 
@		echo "yes: good" 
@	else 
@	 	echo "Error: path to director's default gw does not go through the director. " 1>&2
@		echo "hops to director's gw $HOPS_TO_DIRECTORS_GW " 
@		echo "hops to director $HOPS_TO_DIRECTOR " 
@		echo "this vs-$LVS_TYPE LVS will not work." 
@		echo "you can fix this by changing the IP's, networks and routing of the LVS." 
@		echo "1. the network for the realservers must be private." 
@		echo "2. the default gw for the realservers must be the director." 
@		echo "3. a route to the director is not good enough. " 
@		echo "	The LVS won't work, the director must be the default gw." 
@		echo "4. the realservers must not have any other routes to the client." 
@		echo "(Some routing problems are fixed by rerunning the script.)" 
@		echo " " 
@		echo "To help debug the problem, here's the output of netstat -rn" 
@		$NETSTAT -rn
@		#determine realserver netmask, assume is /24 " 
@		REALSERVER_NETWORK=${SERVER_GW%.*}
@		#echo "REALSERVER_NETWORK=$REALSERVER_NETWORK"
@		#$NETSTAT -rn | $TAIL -n +3 | $CUT -c0-15 | $GREP -v "0.0.0.0"| $GREP -v "127.0.0.0" | $GREP -v -c "$REALSERVER_NETWORK" 
@		#NUMBER_OUTSIDE_ROUTES=`$NETSTAT -rn | $TAIL -n +3 | $CUT -c0-15 | $GREP -v "0.0.0.0"| $GREP -v "127.0.0.0" | $GREP -v -c "$REALSERVER_NETWORK"` 
@		OUTSIDE_ROUTES=`$NETSTAT -rn | $TAIL -n +3 | $CUT -c0-15 | $GREP -v "$REALSERVER_NETWORK " |$GREP -v "0.0.0.0"| $GREP -v "127.0.0.0" | $XARGS` 
@		#echo "OUTSIDE_ROUTES=$OUTSIDE_ROUTES" 
@		for OUTSIDE_ROUTE in $OUTSIDE_ROUTES 
@		do
@			echo "a route to the outside world is " 
@			#$NETSTAT -rn | $GREP "^192.168.2 " 
@			$NETSTAT -rn | $GREP "^$OUTSIDE_ROUTE " 
@			echo "please delete this route, if it routes to the client." 
@			echo "then rerun this script. " 
@		done 
@		echo " " 
@		((NUM_ERRORS++))
@	fi 
@	} #check_realserver_default_route_nat
@
@set_icmp_redirect_state(){
@	#setting icmp redirects (1 on, 0 off) 
@	#only turn off redirects when on one network, so only turn of dip device. 
@	echo "setting icmp redirects (1 on, 0 off) " 
@	for file in all default $SHORT_DIP_DEVICE
@	do 
@		echo "$REDIRECT_STATE" >/proc/sys/net/ipv4/conf/${file}/send_redirects
@		echo -n "/proc/sys/net/ipv4/conf/${file}/send_redirects "
@		$CAT /proc/sys/net/ipv4/conf/${file}/send_redirects
@		if [ `$CAT /proc/sys/net/ipv4/conf/${file}/send_redirects` != "$REDIRECT_STATE" ]
@		then
@			echo "Error: turn on ICMP redirects for $file failed."
@			((NUM_ERRORS++))
@		fi
@	done 
@	} #set_icmp_redirect_state
@			
@turn_off_icmp_redirects(){
@	#if [ $LVS_TYPE = "vs-nat" -a $DIRGW_RIP_SAME_NETWORK = "Y" ] 
@	#Turn of icmp redirects on director if
@	#1. dip is server_gw
@	#2. NIC with dip on director is connection to outside world 
@	#	(in practice this means is also has vip). 
@	#The consequence of this setup is 
@	#packet to 0/0 from the realserver is sent to dip
@	#director sends packet back out dip and autmatically sends icmp redirect to realserver.
@	#In this case icmp redirects must be turned off on dip.
@	#
@	if [ "$SHORT_DIRECTOR_VIP_DEVICE" = "$SHORT_DIP_DEVICE" -a "$SERVER_GW" = "$DIP" ]
@	then
@		echo "director is gw for realservers: turn off icmp redirects."
@		REDIRECT_STATE="0"
@	else
@		echo "director is not gw for realservers: leave icmp redirects on."
@		REDIRECT_STATE="1"
@	fi
@	
@	set_icmp_redirect_state
@	} #turn_off_icmp_redirects
@	
@old_turn_off_icmp_redirects(){
@	if [ $LVS_TYPE = "vs-nat" -a $DIRGW_RIP_SAME_NETWORK = "Y" ] 
@	then 
@		echo "The director gw $DIRECTOR_GW is in the real-server's network $REALSERVER_NETWORK." 
@		echo -n "Turning off ICMP redirects on director (1 on, 0 off): " 
@		echo -n "all " 
@		echo "0" >/proc/sys/net/ipv4/conf/all/send_redirects 
@		echo -n "default " 
@		echo "0" >/proc/sys/net/ipv4/conf/default/send_redirects 
@		echo -n "$SHORT_DIRECTOR_VIP_DEVICE " 
@		echo "0" >/proc/sys/net/ipv4/conf/${SHORT_DIRECTOR_VIP_DEVICE}/send_redirects 
@		#the director has the VIP on device $SHORT_DIRECTOR_VIP_DEVICE 
@		#the director has the DIP on device $SHORT_DIP_DEVICE 
@		#On a 2-NIC director, these will usually be different devices (eg eth0, eth1)
@		#On a 1 network VS-NAT, I expect the director will have 1 NIC.
@		#However if I assume this, 
@		#then the code will break for the first person who does it with 2 NICs.
@		#handle 2 NIC case. 
@		if [ $SHORT_DIRECTOR_VIP_DEVICE = $SHORT_DIP_DEVICE ] 
@		then  
@			echo ""
@			echo "One NIC director, no extra redirects to handle on director" 
@		else  
@			echo "Two NIC director, turning off redirects on $SHORT_DIP_DEVICE as well." 
@			echo "short DIP device = $SHORT_DIP_DEVICE " 
@			echo "short director VIP device = $SHORT_DIRECTOR_VIP_DEVICE " 
@			echo "0" >/proc/sys/net/ipv4/conf/${SHORT_DIP_DEVICE}/send_redirects 
@		fi  
@		echo "Showing state of icmp redirects (1 on, 0 off)." 
@		for DEVICE in all default $SHORT_DIRECTOR_VIP_DEVICE
@		do 
@			FILE="/proc/sys/net/ipv4/conf/${DEVICE}/send_redirects"
@			echo -n "$FILE"
@			$CAT $FILE
@			if [ `$CAT $FILE` != "0" ]
@			then
@				echo "Error: turn off ICMP redirects for $DEVICE failed."
@				((NUM_ERRORS++))
@			fi
@		done 
@	else 
@		echo "Not a 1 net VS-NAT. " 
@		echo "Turning on icmp redirects (1 on, 0 off) " 
@		echo "(incase were turned off in a previous installation)." 
@		for FILE in /proc/sys/net/ipv4/conf/*/send_redirects
@		do 
@			echo "1" >$FILE
@			echo -n "$FILE "
@			$CAT $FILE
@			if [ `$CAT $FILE` != "1" ]
@			then
@				echo "Error: turn on ICMP redirects for $FILE failed."
@				((NUM_ERRORS++))
@			fi
@		done 
@	fi 
@	} #turn_off_icmp_redirects
@
@load_ftp_module(){
@	echo "loading ftp module "
@	if [ $LINUX_KERNEL_SERIES = "2.0" -o $LINUX_KERNEL_SERIES = "2.2" ] 
@	then
@		if [ `$LSMOD | $AWK '{print $1}'| $GREP -c "^ip_masq_ftp"` = "0" ]
@		then
@			/sbin/insmod ip_masq_ftp 
@			if [ "$?" = "0" ]
@			then
@				echo -n ""
@			else
@				echo "Error: loading module ip_masq_ftp failed in kernel $UNAME_R " 1>&2
@ 				((NUM_ERRORS++)) 
@			fi
@		elif [ `$LSMOD | $AWK '{print $1}' | $GREP -c "^ip_masq_ftp"` = "1" ]
@		then
@			echo "ip_masq_ftp already loaded "
@		else
@			echo "Error: expecting 0 or 1 ip_masq_ftp module, found `$LSMOD | $GREP -c ip_masq_ftp` " 1>&2
@ 			((NUM_ERRORS++)) 
@		fi
@	fi
@
@	if [ $LINUX_KERNEL_SERIES = "2.4" ] 
@	then
@		if [ `$LSMOD | $AWK '{print $1}' | $GREP -c "^ip_vs_ftp"` = "0" ]
@		then
@			/sbin/insmod ip_vs_ftp 
@			if [ "$?" = "0" ]
@			then
@				echo -n ""
@			else
@				echo "Error: loading module ip_vs_ftp failed in kernel $UNAME_R " 1>&2
@ 				((NUM_ERRORS++)) 
@			fi
@		elif [ `$LSMOD | $AWK '{print $1}' | $GREP -c "^ip_vs_ftp"` = "1" ]
@		then
@			echo "ip_vs_ftp already loaded "
@		else
@			echo "Error: expecting 0 or 1 ip_vs_ftp module, found `$LSMOD | $GREP -c ip_vs_ftp` " 1>&2
@ 			((NUM_ERRORS++)) 
@		fi
@	fi
@	}
@
@find_kernel_function_name_2_2()
@	{
@	echo "find_kernel_function_name_2_2"
@	KERNEL_FUNCTION=""
@	echo "find_kernel_name_2_2: parameter $1"
@	case $1 in 
@		ip_vs )
@		KERNEL_FUNCTION="ip_vs_init"
@		;;
@		ftp )
@		#this won't find anything as the ftp module can't be built into the kernel in 2.2 
@		KERNEL_FUNCTION="ip_masq_ftp"
@		;;
@		ip_tables )
@		KERNEL_FUNCTION="ip_chains"
@		;;
@	esac
@	if [ $KERNEL_FUNCTION = "" ]
@	then
@		echo "Warning: kernel function $1 not found" 1>&2
@		((NUM_WARNINGS++))
@	fi
@	}
@
@find_kernel_function_name_2_4()
@	{
@	echo "find_kernel_function_name_2_4"
@	KERNEL_FUNCTION=""
@	echo "find_kernel_name_2_4: parameter $1"
@	case $1 in 
@		ip_vs )
@		KERNEL_FUNCTION="ip_vs_init"
@		;;
@		ftp )
@		KERNEL_FUNCTION="ip_vs_ftp_init"
@		;;
@		ip_tables )
@		KERNEL_FUNCTION="ipt_tables"
@		;;
@	esac
@	if [ "$KERNEL_FUNCTION" = "" ]
@	then
@		echo "Warning: kernel function $1 not found" 1>&2
@		((NUM_WARNINGS++))
@	fi
@	}
@
@find_module_name_2_2()
@	{
@	#echo "find_module_name_2_2"
@	MODULE=""
@	echo "find_module_name_2_2: parameter $1"
@	case $1 in 
@		ip_vs )
@		MODULE="ip_vs_rr"
@		;;
@		ftp )
@		MODULE="ip_masq_ftp"
@		MINOR_VERSION_NUMBER=${UNAME_R#2\.2\.}
@		if [ $MINOR_VERSION_NUMBER -ge 19 ]
@		then
@			#echo "MINOR_VERSION_NUMBER $MINOR_VERSION_NUMBER"
@			MODULE='ip_masq_ftp in_ports=21'
@			#echo "find_module_name_2_2: MODULE $MODULE"
@		fi
@		;;
@		ip_chains )
@		MODULE="ip_chains"
@		;;
@	esac
@	if [ "$MODULE" = "" ]
@	then
@		echo "Warning: module name $1 not found" 1>&2
@		((NUM_WARNINGS++))
@	fi
@	}
@
@find_module_name_2_4()
@	{
@	echo "find_module_name_2_4"
@	MODULE=""
@	echo "find_module_name_2_4: parameter $1"
@	case $1 in 
@		ip_vs )
@		MODULE="ip_vs"
@		;;
@		ftp )
@		MODULE="ip_vs_ftp"
@		;;
@		ip_tables )
@		MODULE="ip_tables"
@		;;
@	esac
@	if [ $MODULE = "" ]
@	then
@		echo "Warning: module name $1 not found" 1>&2
@		((NUM_WARNINGS++))
@	else
@		echo "module name $1 is $MODULE"
@	fi
@	}
@
@check_function_in_kernel()
@	{
@		echo "check_function_in_kernel"
@		if [ `$GREP -c " $KERNEL_FUNCTION" $SYSTEM_MAP` = 1 ] 
@		then
@			#in kernel
@			echo "function $KERNEL_FUNCTION in kernel."
@			echo "won't look for module."
@			IN_KERNEL="Y"
@		else
@			echo "function $KERNEL_FUNCTION not in kernel"
@		fi
@	}
@
@load_module()
@	{
@	#not in kernel
@	#echo "load_module"
@	#note blank after module name, so we dont get ip_vs_rr when looking for ip_vs 
@	#MODULE could be of the form 'foo bar' where 'bar' is a parameter.
@	#'bar' will not be seen with lsmod.
@	#must test only for 'foo' with lsmod
@	#test then for ${MODULE%% *}
@	SHORT_MODULE=${MODULE%% *}
@	echo "attempting to load module: $SHORT_MODULE"
@	#note \$ at end of module name. 
@	#this is so we'll pick up ip_vs and not pick up ip_vs_rr etc
@	if [ `$LSMOD | $AWK '{print $1}' | $GREP -c "$SHORT_MODULE\$"` = "0" ]
@	then
@		echo "function $SHORT_MODULE not in kernel, attempt to load module"
@		$INSMOD $MODULE 
@		if [ "$?" = "0" ]
@		then
@			echo "loaded OK"
@		else
@			echo "Error: loading module $MODULE failed in kernel $UNAME_R " 1>&2
@			((NUM_ERRORS++)) 
@		fi
@	elif [ `$LSMOD | $AWK '{print $1}' | $GREP -c "$SHORT_MODULE\$"` = "1" ]
@	then
@		echo "module $SHORT_MODULE already loaded "
@	else
@		echo "Warning: expecting 0 or 1 $MODULE module, found `$LSMOD |$AWK '{print $1}' | $GREP -c "$SHORT_MODULE\$\" | $AWK '{print $1}'` " 1>&2
@		((NUM_WARNINGS++)) 
@	fi
@	}
@
@load_function()
@	{
@	#function will be passed a parameter e.g. ip_vs.
@	#the function will have to figure out if the parameter 
@	#exists as a module or is compiled into the kernel.
@	#How this is done is kernel version dependant.
@
@	if [ $LINUX_KERNEL_SERIES = "2.4" ]
@	then
@		IN_KERNEL="N"
@		if [ $SYSTEM_MAP_FOUND = "Y" ]
@		then
@			find_kernel_function_name_2_4 $1
@			check_function_in_kernel
@		fi
@		MODULE_LOADED="N"
@		if [ "$IN_KERNEL" = "N" ]
@		then
@			#look for module
@			find_module_name_2_4 $1
@			load_module
@		fi
@	
@	elif [ $LINUX_KERNEL_SERIES = "2.2" ]
@	then
@		IN_KERNEL="N"
@		if [ $SYSTEM_MAP_FOUND = "Y" ]
@		then
@			find_kernel_function_name_2_2 $1
@			check_function_in_kernel
@		fi
@		MODULE_LOADED="N"
@		if [ $IN_KERNEL = "N" ]
@		then
@			#look for module
@			find_module_name_2_2 $1
@			#echo "load_function: MODULE $MODULE"
@			load_module
@		fi
@	fi 
@	} #load_function
@
@_load_ip_vs_module(){
@		
@	if [ $LINUX_KERNEL_SERIES = "2.4" ] 
@	then 
@		echo "loading ip_vs module "
@		#check if it's in kernel first
@		#note blank in front of ip_vs_init, so don't get __ip_vs_init
@		if [ `$GREP -c " ip_vs_init" $SYSTEM_MAP` = 1 ] 
@		#in kernel
@		then
@			echo "ip_vs in kernel, module not loaded"
@		else
@			#not in kernel
@			#note blank after ip_vs, so we dont get ip_vs_rr etc as well
@			#if [ `$LSMOD | $AWK '{print $1}' | $GREP -c "ip_vs "` = "0" ]
@			if [ `$LSMOD | $GREP -c "ip_vs " | $AWK '{print $1}'` = "0" ]
@			then
@				$INSMOD ip_vs 
@				if [ "$?" = "0" ]
@				then
@					echo -n ""
@				else
@					echo "Error: loading module ip_vs failed in kernel $UNAME_R " 1>&2
@	 				((NUM_ERRORS++)) 
@				fi
@			#elif [ `$LSMOD | $AWK '{print $1}' | $GREP -c "ip_vs "` = "1" ]
@			elif [ `$LSMOD | $GREP -c "ip_vs " | $AWK '{print $1}'` = "1" ]
@			then
@				echo "ip_vs already loaded "
@			else
@				echo "Error: expecting 0 or 1 ip_vs module, found `$LSMOD | $GREP -c \"ip_vs \" | $AWK '{print $1}'` " 1>&2
@	 			((NUM_ERRORS++)) 
@			fi
@		fi
@	fi 
@	}
@
@load_ip_tables_module(){
@	if [ $LINUX_KERNEL_SERIES = "2.4" ] 
@	then 
@		#must remove ipchains first
@		echo "searching for ipchains"
@		IPCHAINS_OK="Y"
@		if [ `$LSMOD | $AWK '{print $1}' | $GREP -c ipchains` = "0" ]
@		then
@			echo "ipchains not loaded, good"
@		else
@			echo "unloading ipchains module"
@			$RMMOD ipchains
@			if [ $? = "0" ]
@			then
@				echo "ipchains removed"
@			else
@				echo "Error: ipchains could not be removed" 1>&2
@				((NUM_ERRORS++))
@				IPCHAINS_OK="N"
@			fi
@		fi
@		if [ $IPCHAINS_OK = "N" ]
@		then
@			echo "expect loading of ip_tables module to fail"
@		fi
@		#looks like ip_chains doesn't need a module
@		echo "loading ip_tables module "
@		load_function ip_tables
@	elif [ $LINUX_KERNEL_SERIES = "2.2" ]
@	then
@		#looks like ip_chains doesn't need a module
@		#load_function ip_chains	
@		echo "no module needed for ip_chains"
@	else 
@		echo "Error: kernel series $LINUX_KERNEL_SERIES not 2.2 or 2.4"	
@		((NUM_ERRORS++))
@
@	#	if [ `$LSMOD | $AWK '{print $1}' | $GREP -c ip_tables` = "0" ]
@	#	then
@	#		/sbin/insmod ip_tables 
@	#		if [ "$?" = "0" ]
@	#		then
@	#			echo -n ""
@	#		else
@	#			echo "Error: loading module ip_tables failed in kernel $UNAME_R " 1>&2
@ 	#			((NUM_ERRORS++)) 
@	#		fi
@	#	elif [ `$LSMOD | $AWK '{print $1}' | $GREP -c ip_tables` = "1" ]
@	#	then
@	#		echo "ip_tables already loaded "
@	#	else
@	#		echo "Error: expecting 0 or 1 ip_tables module, found `$LSMOD | $GREP -c ip_tables` " 1>&2
@ 	#		((NUM_ERRORS++)) 
@	#	fi
@	fi 
@	}
@
@clear_policy()
@	{
@	#if you have filter rules installed, you will likely have policy = DENY
@	#and will be relying on the filter rules to ACCEPT the packets you want.
@	#If you flush rules, policy is left in place and you'll be frozen out of your machine.
@	#Before flushing rules then, you must change policy to ACCEPT 
@	#on all NICs that are involved in LVS services.
@	#unfortunately policy is global, you can't set it for individual NICs
@	#this give an error
@	#$IPCHAINS -P input -i $SHORT_DIP_DEVICE ACCEPT
@
@	echo "setting default policy to ACCEPT for LVS devices"
@	if [ $LINUX_KERNEL_SERIES = "2.2" ] 
@	then 
@		if [ "$WHOAMI" = "DIRECTOR" ] 
@		then
@			$IPCHAINS -P input ACCEPT
@			$IPCHAINS -P output ACCEPT
@			$IPCHAINS -P forward ACCEPT
@		elif [ "$WHOAMI" = "REALSERVER" ]
@		then
@			$IPCHAINS -P input ACCEPT
@			$IPCHAINS -P output ACCEPT
@			$IPCHAINS -P forward ACCEPT
@		else
@			echo "Error: not director or realserver " 1>&2
@			((NUM_ERRORS++))
@		fi
@	elif [ $LINUX_KERNEL_SERIES = "2.4" ] 
@	then 
@		if [ "$WHOAMI" = "DIRECTOR" ] 
@		then
@			$IPTABLES -P INPUT ACCEPT
@			$IPTABLES -P OUTPUT ACCEPT
@			$IPTABLES -P FORWARD ACCEPT
@		elif [ "$WHOAMI" = "REALSERVER" ]
@		then
@			$IPTABLES -P INPUT ACCEPT
@			$IPTABLES -P OUTPUT ACCEPT
@			$IPTABLES -P FORWARD ACCEPT
@		else
@			echo "Error: not director or realserver " 1>&2
@			((NUM_ERRORS++))
@		fi
@	else 
@		echo "Error: kernel series $LINUX_KERNEL_SERIES unknown" 1>&2
@ 		((NUM_ERRORS++)) 
@	fi 
@	}
@
@clear_ip_tables_chains(){
@	load_ip_tables_module
@	clear_policy
@	if [ $CLEAR_IPVS_TABLES = "YES" ]
@	then 
@		echo "clearing iptables/ipchain rules "
@		if [ $LINUX_KERNEL_SERIES = "2.0" -o $LINUX_KERNEL_SERIES = "2.2" ] 
@		then 
@			$IPCHAINS -F  
@			echo "showing ipchains rules"
@			show_ip_tables_chains
@		elif [ $LINUX_KERNEL_SERIES = "2.4" ] 
@		then 
@			$IPTABLES -F
@			$IPTABLES -F -t nat 
@			show_ip_tables_chains
@		else 
@			echo "Error: kernel series $LINUX_KERNEL_SERIES unknown" 1>&2
@ 			((NUM_ERRORS++)) 
@		fi 
@	fi
@	}
@
@show_ip_tables_chains(){
@	if [ $LINUX_KERNEL_SERIES = "2.0" -o $LINUX_KERNEL_SERIES = "2.2" ] 
@	then 
@		echo "showing iptables/ipchain rules "
@		$IPCHAINS -L 
@	elif [ $LINUX_KERNEL_SERIES = "2.4" ] 
@	then 
@		echo "showing iptables nat rules "
@		$IPTABLES -L -t nat 
@		echo "showing iptables rules "
@		$IPTABLES -L 
@	else 
@		echo "Error: kernel series $LINUX_KERNEL_SERIES unknown" 1>&2
@ 		((NUM_ERRORS++)) 
@	fi 
@	}
@
@poke_holes()
@	{
@	#I need to poke holes in the filter rules for developement.
@	#You might need to do it for production if your real-servers 
@	#need access to DNS etc eg for SMTP.
@	#In the latter case you may wan't to move the rules into setup_core_filter_rules
@	
@	
@	#For developement, director's disks nfs exported to the real-servers and clients.
@	#you won't want this for production and may not want it for your test setup.
@	#Here you poke holes in the filter rules.
@	if [ "$LINUX_KERNEL_SERIES" = "2.4" ]
@	then	
@		echo "poking holes 2.4"
@		if [ "$WHOAMI" = "DIRECTOR" ] 
@		then
@			echo ""
@		elif [ "$WHOAMI" = "REALSERVER" ]
@		then
@			echo ""
@		else
@			echo "Error: not director or realserver " 1>&2
@			((NUM_ERRORS++))
@		fi
@	elif [ "$LINUX_KERNEL_SERIES" = "2.2" ]
@	then
@		echo "poking holes 2.2"
@		if [ "$WHOAMI" = "DIRECTOR" ] 
@		then
@			#this is needed so I can nfs export to the client and real-servers
@			$IPCHAINS -I output 1 -p udp --source-port nfs -i $SHORT_DIP_DEVICE -j ACCEPT
@			$IPCHAINS -I output 1 -p udp --source-port nfs -i $SHORT_DIRECTOR_VIP_DEVICE -j ACCEPT
@			#allow telnet within real-server network
@			$IPCHAINS -I input 1 -p tcp -d $DIP telnet -i $SHORT_DIP_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@			$IPCHAINS -I output 1 -p tcp -s $DIP telnet -i $SHORT_DIP_DEVICE -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@			
@			#allow ssh within real-server network
@			$IPCHAINS -I input 1 -p tcp -d $DIP ssh -i $SHORT_DIP_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@			$IPCHAINS -I output 1 -p tcp -s $DIP ssh -i $SHORT_DIP_DEVICE -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT
@
@		elif [ "$WHOAMI" = "REALSERVER" ]
@		then
@			#this is needed so I can nfs export to the client and real-servers
@			$IPCHAINS -I output 1 -p udp --source-port nfs -i $SERVER_NET_DEVICE -j ACCEPT
@
@			#allow telnet within real-server network
@			$IPCHAINS -I input 1 -p tcp -d $RIP telnet -i $SERVER_NET_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT  
@			$IPCHAINS -I output 1 -p tcp -s $RIP telnet -i $SERVER_NET_DEVICE -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@
@			#allow ssh within real-server network
@			$IPCHAINS -I input 1 -p tcp -d $RIP ssh -i $SERVER_NET_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@			$IPCHAINS -I output 1 -p tcp -s $RIP ssh -i $SERVER_NET_DEVICE -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@		else
@			echo "Error: not director or realserver " 1>&2
@			((NUM_ERRORS++))
@		fi
@	else
@		echo "Error: linux kernel series $LINUX_KERNEL_SERIES not 2.2 or 2.4"
@ 		((NUM_ERRORS++)) 
@	fi
@	}
@
@setup_core_filter_rules_2_2()
@	{
@
@	# PREROUTING chain.
@	# Do some checks for obviously spoofed IP's 
@	#$IPCHAINS -t nat -A PREROUTING -i $SHORT_DIRECTOR_VIP_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS -j DROP
@	
@	# INPUT chain
@	# establish the basic INPUT chain and filter the packets onto the correct chains.
@	#$IPCHAINS -A INPUT -p ALL -i $LAN_IFACE -d $LAN_BCAST_ADRESS -j ACCEPT
@	#$IPCHAINS -A INPUT -p ALL -d $LOCALHOST_IP -j ACCEPT
@	#$IPCHAINS -A INPUT -p ALL -d $LAN_IP -j ACCEPT
@	#$IPCHAINS -A INPUT -p ALL -d $STATIC_IP -m state --state ESTABLISHED,RELATED -j ACCEPT
@	#$IPCHAINS -A INPUT -m limit --limit 3/minute --limit-burst 3 -j LOG --log-level DEBUG --log-prefix "IPT INPUT packet died: "
@	
@	# create user-defined (input) chains
@	# Have to declare them before can add to them.
@	# user-defined chains aren't deleted when -F'ed.
@	# there doesn't seem to be any script way of deleting user-defined chains.
@	# you have to do iptables -L and pick the names out one by one and delete them by hand.
@	# If you've done previous runs of this script, 
@	# you'll get an "already exists" error trying to create these user-defined chains.
@	# If you try to delete them and they don't exist you'll get an error too.
@	# If you redirect the error message to /dev/null, 
@	# you won't know when you've got a real error. 
@	# There's no way of doing this that won't get you an error message ;-/
@	# with ipchains, rule names must be 8 char or less (BASIC anyone?)
@	$IPCHAINS -N lvs_r >/dev/null 2>&1
@	$IPCHAINS -N lvs_s >/dev/null 2>&1
@	$IPCHAINS -N icmp_s >/dev/null 2>&1
@
@	#flush rules
@	if [ $CLEAR_IPVS_TABLES = "YES" ]
@	then
@		echo "clearing ipchains rules"
@		$IPCHAINS -F lvs_r
@		$IPCHAINS -F lvs_s
@		$IPCHAINS -F icmp_s
@	else
@		echo "not clearing ipchains rules"
@	fi
@
@	# "lvs_r*" chains.
@	#
@	# acceptance rules for a packet sent to the LVS.
@	# for a VIP based LVS, this be based on the interface and the VIP.
@	# for a fwmark based LVS, it will be more complicated.
@	# Since no connection is being made to the director 
@	# and the packets are just being routed to the real-servers,
@	# then we don't look at connection state (eg ESTABLISHED).
@	#  
@	if [ "$WHOAMI" = "DIRECTOR" ] 
@	then
@		if [ $LVS_TYPE = "vs-nat" ]
@		then
@			#for VS-NAT, there need to be input and output rules 
@			#for the device carrying the VIP and the device carrying the DIP.
@			#it's too confusing to have all these rule sets in one chain.
@			#for VS-NAT, for VIP device
@			$IPCHAINS -N lvs_r_v >/dev/null 2>&1
@			#for VS-NAT, for DIP device
@			$IPCHAINS -N lvs_r_d >/dev/null 2>&1
@			$IPCHAINS -N lvs_s_v >/dev/null 2>&1
@			$IPCHAINS -N lvs_s_d >/dev/null 2>&1
@			if [ $CLEAR_IPVS_TABLES = "YES" ]
@				then
@				echo "clearing VS-NAT ipchains rules"
@				$IPCHAINS -F lvs_r_v
@				$IPCHAINS -F lvs_r_d
@				$IPCHAINS -F lvs_s_v
@				$IPCHAINS -F lvs_s_d
@			else
@				echo "not clearing ipchains rules"
@			fi
@
@			#packets arriving for the VIP on the VIP device
@			$IPCHAINS -A input -i $SHORT_DIRECTOR_VIP_DEVICE -s ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d $VIP/${VIP_NETMASK_CLASSLESS} -j lvs_r_v
@			#packets leaving from the VIP on the VIP device
@			$IPCHAINS -A output -i $SHORT_DIRECTOR_VIP_DEVICE -d ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -s $VIP/${VIP_NETMASK_CLASSLESS} -j lvs_r_v
@			#packets leaving the DIP device headed for the real-servers
@			$IPCHAINS -A output -i $SHORT_DIP_DEVICE -s ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -j lvs_r_d
@			#packets arriving on the DIP device returning from the real-servers
@			$IPCHAINS -A input -i $SHORT_DIP_DEVICE -d ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -j lvs_r_d
@
@			#allow auth from realservers
@			$IPCHAINS -A output -i $SHORT_DIP_DEVICE -p TCP -s ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} auth -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} 1025: -j ACCEPT 
@		elif [ $LVS_TYPE = "vs-dr" ]
@		then	
@			# packets for the VIP arriving on the VIP interface from the outside,
@			# go to the LVS chain.
@			$IPCHAINS -A input -i $SHORT_DIRECTOR_VIP_DEVICE -d ${VIP}/${VIP_NETMASK_CLASSLESS} -j lvs_r
@		elif [ $LVS_TYPE = "vs-tun" ]
@		then	
@			echo ""
@		else
@			echo "Error: LVS_TYPE $LVS_TYPE not vs-nat, vs-tun or vs-dr" 1>&2
@			((NUM_ERRORS++))
@		fi
@		#get ntp packets from the device carrying the IP, which corresponds 
@		#to the hostname, to itself.
@		#this rule allows ntp from anywhere to anywhere, 
@		#rather than from ethernet devices to itself only, but it's a start
@		#$IPCHAINS -A input -p udp --source-port ntp --destination-port ntp -j ACCEPT
@		$IPCHAINS -A input -p udp -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} ntp -d ${DIP}/32 ntp -j ACCEPT
@		$IPCHAINS -A input -p udp -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} ntp -s ${DIP}/32 ntp -j ACCEPT
@
@		#director makes calls DIP:ntp to DIP:ntp
@		#these packets are local (ie don't go through the SHORT_DIP_DEVICE eg eth0)
@		#even though they are to/from the DIP.
@		$IPCHAINS -A input -p udp -s $DIP ntp -d $DIP ntp -j ACCEPT
@		
@		#allow anything within the real-server network
@		# I had "-i $FILTERED_DEVICE" here as well, 
@		# but the ntp from DIP:123 to DIP:123 was blocked
@		$IPCHAINS -A input -i $SHORT_DIP_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -j ACCEPT  
@
@		
@	elif [ "$WHOAMI" = "REALSERVER" ]
@	then
@		if [ $LVS_TYPE = "vs-nat" ]
@		then
@			# packets for the RIP arriving on the SERVER_NET_DEVICE 
@			# which are not from the real-server network 
@			# are presumably from the outside world and go to the LVS chain.
@			$IPCHAINS -A input -i $SERVER_NET_DEVICE -s ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d ${RIP}/32 -j lvs_r
@			$IPCHAINS -A output -i $SERVER_NET_DEVICE -d ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -s ${RIP}/32 -j lvs_r
@			#allow calls to client:authd
@			$IPCHAINS -A lvs_r -p TCP -i $SERVER_NET_DEVICE -d ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} auth -s ${RIP}/32 -j ACCEPT
@			$IPCHAINS -A lvs_r -p TCP -i $SERVER_NET_DEVICE -s ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} auth -d ${RIP}/32 -j ACCEPT
@			$IPCHAINS -A lvs_r -p TCP -j lvs_s
@			$IPCHAINS -A lvs_r -p UDP -j lvs_s
@
@		elif [ $LVS_TYPE = "vs-dr" ]
@		then	
@			# packets for the VIP arriving on the SERVER_NET_DEVICE 
@			# go to the LVS chain.
@			# returning packets go via router and aren't seen here.
@			$IPCHAINS -A input -i $SERVER_NET_DEVICE -d ${VIP}/${VIP_NETMASK_CLASSLESS} -j lvs_r
@			$IPCHAINS -A lvs_r -p TCP -j lvs_s
@			$IPCHAINS -A lvs_r -p UDP -j lvs_s
@		elif [ $LVS_TYPE = "vs-tun" ]
@		then	
@			#haven't tested this with VS-Tun yet
@			# IPIP packets for the VIP are sent to the RIP.
@			# after decapsulation, the packets for the VIP are delivered locally.
@			$IPCHAINS -A input -i $SERVER_NET_DEVICE -d ${RIP}/32 -j lvs_r
@			$IPCHAINS -A input -d ${VIP}/${VIP_NETMASK_CLASSLESS} -j lvs_r
@		else
@			echo "Error: LVS_TYPE $LVS_TYPE not vs-nat, vs-tun or vs-dr" 1>&2
@			((NUM_ERRORS++))
@		fi
@	
@		#icmp 
@		$IPCHAINS -A input -p icmp --icmp-type 3 -j ACCEPT
@
@		#realservers make calls RIP:ntp to RIP:ntp
@		#these packets are local (ie don't go through the SERVER_NET_DEVICE eg eth0)
@		#even though they are to/from the RIP.
@		$IPCHAINS -A input -p udp -s $RIP ntp -d $RIP ntp -j ACCEPT
@		$IPCHAINS -A input -p udp -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} ntp -d ${RIP}/32 ntp -j ACCEPT
@		$IPCHAINS -A input -p udp -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} ntp -s ${RIP}/32 ntp -j ACCEPT
@		
@		# 127.0.0.1 makes calls to 127.0.0.1:53. 
@		# I assume these are reverse DNS lookups for identd.
@		# If this is so then maybe they should be blocked.
@		# identd calls to the outside world are blocked,
@		# so the problem is partly covered.
@		$IPCHAINS -A input -p tcp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@		$IPCHAINS -A input -p udp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@		
@		#allow anything within the real-server network
@		# I had "-i $FILTERED_DEVICE" here as well, 
@		# but the ntp from RIP:123 to RIP:123 was blocked
@		$IPCHAINS -A input -i $SERVER_NET_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -j ACCEPT  
@
@	fi
@	# udp and tcp packets arriving in chain lvs_r (lvs_rules)
@	# are sent to chain lvs_s (lvs_services).
@	# icmp  packets arriving in chain lvs_r (lvs_rules) are sent to chain icmp_s (icmp_services).
@	# icmp isn't a "service" but LVS does have to handle icmp packets.
@	# Packets which aren't accepted by lvs_s or icmp_s are 
@	# returned here (and rejected and logged).
@	# packets which aren't tcp,udp,icmp are rejected (and logged).
@	# other protocols (from /etc/protocols)
@	#ip      0       IP      # internet protocol, pseudo protocol number
@	#icmp    1       ICMP    # internet control message protocol
@	#igmp    2       IGMP    # internet group multicast protocol
@	#ggp     3       GGP     # gateway-gateway protocol
@	#tcp     6       TCP     # transmission control protocol
@	#pup     12      PUP     # PARC universal packet protocol
@	#udp     17      UDP     # user datagram protocol
@	#idp     22      IDP     # WhatsThis?
@	#raw     255     RAW     # RAW IP interface
@	$IPCHAINS -A lvs_r -p ICMP -j icmp_s
@	$IPCHAINS -A lvs_r -j REJECT -l
@	
@	# rules for lvs packets of type icmp.
@	# 0 echo-reply 			required by ping (we're receiving a ping)
@	# 3 destination-unreachable	required by any tcp/udp traffic
@	# 5 redirect			required by routing if not running routing demon
@	# 8 echo-request		required by ping (we're sending a ping)
@	# 11 time-exceeded		required by traceroute
@	# even though I already know that the packets are of type icmp at this stage,
@	# ipchains barfs if I don't have "-p icmp" in the command.
@	$IPCHAINS -A icmp_s -p icmp --icmp-type 0 -j ACCEPT
@	$IPCHAINS -A icmp_s -p icmp --icmp-type 3 -j ACCEPT
@	$IPCHAINS -A icmp_s -p icmp --icmp-type 5 -j ACCEPT
@	$IPCHAINS -A icmp_s -p icmp --icmp-type 11 -j ACCEPT
@	#packets not accepted here are rejected in lvs_r
@	
@	# rules for lvs services
@	# (none)
@	#the script elsewhere will add/delete rules for each tcp service by running commands like 
@	#$IPCHAINS -A lvs_s -p TCP -s 0/0 --dport 80 -j ACCEPT
@	#$IPCHAINS -I lvs_s 1 -p UDP -s 0/0 --dport 53 -j ACCEPT
@	# identd calls may or may not make it out with VS-NAT (if you set it up).
@	# with VS-Tun/VS-DR there is no hope.
@	# Turn this on if you've masqueraded the identd calls
@	# if [ $LVS_TYPE = "vs-nat" ]
@	# then
@	#	echo "allowing identd replies from client"	
@	#	$IPCHAINS -A lvs_s -p TCP -s 0/0 --source-port identd -j ACCEPT	
@	# fi
@	# deny and log
@	# LVS packets not accepted here are rejected in lvs_r
@
@	#non LVS packets on the input chain
@	#
@	#director input chain has ntp packets of type 127.0.0.1:ntp 127.0.0.1:X and
@	#a corresponding packet of 127.0.0.1:X -> 127.0.0.1:ntp
@	# real-server 127.0.0.1 makes calls to 127.0.0.1:123 but not the other way.
@	$IPCHAINS -A input -p udp -s 127.0.0.1 ntp -d 127.0.0.1 -j ACCEPT
@	$IPCHAINS -A input -p udp -d 127.0.0.1 ntp -s 127.0.0.1 -j ACCEPT
@		
@	#local icmp
@	$IPCHAINS -A input -p icmp --icmp-type 3 -s 127.0.0.1 -d 127.0.0.1 -j ACCEPT
@
@	# 127.0.0.1 makes calls to 127.0.0.1:53 on the input chain. 
@	#I assume this is the resolver at work.
@	#I've only seen udp packets, but could presumably try a zone transfer (tcp)
@	$IPCHAINS -A input -p tcp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@	$IPCHAINS -A input -p udp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@	#
@	#what else is going on in the input chain
@	$IPCHAINS -A input -l
@
@	# default rules, should run these after rules for all interfaces have been setup
@	# logging doesn't work here. 
@	# put logging into separate rules
@	#$IPCHAINS -P input REJECT
@	$IPCHAINS -P input ACCEPT
@
@
@	# OUTPUT chain
@	#
@	# output chain, this covers packets originating in the box 
@	# and going outside (to the real-servers and to the internet).
@	# for VS-DR/VS-Tun, packets can't go out from the VIP on the director.
@	# for VS-NAT, packets _do_ go out from the VIP on the director.
@	#
@	# 127.0.0.1 makes calls to 127.0.0.1:53 on the output chain. 
@	#I assume this is the resolver at work.
@	#I've only seen udp packets, but could presumably try a zone transfer (tcp)
@	$IPCHAINS -A output -p tcp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@	$IPCHAINS -A output -p udp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@	#
@	#dest reachable at least on the director, put it global
@	$IPCHAINS -A output -p icmp --icmp-type 3 -j ACCEPT
@	
@	#
@	#get ntp packets from the main IP on each ethernet device to itself
@	#this allows ntp from anywhere to anywhere, 
@	#rather than from ethernet devices to itself only, but it's a start
@	$IPCHAINS -A output -p udp --source-port ntp --destination-port ntp -j ACCEPT
@
@	#realserver output chain has ntp packets of type 127.0.0.1:ntp 127.0.0.1:X and
@	#a corresponding packet of 127.0.0.1:X -> 127.0.0.1:ntp
@	$IPCHAINS -A output -p udp -s 127.0.0.1 ntp -d 127.0.0.1 -j ACCEPT
@	$IPCHAINS -A output -p udp -d 127.0.0.1 ntp -s 127.0.0.1 -j ACCEPT
@
@
@	if [ "$WHOAMI" = "DIRECTOR" ] 
@	then
@		#if VS-DR/VS-Tun, only output is to real-server
@		#don't allow anything to internet
@		# FILTERED_DEVICE is the VIP_DEVICE
@		# $IPCHAINS -A output -i $FILTERED_DEVICE -d ${FILTERED_IP}/${FILTERED_NETMASK_CLASSLESS} -j REJECT -l
@		# accept packets within the realserver network 
@		$IPCHAINS -A output -i $SHORT_DIP_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -j ACCEPT  
@	
@
@	elif [ "$WHOAMI" = "REALSERVER" ]
@	then
@		#
@		# 127.0.0.1 makes calls to 127.0.0.1:53. I assume this is the resolver at work.
@		$IPCHAINS -A output -p tcp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@		$IPCHAINS -A output -p udp -s 127.0.0.1 -d 127.0.0.1 dns -j ACCEPT
@		
@		# accept packets within the realserver network 
@		$IPCHAINS -A output -i $SERVER_NET_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -d ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS} -j ACCEPT  
@	
@		if [ $LVS_TYPE = "vs-nat" ]
@		then
@			# accept packets leaving from RIP destined for the outside world
@			# should include RIP:port
@			$IPCHAINS -A output -i $SERVER_NET_DEVICE -s ${RIP}/32 -d ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@
@		elif [ $LVS_TYPE = "vs-dr" ]
@		then	
@			# accept packets leaving from VIP destined for the outside world
@			# should include VIP:port
@			$IPCHAINS -A output -i $SERVER_NET_DEVICE -s ${VIP}/${VIP_NETMASK_CLASSLESS} -d ! ${REALSERVER_NETWORK}/${REALSERVER_NETMASK} -j ACCEPT 
@
@		elif [ $LVS_TYPE = "vs-tun" ]
@		then	
@			# accept packets leaving from VIP 
@			$IPCHAINS -A output -i $SEVER_NET_DEVICE -s ${VIP}/${VIP_NETMASK_CLASSLESS} -j ACCEPT 
@		else
@			echo "Error: LVS_TYPE $LVS_TYPE not vs-nat, vs-tun or vs-dr" 1>&2
@			((NUM_ERRORS++))
@		fi
@	
@	fi
@
@	#log in case I've missed anything
@	$IPCHAINS -A output -l
@	#default policy
@	#$IPCHAINS -P output REJECT
@	$IPCHAINS -P output ACCEPT
@
@	# establish the basic OUTPUT chain and filter them onto the correct chain
@	#
@	#$IPCHAINS -A OUTPUT -p ALL -s $LAN_IP -j ACCEPT
@	#$IPCHAINS -A OUTPUT -p ALL -s $STATIC_IP -j ACCEPT
@	#$IPCHAINS -A OUTPUT -m limit --limit 3/minute --limit-burst 3 -j LOG --log-level DEBUG --log-prefix "IPT OUTPUT packet died: "
@
@	# forward chain, this covers packets travelling inside->outside and outside->inside
@	# just monitor for the moment 
@	#forward chain, seems to be pretty simple for both director and realservers.
@	$IPCHAINS -A forward -j REJECT -l
@	$IPCHAINS -P forward REJECT
@
@	} #setup_core_filter_rules_2_2
@
@setup_core_filter_rules_2_4()
@	{
@	# default rules, should run these after rules for all interfaces have been setup
@	#$IPTABLES -P INPUT REJECT
@	#$IPTABLES -P OUTPUT REJECT
@	#$IPTABLES -P FORWARD REJECT
@	
@	# create user-defined chains
@	# the LVS chains. Have to declare them before can add to them.
@	# user-defined chains aren't deleted when -F'ed.
@	# there doesn't seem to be any script way of deleting user-defined chains.
@	# you have to do iptables -L and pick the names out one by one and delete them by hand.
@	# So if you've done previous runs of this script, 
@	# you'll get an "already exists" error trying to create these user-defined chains.
@	# If you try to delete them and they don't exist you'll get an error too.
@	# if you redirect the error message to /dev/null, 
@	# you won't know when you've got a real error. 
@	# There's no way of doing this that won't get you an error message ;-/
@	$IPTABLES -N lvs_rules >/dev/null 2>&1
@	$IPTABLES -N lvs_services >/dev/null 2>&1
@	$IPTABLES -N icmp_services >/dev/null 2>&1
@
@	#flush rules
@	if [ $CLEAR_IPVS_TABLES = "YES" ]
@	then
@		echo "clearing filter rules"
@		$IPTABLES -F lvs_rules
@		$IPTABLES -F lvs_services
@		$IPTABLES -F icmp_services
@	else
@		echo "not clearing filter rules"
@	fi
@
@	# send packets to the "lvs_rules" chain.
@	#
@	# acceptance rules for a packet sent to the LVS.
@	# for a VIP based LVS, this be based on the interface and the VIP.
@	# for a fwmark based LVS, it will be more complicated.
@	# Since no connection is being made to the director 
@	# and the packets are just being routed to the real-servers,
@	# then we don't look at connection state (eg ESTABLISHED).
@	#  
@	# packets for the VIP arriving on the VIP interface go to the LVS chain.
@	$IPTABLES -A INPUT -i $FILTERED_DEVICE -d ${FILTERED_IP}/${FILTERED_NETMASK_CLASSLESS} -j lvs_rules
@	
@	# lvs_services (udp and tcp) are handled by chain lvs_services
@	# lvs packets of type icmp are sent to icmp_services.
@	# icmp isn't a "service" but LVS does handle icmp packets.
@	# packets not accepted by lvs_services or icmp_services are rejected here.
@	$IPTABLES -A lvs_rules -p TCP -j lvs_services
@	$IPTABLES -A lvs_rules -p UDP -j lvs_services
@	$IPTABLES -A lvs_rules -p ICMP -j icmp_services
@	$IPTABLES -A lvs_rules -j REJECT 
@	
@	# rules for lvs packets of type icmp
@	# even though I already know that the packets are of type icmp at this stage,
@	# iptables barfs if I don't have "-p icmp" in the command.
@ 	#
@       # packets which aren't tcp,udp,icmp are rejected and logged.
@       # other protocols (from /etc/protocols)
@       #ip      0       IP      # internet protocol, pseudo protocol number
@       #icmp    1       ICMP    # internet control message protocol
@       #igmp    2       IGMP    # internet group multicast protocol
@       #ggp     3       GGP     # gateway-gateway protocol
@       #tcp     6       TCP     # transmission control protocol
@       #pup     12      PUP     # PARC universal packet protocol
@       #udp     17      UDP     # user datagram protocol
@       #idp     22      IDP     # WhatsThis?
@       #raw     255     RAW     # RAW IP interface
@	
@	$IPTABLES -A icmp_services -p icmp --icmp-type 0 -j ACCEPT
@	$IPTABLES -A icmp_services -p icmp --icmp-type 3 -j ACCEPT
@	$IPTABLES -A icmp_services -p icmp --icmp-type 5 -j ACCEPT
@	$IPTABLES -A icmp_services -p icmp --icmp-type 11 -j ACCEPT
@	$IPTABLES -A icmp_services -m limit --limit 3/minute --limit-burst 3 -j LOG --log-level DEBUG --log-prefix "icmp reject: "
@       #packets not accepted here are rejected in lvs_rules.
@       
@	# rules for lvs services
@	#the script elsewhere will add/delete rules for each tcp service by running commands like 
@	#$IPTABLES -A lvs_services -p TCP -s 0/0 --dport 80 -j ACCEPT
@	#$IPTABLES -I lvs_services -p UDP -s 0/0 --dport 53 -j ACCEPT
@       # identd calls may or may not make it out with VS-NAT (if you set it up).
@       # with VS-Tun/VS-DR there is no hope.
@       # Turn this on if you've masqueraded the identd calls
@       # if [ $LVS_TYPE = "vs-nat" ]
@       # then
@       #       echo "allowing identd replies from client"     
@       #       $IPTABLES -A lvs_services -p TCP -s 0/0 --source-port identd -j ACCEPT 
@       # fi
@       # log packets not accepted here
@	$IPTABLES -A lvs_services -m limit --limit 3/minute --limit-burst 3 -j LOG --log-level DEBUG --log-prefix "lvs_services reject: "
@       #packets not accepted here are rejected in lvs_rules
@
@	# PREROUTING chain.
@	# Do some checks for obviously spoofed IP's 
@	#$IPTABLES -t nat -A PREROUTING -i $SHORT_DIRECTOR_VIP_DEVICE -s ${REALSERVER_NETWORK}/${REALSERVER_NETMASK_CLASSLESS -j DROP
@	
@	# INPUT chain
@	#
@	# establish the basic INPUT chain and filter the packets onto the correct chains.
@	#	
@	#$IPTABLES -A INPUT -p ALL -i $LAN_IFACE -d $LAN_BCAST_ADRESS -j ACCEPT
@	#$IPTABLES -A INPUT -p ALL -d $LOCALHOST_IP -j ACCEPT
@	#$IPTABLES -A INPUT -p ALL -d $LAN_IP -j ACCEPT
@	#$IPTABLES -A INPUT -p ALL -d $STATIC_IP -m state --state ESTABLISHED,RELATED -j ACCEPT
@	#$IPTABLES -A INPUT -m limit --limit 3/minute --limit-burst 3 -j LOG --log-level DEBUG --log-prefix "IPT INPUT packet died: "
@	
@	#
@	# OUTPUT chain
@	#
@	# establish the basic OUTPUT chain and filter them onto the correct chain
@	#
@	
@	#$IPTABLES -A OUTPUT -p ALL -s $LOCALHOST_IP -j ACCEPT
@	#$IPTABLES -A OUTPUT -p ALL -s $LAN_IP -j ACCEPT
@	#$IPTABLES -A OUTPUT -p ALL -s $STATIC_IP -j ACCEPT
@	#$IPTABLES -A OUTPUT -m limit --limit 3/minute --limit-burst 3 -j LOG --log-level DEBUG --log-prefix "IPT OUTPUT packet died: "
@	} #setup_core_filter_rules_2_4
@
@whats_my_ip()
@	{
@	#what's my IP
@	MY_NETWORK=$REALSERVER_NETWORK
@	MY_NETMASK=$REALSERVER_NETMASK
@	#hmm never get here anymore if are a director.
@	if [ "$WHOAMI" = "DIRECTOR" ] 
@	then
@		MY_IP=$DIP
@	elif [ "$WHOAMI" = "REALSERVER" ]
@	then
@		# presumably I could pass this to the routine, 
@		# but I never saved it in the first place
@
@		RIP=`$IFCONFIG $SERVER_NET_DEVICE | $GREP "inet addr" | $AWK '{print $2}'`
@		# RIP is now "addr:192.168.1.12", chop off "addr:"
@		RIP=${RIP#addr:}
@		#echo "RIP $RIP"
@	else
@		echo "Error: I'm $WHOAMI, not a director and not a realserver" 1>&2
@		((NUM_ERRORS++))
@	fi
@	}
@		
@setup_core_filter_rules()
@	{
@	#filtering rules are based on rc.firewall (C) Oskar Andreasson blueflux@koffein.net
@	#at http://www.boingworld.com/workshops/linux/iptables-tutorial/rc.firewall.txt
@	#for tutorial see http://www.boingworld.com/workshops/linux/iptables-tutorial/
@	#
@	#thanks also to Derek Glidden and Ratz on the LVS mailing list
@	# sets up rules only for the interface carrying the VIP.
@	# accepts icmp packets and packets for services accepted at the VIP.
@	#
@	#aims:
@	#Here rules are setup for packets arriving 
@	#on the interface facing the internet to be accepted by the LVS.
@	#Rules for other interfaces are not handled here.
@	#The DIP may or may not be on the same interface.
@	#(I'll make up rules for the DIP later.)
@	#In the case of classic VS-DR/VS-Tun, the director has no default gw, 
@	#so nothing can be sent to the outside world.
@	#In the case of VS-DR with Julian's martian modification,
@	#the director requires a default gw.
@	#For VS-NAT the director requires a default gw.
@	#the network of the DIP and the real-servers is not filtered.
@	#
@	# rules:
@	# accepted: 	packets for LVS services
@	#
@	# denied:	all else. the director doesn't need DNS.
@	#
@	if [ "$LINUX_KERNEL_SERIES" = "2.4" ]
@	then	
@		setup_core_filter_rules_2_4
@	elif [ "$LINUX_KERNEL_SERIES" = "2.2" ]
@	then
@		setup_core_filter_rules_2_2
@	else
@		echo "Error: linux kernel series $LINUX_KERNEL_SERIES not 2.2 or 2.4"
@ 		((NUM_ERRORS++)) 
@	fi
@	}
@	
@show_filter_rules()
@	{
@	echo "ipchains rules for director"
@	if [ "$LINUX_KERNEL_SERIES" = "2.4" ]
@	then	
@		$IPTABLES -L
@	elif [ "$LINUX_KERNEL_SERIES" = "2.2" ]
@	then
@		$IPCHAINS -L
@	else
@		echo "Error: linux kernel series $LINUX_KERNEL_SERIES not 2.2 or 2.4"
@ 		((NUM_ERRORS++)) 
@	fi
@	}
@
@#end boilerplate------------------------------------
BOILERPLATE

#----------------------------------------------------

sub calc_log_2
	{
	$log_2 = 0;
	my $temp = $_[0];
	#print "$temp \n";
	while ($temp != 1)
		{
		$temp = $temp >> 1;
		$log_2++;
		#print "$temp $log_2 \n";
		}
	#print "$log_2 \n";
	return ($log_2);
	}

sub dotquad_to_classless
	{
	#takes 255.255.255.0, converts it to 24
	$dot_quad = $_[0];
	#print "dot_quad: $dot_quad \n";
	$dot_quad =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ ;
	#print "$1 $2 $3 $4 \n";
	$classless = calc_log_2(eval($1+1)) + calc_log_2(eval($2+1)) + calc_log_2(eval($3+1)) + calc_log_2(eval($4+1)); 
	#print "classless $classless \n";
	return ($classless);
	}

sub dotquad_to_number
	{
	my $number;
	#takes 192.168.1.0, converts it to 192*256^3+168*256^2+1*256+0=3232235776
	$dot_quad = $_[0];
	#print "dot_quad: $dot_quad \n";
	$dot_quad =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/ ;
	#print "$1 $2 $3 $4 \n";
	$number = eval($1*256*256*256) + eval($2*256*256) + eval($3*256) + eval($4); 
	#print "number $number \n";
	return ($number);
	}

sub parse_hostname
	{
	my $host = $_[0];
	if ($host =~ /\s/) {print "parse_hostname: error - null hostname or ip \n";}
	#print "parse_hostname: $host\n";
	my ($host_name, $host_ip, $packed_ip); 

	#host can be a host_ip (eg 192.168.1.19) or a name in /etc/hosts (eg grumpy, grumpy.mack.net)
	#whichever version is given, we need to find the other version too

	#if $host contains letters, then assume it is a name and find corresponding host_ip 
	if ($host =~ /\d{1,3}.\d{1,3}.\d{1,3}.\d{1,3}/)	
		{
		#$host is a host_ip number and not a host name
		$host_ip = $host;
		$host_name = gethostbyaddr(inet_aton("$host_ip"),AF_INET);
		if ($host_name eq '') 
			{
			#accept and continue
			#print "parse_hostname: no name available for host on host_ip $host_ip, will use host_ip number instead. \n";
			#print "you could enter a name in /etc/hosts file for the host on host_ip $host_ip \n";
			#print "and this message will go away. \n";
			$host_name = $host;
			}
		else
			{
			#print "parse_hostname: host for host_ip $host is $host_name \n";
			}
		}
	else
		{
		#$host is a name
		$host_name = $host;
		$packed_ip = gethostbyname($host_name) or die "parse_hostname: error: gethostbyname failure - Couldn't lookup hostname $host_name : $! \n";
		$host_ip = inet_ntoa($packed_ip);
		if ($host_ip eq '')
			{
			print "parse_hostname: host $host_name does not resolve to a valid host_ip number \n";
			exit 1;
			}
		else
			{
			#print "parse_hostname: host_ip for host $host_name is $host_ip \n";
			}
		}
	return ($host_name, $host_ip);
	}

sub parse_service
	{
	my $service = $_[0];	
	my ($service_name, $port); 

	#service can be a port (eg 80) or a name in /etc/services (eg http)
	#whichever version is given, we need to find the other version too

	#if $service contains no digits, then assume it is a name and find corresponding port 
	if ($service =~ /\D/)	
		{
		#$service is a name
		$service_name = $service;
		$port = getservbyname($service, $protocol);
		if ($port eq '')
			{
			print "service $service_name does not give a valid port number in /etc/services \n";
			exit 1;
			}
		else
			{
			#print "port for service $service_name is $port \n";
			}
		}
	elsif ($service eq "")
		{
		print "Error: no port information (name/number) available. \n";
		}
	else 	
		{
		#$service is a port number and not a service name
		$port = $service;
		$service_name = getservbyport($service,$protocol);
		if ($service_name eq '') 
			{
			#accept and continue
			#is it port 0 (ie persistence)?
			if ($port eq 0)
				{
				print "Error: cannot use port $port in service line. \n";	
				}
			else
				{
				print "No name available in /etc/services for port $port \n";
				print "Will use port number instead of the name. \n";
				print "If you put an entry in /etc/services, \n";
				print "this message will go away. \n";
				}
			$service_name = $service;
			}
		else
			{
			#print "service for port $service is $service_name \n";
			}
		}
	return ($service_name, $port);
	}

sub set_monitor
	{
	my $temp;
	#does this service have a monitor? if not set fping as the default monitor
	$monitor = "fping";
	foreach $temp (@monitors)
		{
		if ($temp eq $service_name)
			{
			$monitor = $temp;
			}
		}			
	}
		
sub write_mon_hostgroup
	{
	print MON_FILE "#------\n";
	print MON_FILE "hostgroup LVS$server_counter $server_ip \n";
	print MON_FILE "\n"; 
	print MON_FILE "watch LVS$server_counter \n";
	print MON_FILE "#the string/text following service (to OEL) is put in header of mail messages\n"; 
	#print MON_FILE "#The mon docs imply that a string is required (ie no blanks), but the test.alert and mail.alert handle text (several words) just fine.\n"; 
	#print MON_FILE "#virtualserver.alert bombs if text is sent (appears to be multiple parameters to Getopt), so the text is encased in \"\".\n"; 
	print MON_FILE "#service \"$service_name on LVS$server_counter $server_ip\" \n";
	#modified for mon-0.38.21
	print MON_FILE "service $service_name \n";
	print MON_FILE "	interval 15s \n";
	if ($monitor eq "dns")
		{
		#the dns.monitor needs to know a zone and the master DNS machine for that zone.
		#the monitor checks the master for the SOA/serial number of the zone
		#and then requests the same info from the machine being monitored (here the realserver)
		#Since any zone will do, we will monitor the domain that the LVS machine belongs to.
		#Since the only domain the script knows about is the domain of the LVS, the script
		#will look for the LVS domain in /etc/resolv.conf
		#The director will be the master machine for the domain, whose only function need be to serve up
		#a serial number for the domain and the IP/name of the director machine.
		#The realservers need only be caching secondaries for the LVS domain. 
		#For real queries they will have to go out onto the net anyhow.
		#Once the LVS is running, if the files on the master machine are updated, 
		#the serial number on the copy on the realservers will not be updated till the 
		#timeout/refresh period (usually several hours).
		#In this case named on the realservers will have to be shutdown, 
		#files used as cache backup deleted and then named on the realservers restarted 
		#

		#look for domain in resolv.conf, then look for SOA nameserver for domain
		my $resolv_conf_file = "/etc/resolv.conf";
		open (RESOLV_CONF_FILE, "<$resolv_conf_file")|| die "Can't read $resolv_conf_file $!\n";
		my $domain;
		my $ns;
		my $line_counter=0;
		while (<RESOLV_CONF_FILE>)
			{
			$line_counter++;
			chomp;
			#print "resolv.conf $_\n";
			if ($_ =~ /^domain (.*) */)
				{
				$domain = $1;
				#print "domain: $domain $line_counter\n";
				} 
			}
		#find the nameservers for $domain
		#code modelled on dns.monitor (C) 1998 David Eckelkamp
		#and on Net::DNS man page
		my $res = new Net::DNS::Resolver;
		my $query = $res->query ($domain, "SOA");
		if ($query)
			{
			#($query->answer)[0] ->print;
			unless ($query->header->aa)
				{
				#print "nameserver not authoritative for $domain \n";
				}
			$ns = ($query->answer)[0]->string;
			#print "ns $ns\n";
			#non-greedy matching	
			if ($ns =~ /SOA\s+(.*?)\.\s+/)
				{
				#print "chopped ns $1 \n";
				$ns = $1;
				print MON_FILE "	monitor $monitor.monitor -zone $domain -master $ns \n";
				}
			}
		else
			{
			print "search for SOA for $domain failed: ", $res->errorstring, "\n";
			exit 1;
			}

		#search for nameservers (is SOA's)
		#my $query = $res->query($domain, "NS");
		#my $rr;
		#if ($query)
		#	{
		#	foreach $rr ($query->answer)
		#		{
		#		next unless $rr->type eq "NS";
		#		print $rr->nsdname, "\n";
		#		}
		#	}
		#else
		#	{
		#	print "search for nameserver for $domain failed: ", $res->errorstring, "\n";
		#	}
#		print MON_FILE "	monitor $monitor.monitor -zone mack.net -master di.mack.net \n";
		}
	else
		{
		print MON_FILE "	monitor $monitor.monitor \n";
		}
	print MON_FILE "	allow_empty_group \n";
	print MON_FILE "	period wd {Sun-Sat} \n";
	#next line thanks to Tony Bassette <kult@april.org>
	print MON_FILE "	numalerts 2 \n";
	print MON_FILE "	#alertevery 1h \n";
	print MON_FILE "		#alert mail.alert root \n";
	print MON_FILE "		#upalert mail.alert root \n";
	print MON_FILE "		#-V is virtual service, -R is remote service, -P is protocol, -A is add/delete (t|u)\n";
	if ($lvs_type =~ /tun|dr/)
		{
		print MON_FILE "		alert virtualserver.alert -A -d -P -$proto -V $vip:$port -R $server_ip \n";
		print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip -T $connection_parameter -w $weight\n";
		}
	elsif ($lvs_type eq "nat")
		{
		#print MON_FILE "		alert virtualserver.alert -A -d -P -$proto -V $dip:$port -R $server_ip:{$server_port} \n";
		#print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $dip:$port -R ${server_ip}:${server_port} -T $connection_parameter -w $weight\n";
		#argh, Mar 2001. not dip, but vip!
		print MON_FILE "		alert virtualserver.alert -A -d -P -$proto -V $vip:$port -R $server_ip:$server_port \n";
		print MON_FILE "		upalert virtualserver.alert -A -a -P -$proto -V $vip:$port -R $server_ip:$server_port -T $connection_parameter -w $weight\n";
		}
	else	
		{
		print "error: unknown LVS_type $lvs_type \n";
		}

	print MON_FILE "\n";
	print MON_FILE "#the line above must be blank\n";
	print MON_FILE "\n";
	}

sub create_directories
	{
	my @directories=("$rc_lvs_home", "$rc_mon_home", "$rc_ntp_home");
	my $directory; 
	foreach $directory (@directories)
		{
		#print ("create_directories: checking directory $directory \n");
		if ( -d $directory)
			{
			;
			}
		else	{
			#print ("create_directories: name $directory is not a directory/ \n");
			if ( -x $directory )
				{
				print("error: $directory exists but is not a directory. \n"); 
				$input_errors++;
				}
			else
				{
				mkdir("$directory",0644) || die "cannot mkdir $directory: $!";
				}
			}
		}			
	}

sub open_files
	{
	$rc_file = "rc.lvs_$lvs_type";
	#print "rc_lvs_home $rc_lvs_home rc_mon_home $rc_mon_home rc_ntp_home $rc_ntp_home \n"; 
	open (RC_FILE, ">$rc_lvs_home"."$rc_file")|| die "Can't write to $rc_file $!\n";
	#$mon_file = "mon_lvs${lvs_type}.cf";
	$mon_file = "mon.cf";
	open (MON_FILE, ">$rc_mon_home"."$mon_file")|| die "Can't write to $mon_file $!\n";
	$director_ntp_file = "ntp.conf.director";
	open (DIRECTOR_NTP_FILE, ">$rc_ntp_home"."$director_ntp_file")|| die "Can't write to $director_ntp_file $!\n";
	$realserver_ntp_file = "ntp.conf.realserver";
	open (REALSERVER_NTP_FILE, ">$rc_ntp_home"."$realserver_ntp_file")|| die "Can't write to $realserver_ntp_file $!\n";
	}

sub print_headers
	{
	print RC_FILE "#!/bin/bash \n";
	print RC_FILE "#rc.lvs_${lvs_type} (C) Joseph Mack 1999,2000.\n"; 
	print RC_FILE "#autogenerated by configure_lvs.pl from lvs_${lvs_type}.conf.\n";
	print RC_FILE "#Feel free to edit.\n";
	print RC_FILE "#This file needs a version of sh that has subroutines.\n";
	print RC_FILE "#sh on Solaris 2.7 and bash on Linux are fine.\n";
	print RC_FILE "#\n";
	print RC_FILE "#Setup for a Linux Virtual Server (LVS) running in $lvs_type mode.\n";
	print RC_FILE "#This code is part of the LVS project http://www.linuxvirtualserver.org\n";
	print RC_FILE "#The rc.lvs files produced by this code runs on Solaris machines, \n";
	print RC_FILE "#but I haven't been able to test whether it actually sets up a working realserver yet\n";
	print RC_FILE "#\n";
	print RC_FILE "#---begin boilerplate-------------------------------\n";
	print RC_FILE "#\n";
	#print RC_FILE "$ratz";
	#print RC_FILE "$horms";
	print RC_FILE "$general_boilerplate";

	print MON_FILE "#mon_lvs${lvs_type}.cf \n";
	print MON_FILE "\n";
	print MON_FILE "#an example mon.cf file for a group of LVS machines running in $lvs_type mode\n";
	print MON_FILE "#generated by configure_lvs.pl - Feel free to edit\n";
	print MON_FILE "#This code is part of the LVS project http://www.linuxvirtualserver.org/\n";
	print MON_FILE "#for more information about LVS please see the LVS webpage\n";
	print MON_FILE " \n";
	print MON_FILE "#adapted from the mon.cf script supplied by Jim Trocki as part of the mon distribution.\n";
	print MON_FILE "#for info about mon see http://www.kernel.org/software/mon/\n";
	print MON_FILE " \n";
	print MON_FILE "#<IMPORTANT NOTE FROM Jim Trocki>\n";
	print MON_FILE "#\n";
	print MON_FILE "#NB:  hostgroup and watch entries are terminated with a blank line,\n";
	print MON_FILE "#or end of file. Don\'t forget the blank lines between them or you lose.\n";
	print MON_FILE "#\n";
	print MON_FILE "#</IMPORTANT NOTE FROM Jim Trocki>\n";
	print MON_FILE "#--------------------------------------------------\n";
	print MON_FILE "#\n";
	print MON_FILE "#mon config info, you probably don\'t need to change this very much\n";
	print MON_FILE "#\n";
	print MON_FILE "\n";
	print MON_FILE "alertdir   = /usr/lib/mon/alert.d/ \n";
	print MON_FILE "mondir     = /usr/lib/mon/mon.d/ \n";
	print MON_FILE "#mon wants writes to a state file, or else will fill your syslog file with complaints. \n";
	print MON_FILE "#The default statedir is the same as the alertdir (or basedir) \n";
	print MON_FILE "#However I don't want anything writing to /usr so put it in the log file directory.\n";
	print MON_FILE "#you need to touch files 'opstatus' and 'disabled' in statedir, or mon will complain about not being able to open statedir files. \n";
	print MON_FILE "statedir     = /var/adm/ \n";
	print MON_FILE "#maxprocs    = 20\n";
	print MON_FILE "histlength = 100\n";
	print MON_FILE "#delay before starting\n";
	print MON_FILE "randstart = 60s\n";
	print MON_FILE "\n";
	} 

sub print_close_mon_file
	{
	print MON_FILE      "#mon_lvs${lvs_type}.cf---------------------------\n";
	}

sub is_kernel_too_old
	{
	#$uname_r =~ /^2\.2\.(\d+)/ ;
	$uname_r =~ /^2\.(\d+).(\d+)/ ;
	my $kernel_major_version = $1;
	my $kernel_minor_version = $2;
	#print "kernel major version $kernel_major_version minor version $kernel_minor_version \n";
	if (($kernel_major_version == 2) &&  ($kernel_minor_version < 13))
		{
		print "This version of LVS (kernel $uname_r) is old. \n";
		print "It does not handle icmp packets. \n";
		print "suggest you upgrade your kernel and LVS versions. \n";
		}
	}

sub setup_director_TP
	{
	#print "setup_director_TP: start \n";
	print RC_FILE "	#setup accepting connection by TP\n";
	print RC_FILE "	echo \" \"\n";
	print RC_FILE "	echo \"Director is accepting packets by transparent proxy (TP).\"\n";
	print RC_FILE "	echo \"When using TP, the director does not have an IP for the VIP.\"\n";
	print RC_FILE "	echo \"You must install a route on the router/test client telling it how to get to the VIP.\"\n";
	print RC_FILE "	echo \"You can put a host route to an IP already on the director \"\n";
	print RC_FILE "	echo \"eg route add -host \$VIP gw director \"\n";
	print RC_FILE "	echo \"or \"\n";
	print RC_FILE "	echo \"put a (permanent) arp entry (arp -f /etc/ethers or arp -s) pointing to a MAC address on the director.\"\n";
	print RC_FILE "	echo \"\"\n";
	print RC_FILE "	echo \"Turn on ip_forward, required for TP (1,on. 2,off) \" \n";
	print RC_FILE "	echo \"1\" >/proc/sys/net/ipv4/ip_forward \n";	
	print RC_FILE "	echo \"Installing director redirect rules...\"\n";
	foreach $line_reference (@redirect_lines)
		{
		($server_ip, $service_name, $protocol) = @$line_reference; 
		#print "setup_director_TP: $server_ip, $service_name, $protocol \n";
		print RC_FILE " 	echo \"redirecting \${VIP}:$service_name:$protocol 	to local:$service_name \" \n";
		print RC_FILE "		if [ \$LINUX_KERNEL_SERIES = \"2.0\" -o \$LINUX_KERNEL_SERIES = \"2.2\" ] \n";
		print RC_FILE "		then \n";
		print RC_FILE "			\$IPCHAINS -A input -j REDIRECT $service_name -d \$VIP $service_name -p $protocol \n";
		print RC_FILE " 		if [ \$? != \"0\" ] \n";
		print RC_FILE " 		then \n";
		print RC_FILE " 			echo \"Error: setup_director_TP - transparent proxy not installed?\" 1>&2\n";
		print RC_FILE " 			((NUM_ERRORS++)) \n";
		print RC_FILE " 		fi\n";
		print RC_FILE "		elif [ \$LINUX_KERNEL_SERIES = \"2.4\" ] \n";
		print RC_FILE "		then \n";
		print RC_FILE "			\$IPTABLES -t nat -A PREROUTING -p tcp -d \$VIP --dport $service_name -j REDIRECT \n";
		print RC_FILE " 		if [ \$? != \"0\" ] \n";
		print RC_FILE " 		then \n";
		print RC_FILE " 			echo \"Error: setup_director_TP - transparent proxy not installed?\" 1>&2\n";
		print RC_FILE " 			((NUM_ERRORS++) \n";
		print RC_FILE " 		fi\n";
		print RC_FILE "		else \n";
		print RC_FILE " 		echo \"Error: setup_director_TP - kernel series \$LINUX_KERNEL_SERIES unknown\" 1>&2\n";
		print RC_FILE " 		((NUM_ERRORS++)) \n";
		print RC_FILE "		fi \n";
		print RC_FILE "	\n";
		}
	print RC_FILE "	show_ip_tables_chains \n";
	print RC_FILE "	\n";
	#print "\n";
	}

sub setup_director_masq_table
	{
	my $temp;
	my $server_found="N";
	#my ($server_name, $server_ip);
	print RC_FILE "\n";
	#only do if VS-NAT, 2.0 or 2.2, 
	#for 2.4 the LVS code handles masquerading out the calls from the real-servers
	print RC_FILE "	if [ \$LINUX_KERNEL_SERIES = \"2.0\" -o \$LINUX_KERNEL_SERIES = \"2.2\" ] \n";
	print RC_FILE "	then \n";
	#if ($lvs_type =~ /nat/)
	print RC_FILE "		if [ \$LVS_TYPE = \"vs-nat\" ] \n";
	print RC_FILE "		then \n";
	#used to masquerade all IPs, all ports
	#now just masquerade IP:services being LVS'ed
	#my $classless;
	#$classless = dotquad_to_classless($realserver_netmask);
	#print RC_FILE "	\$IPCHAINS -A forward -j MASQ -s ${realserver_network}/${classless} -d 0.0.0.0/0\n";
	print RC_FILE " 	";	
	print RC_FILE "		echo \"installing ipchain rules\"\n";
	foreach $line_reference (@redirect_lines)
		{
		($server_name, $service_name, $protocol) = @$line_reference; 
		($server_name, $server_ip) = parse_hostname($server_name);
		#print "setup_director_masquerade: $server_name, $service_name, $protocol \n";
		#print "setup_director_masquerade: $server_ip, $service_name, $protocol \n";
		#print RC_FILE " 		echo \"masquerading $protocol $server_ip:$service_name:$protocol to outside world\" \n";
		print RC_FILE " 			echo \"masquerading $protocol $server_name:$service_name:$protocol to outside world\" \n";
		#print RC_FILE " 		\$IPCHAINS -A forward -p $protocol -j MASQ -s $server_ip $service_name -d 0.0.0.0/0\n";
		print RC_FILE " 			\$IPCHAINS -A forward -p $protocol -j MASQ -s $server_name $service_name -d 0.0.0.0/0\n";
		if ($service_name eq "ftp")
			{
			print RC_FILE " 			echo \"adding ftp-data masquerade to match ftp \"\n";
			print RC_FILE " 			\$IPCHAINS -A forward -p $protocol -j MASQ -s $server_name ftp-data -d 0.0.0.0/0\n";
			}

		#in early versions (=<0.8) I issued a command to masquerade high ports each pass 
		#through here, whether the high ports had been masqueraded aleady or not.
		#This added redundant entries to the masq table.
		#It worked, but Alois Treindl Alois@astro.ch pointed out that I was doing it.
		#Here I fixed it (May 2001).

		#have we masqueraded high ports for $server_name?
		foreach $temp (@high_ports_masqueraded)
			{
			if ($temp eq $server_name)
				{
				$server_found="Y";
				#print ("setup_director_masq_table: server $server_name already masqueraded \n");	
				}
			}
		if ($server_found eq "Y")
			{
			#do nothing
			}
		else
			{
			#update list
			push (@high_ports_masqueraded, $server_name);	
			#print ("@high_ports_masqueraded \n");
			print RC_FILE " 			echo \"masquerading high ports 1025:65535 for $server_name \" \n";
			print RC_FILE " 			\$IPCHAINS -A forward -p $protocol -j MASQ -s $server_name 1025:65535 -d 0.0.0.0/0\n";
			}
		}
	print RC_FILE "			\n";
	print RC_FILE "			echo \"ipchain rules \"\n";
	print RC_FILE "			\$IPCHAINS -L\n";
	print RC_FILE "		fi \n";
	print RC_FILE "	elif [ \$LINUX_KERNEL_SERIES = \"2.4\" ] \n";
	print RC_FILE "	then \n";
	print RC_FILE "		echo \"masquerading handled by LVS code. \"\n";
	print RC_FILE "		\$IPTABLES -L -t nat \n";
	print RC_FILE "	else \n";
	print RC_FILE "		echo \"Error: setup_director_masq_table - kernel series \$LINUX_KERNEL_SERIES unknown\" 1>&2\n";
	print RC_FILE "		((NUM_ERRORS++)) \n";
	print RC_FILE "	fi \n";
	}

sub print_verify_inputs
	{
	#sanity check input
	if ($lvsconf_format_is_ok eq "Y")
		{
		}
	else
		{
		print "Error: conf file $conf_file uses LVSCONF_FORMAT $unknown_format \n";
		print "This configure script understands LVSCONF_FORMAT ";
		foreach $this_format (@formats_ok)
			{
			print "$this_format ";
			}
		print "\n";
		print "Please match your configure script and conf file versions. \n";
		print "To upgrade from 1.0 to 1.1 change DIRECTOR_INSIDEIP to DIP \n";
		$input_errors++ ;
		}

	if (($lvs_type eq "nat") || ($lvs_type eq "dr") || ($lvs_type eq "tun"))
		{
		}
	else
		{
		print "Error: invalid LVS_TYPE $lvs_type in conf file \n";
		$input_errors++ ;
		}

	if (($initial_state eq "ON") || ($initial_state eq "OFF"))

		{
		}
	else
		{
		print "Error: invalid INITIAL_STATE $initial_state in conf file \n";
		$input_errors++ ;
		}

	if ($vip eq "")
		{
		print "Error: no value for VIP \n";
		$input_errors++ ;
		}

	if ($server_name eq "")
		{
		print "Error: VIP $vip not resolvable to a hostname \n";
		$input_errors++ ;
		}

#hmm, haven't parsed service line yet, so don't know port number
#	if ($service_name eq "")
#		{
#		print "Error: cannot use service 0 in a SERVICE line. \n";
#		$input_errors++ ;
#		}

	if ($director_vip_device eq "")
		{
		print "Error: no device for VIP \n";
		$input_errors++ ;
		}

	if ($vip_netmask eq "")
		{
		print "Error: no netmask for VIP \n";
		$input_errors++ ;
		}

	if ($vip_broadcast eq "")
		{
		print "Error: no broadcast address for VIP \n";
		$input_errors++ ;
		}

	if ($dip_device eq "")
		{
		print "Error: no device for DIP \n";
		$input_errors++ ;
		}

	if ($dip eq "")
		{
		print "Error: no IP specified for DIP \n";
		$input_errors++ ;
		}

	if ($realserver_network eq "")
		{
		print "Error: no network specified for DIP \n";
		$input_errors++ ;
		}

	if ($realserver_netmask eq "")
		{
		print "Error: no netmask specified for DIP \n";
		$input_errors++ ;
		}

	if ($lvs_type eq "nat")
		{
		if ($director_gw eq "")
			{
			print "Error: no default gw specified for director for $lvs_type \n";
			$input_errors++ ;
			}
		}

# will need a notice that handles the TP case and the martian modification	
#	if (($lvs_type eq "dr") || ($lvs_type eq "tun"))
#		{
#		if ($director_gw ne "")
#			{
#			print "Error: for security reasons, there is no default gw for the director with $lvs_type \n";
#			$input_errors++ ;
#			}
#		}
#
	if (($lvs_type eq "dr") || ($lvs_type eq "tun"))
		{
		if ($server_vip_device eq "")
			{
			print "Error: no device specified for VIP on real-server \n"; 
			$input_errors++ ;
			}
		#print "server_vip_device $server_vip_device, short_server_vip_device $short_server_vip_device \n";
		}

	if (	($server_vip_device =~ /tunl/) ||
		($server_vip_device =~ /lo/) ||
		($server_vip_device =~ /dummy/) 
	   )	
		{
		$is_server_vip_device_local = "Y";
		
		if ($short_server_vip_device eq $server_vip_device)
 			{
			#print "print_verify_inputs: server_vip_device $server_vip_device, short_server_vip_device $short_server_vip_device \n";
			print "\n";
			print "Error: the real-server VIP device $server_vip_device is a local device \n";
			print "and must be an alias, eg $short_server_vip_device:0 rather than $short_server_vip_device.\n";
			print "The LVS will work with $short_server_vip_device, but only aliased \n";
			print "lo devices (eg $short_server_vip_device:0) can be removed \n";
			print "from the ifconfig table without rebooting.\n";
			print "If you configure an LVS with $short_server_vip_device:0 for the VIP on \n";
			print "the real-servers, you will be able to later re-configure \n";
			print "an LVS with a different VIP device without rebooting. \n";
			$input_errors++ ;
			} 
		}

	if ($server_net_device eq "")

		{
		print "Error: SERVER_NET_DEVICE not set\n";
		$input_errors++ ;
		}

	if ($lvs_type eq "nat")
		{
		if ($server_gw ne "")
			{
			print "Warning: SERVER_GW is set for $lvs_type.\n";
			print "It should not be set for a normal $lvs_type LVS. \n";
			print "You must know what you're doing to set this. \n";
			}
		else
			{
			#print "Setting default gw for $lvs_type LVS to the director $dip \n";
			$server_gw = $dip;
			}
		if ($server_vip_device ne "")
			{
			print "Warning: There should be no VIP on the real-server for a $lvs_type LVS. \n"; 
			print "Ignoring VIP device $server_vip_device specified for real-server. \n"; 
			$server_vip_device="";
			}
		}

	if (($lvs_type eq "dr") || ($lvs_type eq "tun"))
		{
		if ($server_gw eq "")
			{
			print "Error: SERVER_GW is not set for $lvs_type.\n";
			$input_errors++ ;
			}
		}

	if ($kernel_series eq "2.4")
		{
		if ($director_vip_device eq "TP")
			{
			print "Error: director can't accept packets by $director_vip_device in kernel series $kernel_series.\n";
			print "I'm going to let this pass so you can do developement here. \n";
			print "You'll connect with the service on the director. \n";
			#$input_errors++ ;
			}
		if ($server_vip_device eq "TP")
			{
			print "Warning: TP is different in 2.4.x kernels. Please read the HOWTO-1.1 or later for more info.\n";
			}
		}

	if ($number_service_lines == 0)
		{
		print "Error: no SERVICE lines.\n";
		$input_errors++ ;
		}
		
	if ($input_errors != 0)
		{
		die "$input_errors errors in $conf_file.";
		}
	} #print_verify_inputs

sub store_redirect_line
	{
	my (@redirect_line);	#for rc.horms, stores hostname, service, port, proto for TP
				#need new copy for each iteration
	#generate data for TP
	($server_name,$server_ip) = parse_hostname($server_ip);
	#push (@redirect_line, $server_ip);
	push (@redirect_line, $server_name);
	push (@redirect_line, $service_name);
	push (@redirect_line, $protocol);
	#print "store_redirect_line: service data for TP \n";
	#for $element (@redirect_line)
	#	{
	#	print "$element ";
	#	}
	#print "\n";
	push (@redirect_lines, \@redirect_line);	#store reference to @redirect_line
	#foreach $line_reference (@redirect_lines)
	#	{
	#	foreach $element (@$line_reference) { print "$element "; } print "\n";
	#	}
	#print "store_redirec_line: end \n";
	}

sub print_TP_line
	{
	#print "print_TP_line: start \n";
	#foreach $line_reference (@redirect_lines)
	#	{
	#	foreach $element (@$line_reference) { print "$element "; } print "\n";
	#	}
	#print "print_TP_line: \n";
	#flush ipchains table
	print RC_FILE "		clear_ip_tables_chains \n";
	print RC_FILE "		echo \"installing redirect rules\" \n";
	foreach $line_reference (@redirect_lines)
		{
		($server_name, $service_name, $protocol) = @$line_reference; 
		($server_name,$server_ip) = parse_hostname($server_name);
		# print "print_TP_line: $server_ip, $service_name, $protocol \n";
		# note blank after $server_ip. 
		# This is so that 192.168.1.1 doesn't grep match 192.168.1.11
		print RC_FILE "	if [ \`\$IFCONFIG -a | \$GREP -c \"$server_ip \"\` = \"1\" ]\n"; 
		print RC_FILE "	then \n";
		print RC_FILE " 		echo \"redirecting \${VIP}:$service_name:$protocol 	to local:$service_name 	on $server_ip\" \n";
		print RC_FILE "		if [ \$LINUX_KERNEL_SERIES = \"2.0\" -o \$LINUX_KERNEL_SERIES = \"2.2\" ] \n";
		print RC_FILE "		then \n";
		print RC_FILE " 		\$IPCHAINS -A input -j REDIRECT $service_name -d \$VIP $service_name -p $protocol \n";
		print RC_FILE " 		if [ \$? != \"0\" ] \n";
		print RC_FILE " 		then \n";
		print RC_FILE " 			echo \"Error: setup_TP - transparent proxy not installed?\" 1>&2\n";
		print RC_FILE " 			((NUM_ERRORS++)) \n";
		print RC_FILE " 		fi\n";
		print RC_FILE "		elif [ \$LINUX_KERNEL_SERIES = \"2.4\" ] \n";
		print RC_FILE "		then \n";
		print RC_FILE "			\$IPTABLES -t nat -A PREROUTING -i \$SERVER_NET_DEVICE -d \$VIP -p $protocol --dport $service_name -j REDIRECT \n";
		print RC_FILE " 		if [ \$? != \"0\" ] \n";
		print RC_FILE " 		then \n";
		print RC_FILE " 			echo \"Error: setup_TP - transparent proxy not installed?\" 1>&2\n";
		print RC_FILE " 			((NUM_ERRORS++)) \n";
		print RC_FILE " 		fi\n";
		print RC_FILE "		else \n";
		print RC_FILE "			echo \"Error: print_TP_line - kernel series \$LINUX_KERNEL_SERIES unknown\" 1>&2\n";
	print RC_FILE " 			((NUM_ERRORS++)) \n";
		print RC_FILE "		fi \n";
		print RC_FILE "	fi \n";
		}
	#list ipchain rules
	print RC_FILE "	echo \"listing redirect rules\" \n";
	print RC_FILE "	show_ip_tables_chains \n";
	}

sub check_conf_file_input_line
	{
	#empty @params (should have been read out by read_conf_file
	foreach $element (@params)
		{
		print "Error: syntax error in $conf_file \n";
		print "param array is not empty, contains element \"$element\" \n";
		print "in line $line \n";
		print "Possible unknown parameter or wrong number of parameters in line \n";
		shift @params;
		}
	} #check_conf_file_input_line

sub install_print_redirect_services
	{
	#for real-servers
	#print "install_print_redirect_services \n";
	print RC_FILE "print_redirect_services() {\n";
	print RC_FILE "	#for transparent proxy \n";
		print_TP_line;
	print RC_FILE  "	}\n";
	print RC_FILE  "\n";
	} #install_print_redirect_services

sub add_service_line
	{
	#the service line is a (possibly long) string of somewhat arbitrary syntax
	#which may specify several nodes and may (or may not) have weight(s) for nodes
	my (@service_params);	#since are using references, need a new array for each iteration
	my $counter=0;
	my $i;
	foreach $element (@params)
		{
		#print "add_service_line: $element \n";
		push (@service_params, $element);
		$counter++;
		}
	for ($i =0; $i < $counter; $i++) { shift @params ;}
	push (@service_lines, \@service_params);
	$number_service_lines = @service_lines;
	#print "add_service_line: number of service lines $number_service_lines \n";
	#print "add_service_line: ";
	#foreach $line_reference (@service_lines)
	#	{
	#	foreach $element (@$line_reference) { print "$element "; } print "\n";
	#	}
	#print "\n";
	}



#use strict;
#my $server;
#my $service;
#my @services;
#my @local_keys; 
#my %servers_and_services = 
#		(
#		RS1 => {
#			telnet => ["t","rr",1],
#			ftp => ["t","wlc",1]
#			},
#		
#		RS2 => {
#			http => ["t","wrr",1]
#			},
#
#		dopey => {
#			nntp => ["t","lc",1]
#			},
#
#		doc => {
#			ftp => ["t","wlc",1],
#			http => ["t","wrr",1],
#			https => ["t","wrr",1]
#			}
#		);
#	

sub find_uniq
	{
	#find unique services, p102 Cookbook
	my %seen = ();
	my $item;
	my @uniq;
	#print "find_uniq: @_ \n";
	foreach $item (sort @_)
		{
		unless ($seen{$item})
			{
			#print "find_uniq: new item $item \n";
			#if we get here, we haven't seen it
			$seen{$item} = 1;
			push (@uniq, $item);
			}
		}
	#print "find_uniq: @uniq \n";
	return (@uniq);
	}

sub by_name_or_ip
	{
	#print "by_ip: $_[0] $a $b \n";
	if ($_[0] eq "name")
		{return $a cmp $b;}
	else
		{return dotquad_to_number($a) <=> dotquad_to_number($b);} 
	}

sub by_name_number_or_install
	{
	my $service;
	my $a_persistent = "N";
	my $b_persistent = "N";
	#print "by_name_number_or_install: \$a = $a, \$b = $b ";

	if ($_[0] eq "name")
		{return $a cmp $b;}
	if ($_[0] eq "number")
		{return (parse_service($a))[1] <=> (parse_service($b))[1];} 
	if ($_[0] eq "install")
		{
		#print "persistent_services @persistent_services \n";
		#install persistent_services first
		#no ordering within persistent_services 
		foreach $service (@persistent_services)
			{
			#turn them into numbers just in case some are numbers and some names
			if 	(
				(parse_service($a))[1] eq (parse_service($service))[1]
				)	
				{$a_persistent = "Y";} 
			if 	(
				(parse_service($b))[1] eq (parse_service($service))[1]
				)	
				{$b_persistent = "Y";} 
			}

		if( ($a_persistent eq "Y") && ($b_persistent eq "Y") )
			{
			#print "$a and $b persistent \n";
			return(0);
			} 
		if($a_persistent eq "Y") 
			{
			#print "$a persistent, \$b = $b \n";
			return(-1);
			} 
		if($b_persistent eq "Y") 
			{
			#print "$b persistent, \$a = $a \n";
			return(1);
			} 
			#print "neither $a nor $b persistent \n";
			return (parse_service($a))[1] <=> (parse_service($b))[1]; 
		}
	}

sub sort_by_server
	{
	#print "sort_by_server: $_[0]\n";
	my @local_keys;
	foreach $server (sort {by_name_or_ip($_[0])} keys %servers_and_services)
		{
		#print "$server, ";
		#print ref($servers_and_services{$server}),"\n";
		
		#values are hashes
		@local_keys = sort keys %{$servers_and_services{$server}};
		#print "\t@local_keys \n";
		
		foreach $service (sort @local_keys)
			{
			#this works
			#my $ra = %{$servers_and_services{$server}}->{$service};
			#print @$ra;
	
			#hot shot version
			printf ("%-10s %-10s %s %s %s\n", $server, $service, @{%{$servers_and_services{$server}}->{$service}});
			}
		#print "\n";
		}
	}

sub sort_by_service
	{
	#print "sort_by_service: $_[0] \n";
	my @local_keys;
	my $local_service;
	my @uniq_services;
	#find all services, servers
	foreach $server (sort keys %servers_and_services)
		{
		#print "$server, ";
		#print ref($servers_and_services{$server}),"\n";
		
		#values are hashes
		@local_keys = sort keys %{$servers_and_services{$server}};
		#print "\t@local_keys \n";
		push (@services, @local_keys);
		#print "@services \n";
		}
	
#	#find unique services, p102 Cookbook
#	my %seen = ();
#	my @uniq_services = ();
#	foreach $service (sort @services)
#		{
#		unless ($seen{$service})
#			{
#			#print "seen $service \n";
#			#if we get here, we haven't seen it
#			$seen{$service} = 1;
#			push (@uniq_services, $service);
#			}
#		}
#	#print "unique services: @uniq_services \n";
	
	@uniq_services = find_uniq(@services);
	
	#now have a list of servers and services, 
	#not all servers will have all services 
	#have to check each server for the service as contents are listed.
	
	foreach $service (sort {by_name_number_or_install($_[0])} @uniq_services)
		{
		#this works
		#my $ra = %{$servers_and_services{$server}}->{$service};
		#print @$ra;
	
		foreach $server (sort keys %servers_and_services)
			{
			#looking for services on $server, will be unique.
			#not all services found on LVS will be on $server.
			#check each service before outputting.
			for $local_service (sort keys %{$servers_and_services{$server}})
				{
				if ($service eq $local_service)
					{
					printf ("%-10s %-10s %s %s %s\n", $server, $service, @{%{$servers_and_services{$server}}->{$service}});
					}
				}
			}
		}
	#print "\n";
	}

sub sort_by_proto_service
	{
	#for setting up director filter_rules
	#need all services forwarded by director as (proto,service) pairs
	#print "sort_by_proto_service: \n";
	my @local_keys;
	my $key;
	my $service;
	my $local_service;
	my @uniq_services;

	#find all services, servers
	foreach $server (sort keys %servers_and_services)
		{
		#print "$server, ";
		#print ref($servers_and_services{$server}),"\n";
		
		#values are hashes
		@local_keys = sort keys %{$servers_and_services{$server}};
		#print "\t@local_keys \n";
		push (@services, @local_keys);
		#print "@services \n";
		}
	
	@uniq_services = find_uniq(@services);
	
	#now have a list of servers and services, 
	#not all servers will have all services 
	#have to check each server for the service as contents are listed.
	
	foreach $service (@uniq_services)
		{
		#this works
		#my $ra = %{$servers_and_services{$server}}->{$service};
		#print @$ra;

		my $rh;	#contains proto, 1
		my $ra;

		foreach $server (sort keys %servers_and_services)
			{
			#looking for services on $server, will be unique.
			#not all services found on LVS will be on $server.
			#check each service before outputting.
			for $local_service (sort keys %{$servers_and_services{$server}})
				{
				if ($service eq $local_service)
					{
					#printf ("sort_by_proto_service: %-10s %-10s %s %s %s\n", $server, $service, @{%{$servers_and_services{$server}}->{$service}});
					
					#$ra = \@{%{$servers_and_services{$server}}->{$service}}; 
					#print ("dereference array: $$ra[0] \n");
					#print ("dereference array: $ra->[0] \n");
					#print ("@{%{$servers_and_services{$server}}->{$service}}[0] \n");
					#
					#save the proto into the hash as a key, 
					#give it a bogus value of "1".
					#storing this in a hash unique-ifies the data
					#so you don't have to uniq-ify the multiple (telnet,tcp) entries
					$proto_service{$service} = {$proto,1};
					}
				}
			}
		}
	#print "\n";
	#print_hash(\%proto_service);
	} #sort_by_proto_service

sub add_service_rules()
	{
	print RC_FILE "	echo \"installing filter rules for services \" \n";
	sort_by_proto_service;	#global %proto_service is filled
	foreach $service (keys %proto_service)
		{
		#print ("$service ");
		foreach $proto (keys %{$proto_service{$service}})
			{
			#print ("service $service proto $proto \n");
			if ($proto eq "t") {$proto = "tcp";}
			elsif ($proto eq "u") {$proto = "udp";}
			else {print ("error: proto $proto not t|u \n'"); $input_errors++;}
			#if ftp:tcp, add rules for active and passive ftp
			#check really is tcp or one day will have to deal with ftp:udp 
			if (($service eq "ftp") && ($proto eq "tcp"))
				{
				print RC_FILE "	if [ \"\$LINUX_KERNEL_SERIES\" = \"2.4\" ] \n";
				print RC_FILE "	then \n";
				print RC_FILE "		if [ \"\$WHOAMI\" = \"DIRECTOR\" ] \n";
				print RC_FILE "		then \n";
				print RC_FILE "			\$IPTABLES -I lvs_services 1 -p $proto -s 0/0 --dport ftp-data:ftp -j ACCEPT \n";
				print RC_FILE "			\$IPTABLES -I lvs_services 1 -p $proto -s 0/0 --dport 1025: -j ACCEPT \n";
				print RC_FILE "		elif [ \"\$WHOAMI\" = \"REALSERVER\" ] \n";
				print RC_FILE "		then \n";
				print RC_FILE "			\$IPTABLES -I lvs_services 1 -p $proto -s 0/0 --dport ftp-data:ftp -j ACCEPT \n";
				print RC_FILE "			\$IPTABLES -I lvs_services 1 -p $proto -s 0/0 --dport 1025: -j ACCEPT \n";
				print RC_FILE "		else \n";
				print RC_FILE "			echo \"Error: not director or realserver \" 1>&2 \n";
				print RC_FILE "			((NUM_ERRORS++)) \n";
				print RC_FILE "		fi \n";
				print RC_FILE "	elif [ \"\$LINUX_KERNEL_SERIES\" = \"2.2\" ] \n"; 
				print RC_FILE "	then \n";
				print RC_FILE "		if [ \"\$WHOAMI\" = \"DIRECTOR\" ] \n";
				print RC_FILE "		then \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK} --destination-port ftp-data:ftp -j ACCEPT \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK} --destination-port 1025: -j ACCEPT \n";
				print RC_FILE "		elif [ \"\$WHOAMI\" = \"REALSERVER\" ] \n";
				print RC_FILE "		then \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK} --destination-port ftp-data:ftp -j ACCEPT \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK} --destination-port 1025: -j ACCEPT \n";

				print RC_FILE "		else \n";
				print RC_FILE "			echo \"Error: not director or realserver \" 1>&2 \n";
				print RC_FILE "			((NUM_ERRORS++)) \n";
				print RC_FILE "		fi \n";
				print RC_FILE "	else \n"; 	
				print RC_FILE "		echo \"Error: kernel series \$LINUX_KERNEL_SERIES is not 2.2. or 2.4 \" 1>&2 \n";
				print RC_FILE "		((NUM_ERRORS++)) \n";
				print RC_FILE "	fi \n"; 	
				print RC_FILE "	\n"; 	
				}
			else
				{
				#not ftp
				print RC_FILE "	if [ \"\$LINUX_KERNEL_SERIES\" = \"2.4\" ] \n";
				print RC_FILE "	then \n";
				print RC_FILE "		\$IPTABLES -I lvs_services 1 -p $proto -s 0/0 --dport $service -j ACCEPT \n";
				print RC_FILE "	elif [ \"\$LINUX_KERNEL_SERIES\" = \"2.2\" ] \n"; 
				print RC_FILE "	then \n";
				print RC_FILE "		if [ \"\$WHOAMI\" = \"DIRECTOR\" ] \n";
				print RC_FILE "		then \n";
				print RC_FILE "			if [ \$LVS_TYPE = \"vs-nat\" ] \n";
				print RC_FILE "			then \n";
				print RC_FILE "			#packets arriving for the VIP on the VIP device \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s_v 1 -i \$SHORT_DIRECTOR_VIP_DEVICE -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -d \$VIP -j ACCEPT \n";
				print RC_FILE "			#packets leaving the DIP device headed for the real-servers \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s_d 1 -i \$SHORT_DIP_DEVICE -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -d \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -j ACCEPT \n";
				print RC_FILE "			#packets arriving on the DIP device returning from the real-servers \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s_d 1 -i \$SHORT_DIP_DEVICE -d ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -s \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -j ACCEPT \n";
				print RC_FILE "			elif [ \$LVS_TYPE = \"vs-dr\" ] \n";
				print RC_FILE "			then	 \n";
				print RC_FILE "			#packets arriving at director.\n";
				print RC_FILE "			#packets returning go out router and aren't seen here. \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s 0/0 --destination-port $service -j ACCEPT \n";
				print RC_FILE "			elif [ \$LVS_TYPE = \"vs-tun\" ] \n";
				print RC_FILE "			then	 \n";
				print RC_FILE "				echo \"\"  \n";
				print RC_FILE "			else \n";
				print RC_FILE "				echo \"Error: LVS_TYPE \$LVS_TYPE not vs-nat, vs-tun or vs-dr\" 1>&2 \n";
				print RC_FILE "				((NUM_ERRORS++)) \n";
				print RC_FILE "			fi \n";
				print RC_FILE "		elif [ \"\$WHOAMI\" = \"REALSERVER\" ] \n";
				print RC_FILE "		then \n";
				print RC_FILE "			if [ \$LVS_TYPE = \"vs-nat\" ] \n";
				print RC_FILE "			then \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -d \${RIP}/32 $service -j ACCEPT \n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -d ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK_CLASSLESS} -s \${RIP}/32 $service -j ACCEPT \n";
				print RC_FILE "			elif [ \$LVS_TYPE = \"vs-dr\" ] \n";
				print RC_FILE "			then	 \n";
				print RC_FILE "			#packets arriving at realserver.\n";
				print RC_FILE "			\$IPCHAINS -I lvs_s 1 -p $proto -s ! \${REALSERVER_NETWORK}/\${REALSERVER_NETMASK} --destination-port $service -j ACCEPT \n";

				print RC_FILE "			elif [ \$LVS_TYPE = \"vs-tun\" ] \n";
				print RC_FILE "			then	 \n";
				print RC_FILE "				echo \"\"  \n";
				print RC_FILE "			else \n";
				print RC_FILE "				echo \"Error: LVS_TYPE \$LVS_TYPE not vs-nat, vs-tun or vs-dr\" 1>&2 \n";
				print RC_FILE "				((NUM_ERRORS++)) \n";
				print RC_FILE "			fi \n";
				print RC_FILE "		else \n";
				print RC_FILE "			echo \"Error: not director or realserver \" 1>&2 \n";
				print RC_FILE "			((NUM_ERRORS++)) \n";
				print RC_FILE "		fi \n";
				print RC_FILE "	else \n"; 	
				print RC_FILE "		echo \"Error: kernel series \$LINUX_KERNEL_SERIES is not 2.2. or 2.4 \" 1>&2 \n";
				print RC_FILE "		((NUM_ERRORS++)) \n";
				print RC_FILE "	fi \n"; 	
				print RC_FILE "	\n"; 	
				}
			}
		}
	} #add_service_rules

sub orig_add_service_rules()
	{
	print RC_FILE "	echo \"installing filter rules for services \" \n";
	sort_by_proto_service;	#global %proto_service is filled
	foreach $service (keys %proto_service)
		{
		#print ("$service ");
		foreach $proto (keys %{$proto_service{$service}})
			{
			#print ("service $service proto $proto \n");
			if ($proto eq "t") {$proto = "tcp";}
			elsif ($proto eq "u") {$proto = "udp";}
			else {print ("error: proto $proto not t|u \n'"); $input_errors++;}
			#if ftp:tcp, add rules for active and passive ftp
			#check really is tcp or one day will have to deal with ftp:udp 
			if (($service eq "ftp") && ($proto eq "tcp"))
				{
				if ($kernel_series eq "2.4")
					{
					#print ("adding filter rules for service $service to kernel series $kernel_series \n");
					print RC_FILE "	\$IPTABLES -A lvs_services -p $proto -s 0/0 --dport ftp-data:ftp -j ACCEPT \n";
					print RC_FILE "	\$IPTABLES -A lvs_services -p $proto -s 0/0 --dport 1025: -j ACCEPT \n";
					}
				elsif ($kernel_series eq "2.2")
					{
					#print ("adding filter rules for service $service to kernel series $kernel_series \n");
					print RC_FILE "	\$IPCHAINS -I lvs_s 1 -p $proto -s 0/0 --destination-port ftp-data:ftp -j ACCEPT \n";
					print RC_FILE "	\$IPCHAINS -I lvs_s 1 -p $proto -s 0/0 --destination-port 1025: -j ACCEPT \n";
					}
				else 	
					{
					print ("error: kernel series $kernel_series not 2.2. or 2.4 \n");
					$input_errors++;
					}
				}
			else
				{
				if ($kernel_series eq "2.4")
					{
					#print ("adding filter rules for service $service to kernel series $kernel_series \n");
					print RC_FILE "	\$IPTABLES -A lvs_services -p $proto -s 0/0 --dport $service -j ACCEPT \n";
					}
				elsif ($kernel_series eq "2.2")
					{
					#print ("adding filter rules for service $service to kernel series $kernel_series \n");
					print RC_FILE "	\$IPCHAINS -I lvs_s 1 -p $proto -s 0/0 --destination-port $service -j ACCEPT \n";
					}
				else 	
					{
					print ("error: kernel series $kernel_series not 2.2. or 2.4 \n");
					$input_errors++;
					}
				}
			}
		}
	} #add_service_rules

#sub add_service_rules()
#	{
#	#parameters $add_delete $proto ${VIP} $service_name ${server_name}
#	if ($kernel_series eq "2.4")
#		{
#		#print ("add_service_rules: for kernel $kernel_series \n");
#		add_service_rules_2_4; 
#		}
#	elsif ($kernel_series eq "2.2")
#		{
#		add_service_rules_2_2;
#		#print ("don't have filter rules for linux kernel series 2.2.x yet. \n");
#		}
#	else
#		{
#		print ("error: linux kernel series $kernel_series not 2.2 or 2.4 \n");
#		$input_errors++; 
#		}
#	}

sub sort_ipvsadm
	{
	#sorts services and servers for output in various orders.
	#some of these are useful for a potential gui.
	#some are useful for installing services.
	if ($_[0] eq "by_servername") {sort_by_server("name");return 0;}
	if ($_[0] eq "by_serverip") {sort_by_server("ip");return 0;}
	if ($_[0] eq "by_servicename") {sort_by_service("name");return 0;}
	if ($_[0] eq "by_servicenumber") {sort_by_service("number");return 0;}
	if ($_[0] eq "by_proto_service") {sort_by_proto_service;return 0;}
	#"installorder" has persistent services first.
	#makes it easier to install services, 
	#if I know that all the persistent services have already been installed.
	if ($_[0] eq "by_installorder") {sort_by_service("install");return 0;} 
	print "sort_ipvsadm: sort order $_[0] unknown \n";
	}

sub old_working_store_service
	{
	print "store_service: \n";
	print_array(@_);
	my @service_params;
	my $ra_service_params; #reference to @service_params;
	my $rh_services_and_params; #reference to %service_and_params; 
	my $rh_services_and_services; #reference to %service_and_services;

	#print "store_service: $_[0] $_[1] $_[2] $_[3] $_[4] $_[5]\n";
	#will be RS1 ftp t rr 1 (and any other values we add later)
	#$servers_and_services{$_[0]} = $_[1];
	#print_hash(\%servers_and_services);
	
	#this works
	#@service_params = ("$_[2]","$_[3]","$_[4]","$_[5]");
	#$ra_service_params = \@service_params;

	#$ra_service_params is a reference to an anonymous array containing t rr 1 
	#(and anything else we want)
	$ra_service_params = ["$_[2]","$_[3]","$_[4]"];
	#print "ref: ra_service_params ", ref($ra_service_params), "\n";
	#print_array(@$ra_service_params);

	#this works
	#$services_and_params{$_[1]} = $ra_service_params;
	#how about this
	#$services_and_params{$_[1]} = ["$_[2]","$_[3]","$_[4]"];
	#print_hash(\%services_and_params);

	#$rh_services_and_params = \%services_and_params;
	#print_hash($rh_services_and_params);
	#in hash, key RS1 is associated with a value which is a reference.  
	#The reference is to a hash of 
	#{service, reference to params for this service on RS1}
	#$servers_and_services{$_[0]} is a reference to a hash
	#$servers_and_services{$_[0]} = $rh_services_and_params;
	#$servers_and_services{$_[0]} = \%services_and_params;
	#$servers_and_services{$_[0]} = {
	#				"$_[1]" => ["$_[2]","$_[3]","$_[4]"]
	#				};

	#print "got here 1 \n";
	#print_hash(\%servers_and_services);
	#print "got here 2 \n";
	#print_hash($servers_and_services{$_[0]});

	%{$servers_and_services{$_[0]}}->{$_[1]} = ["$_[2]","$_[3]","$_[4]"] ;
	print "got here 3 \n";
	print_hash($servers_and_services{$_[0]});

	#print "refs \n";
	#print ref(%servers_and_services),"\n"; 			#hash not a reference
	#print ref($servers_and_services{$_[0]}),"\n";		#reference to %services_and_params
	#print "got here 2 \n";
	#print_hash($servers_and_services{$_[0]});
	#print ref($servers_and_services{"RS1"}),"\n";	#reference to %services_and_params
	#print "keys: ", keys(%{$servers_and_services{$_[0]}}),"\n";
	#print "keys RS1  : ", keys(%{$servers_and_services{"RS1"}}),"\n";
	#print "keys RS2: ", keys(%{$servers_and_services{"RS2"}}),"\n";
	#print ref(%{$servers_and_services{$_[0]}}->{0}),"\n";	#reference to array
	#print "array member: ", @{%{$servers_and_services{$_[0]}}->{0}}[0],"\n";	#syntax error
	#print ref(%{$servers_and_services{$_[0]}}->{0}->[0]),"\n";
	#print "@{%{$servers_and_services{$_[0]}}->{0}}->[0] \n";
	#print "end refs \n";
	#print "\n";
	#print "keys: ", keys(%servers_and_services) ,"\n";
	#print "values: ", values(%servers_and_services) ,"\n";
	#print ref($servers_and_services{$_[0]}{0}),"\n";
	#print ref($servers_and_services{$_[0]}{0}[0]),"\n";
	#print "$servers_and_services{$_[0]} \n";

	#print "store_service: servers_and_services $_[0] $_[1] \n";
	#print "store_service: $servers_and_services{$_[0]}{0}[0] \n";
	#print "store_service: $servers_and_services{$_[0]}{0}[1] \n";
	#print "store_service: $servers_and_services{$_[0]}{1}[1] \n";
	#print "store_service: $servers_and_services{$_[0]}{1}[0] \n";
	#print "\n";
	#show_service($_[0], $_[1]);
	}

sub old_store_service
	{
	my @service_params;
	my $ra_service_params; #reference to @service_params;
	my $element;

	#print "store_service: $_[0] $_[1] $_[2] $_[3] $_[4] $_[5]\n";
	$servers_and_services{$_[0]} = $_[1];

	#make array of $proto, $scheduler, $weight
	#this doesn't work (p93 Cookbook, q... doesn't do variable interpolation)
	#@service_params = qw($_[2] $_[3] $_[4] $_[5]);	
	#this does
	@service_params = ("$_[2]","$_[3]","$_[4]","$_[5]");

	#print them out
	#print "store_service: server $_[0] $server_and_services{$_[0]} ";
	#foreach $element (@service_params) {print "$element ";}
	#print "\n";
	
	#this works
	#$ra_service_params = ["$_[2]","$_[3]","$_[4]","$_[5]"];
	#so does this
	$ra_service_params = \@service_params;


	#print "store_service: array reference @$ra_service_params[0] @$ra_service_params[1] @$ra_service_params[2] \n";
	$services_and_params{$_[1]} = $ra_service_params;
	#print "ref ref($services_and_params{$_[1]}) \n";
	print "store_service: services_and_params service $_[1] $services_and_params{$_[1]}[0] $services_and_params{$_[1]}[1] $services_and_params{$_[1]}[2] \n";
	print "\n";
	show_service($_[0], $_[1]);
	}

sub print_array
	{
	my $this_function = (caller(0))[3];
	my $number_members = scalar(@_);
	#we have only arrays of scalars, 
	#don't need to test each member for non-scalar-ness  
	#for ($temp=0; $temp < $num_tabs; $temp++){print "\t";}
	print "members = $number_members, @_ \n";
	#print "\n";
	}

sub print_hash
	{
	#parameter passed is a one member array,
	#the member is a reference to a hash
	my $rh = $_[0];
	my $temp;
	$num_tabs++; #offset printing of nested calls
	#my %local_hash = %$_; #this is fine till pass two references
	#my %local_hash = %($_[0]); #syntax error
	my $this_function = (caller(0))[3];
	my ($key, $value);
	for ($temp=0; $temp < $num_tabs; $temp++){print "\t";}
	print "$this_function: \n";
	while (($key,$value) = each (%$rh))
		{
		for ($temp=0; $temp < $num_tabs; $temp++){print "\t";}
		if (ref($value) eq "ARRAY")
			{
			print "key = $key \tvalue is array: ";
			print_array(@$value);
			}
		elsif (ref($value) eq "HASH")
			{
			print "key = $key \tvalue is hash: \n";
			print_hash(\%$value);
			}
		else
			{
			print "$key, $value \n";
			}
		}
	for ($temp=0; $temp < $num_tabs; $temp++){print "\t";}
	print "end $this_function: \n";
	$num_tabs--;
	}

sub show_service
	{
	print "show_service: services_and_params service $_[1] $services_and_params{$_[1]}[0] $services_and_params{$_[1]}[1] $services_and_params{$_[1]}[2] \n";
	print "\n";
	}

sub show_services
	{
	#parameter passed is one element array = ref to %servers_and_services
	#print "show_services \n";
	#my $rh = $_[0];

	#test hash
	my $food;
	my %food_color = (
			apple => "red",
			banana => "yellow",
			carrot => "orange",
			lettuce => "green"
			);

	#just hand it the hash now

	my $this_function = (caller(0))[3];
	my ($key, $value);
	my ($sub_key, $sub_value);
	print "$this_function: \n";
	#while (($key,$value) = each (%$rh))	#old way

	while (($key,$value) = each (%servers_and_services))
		{
		while (($sub_key,$sub_value) = each (%$value))
			{
			printf ("%16s %8s %4s %4s %4s \n", $key, $sub_key, @$sub_value);
			}
		}

	#foreach $key (sort keys %servers_and_services)
	#foreach $key (keys %servers_and_services)
	foreach $food (keys %food_color)
		{
		#print "\tshow_services: ", $key, "\t", ref($key),"\n";
		print "\tshow_services: ", $food,"\n";
		#while (($sub_key,$sub_value) = each (%$servers_and_services{$key}))
		#	{
		#	printf ("%16s %8s %4s %4s %4s \n", $key, $sub_key, @$sub_value);
		#	}
		}


	print "end $this_function: \n";
	} #show_services

sub old_list_services_on_server
	{
	#returns parameters for $server
	#"what services does $server have?"
	#my ($ra,$rh);
	my @local_services;
	
	if (exists($servers_and_services{$_[0]}))
		{
		#print "list_services_on_server: entry found for $_[0] \n";
		#$rh = $servers_and_services{$_[0]};
		@local_services = sort keys %{$servers_and_services{$_[0]}};
		#print "list_services_on_server: services for $_[0] @local_services \n";
		return (@local_services);
		}
	else
		{
		#print "list_services_on_server: no entry for $_[0] \n";
		return (-1);	#no entry for server
		}
	}

sub list_services_on_server
	{
	#returns servers which have $service
	#"which servers have ftp installed"
	#print "list_services_on_server: looking for servers with $_[0] \n";
	my @ra;
	my @local_servers;
	my $local_server;
	my @return_list;
	my $size_of_return_list;
	
	#find all servers
	@local_servers = keys %servers_and_services;
	for $local_server (@local_servers)
		{
		#check if service $_[0] is on this server
		#print "list_services_on_server: server $local_server \n";
		if (exists($servers_and_services{$local_server}->{$_[0]}))
			{
			#print "list_services_on_server: entry found for server $local_server \n";
			push (@return_list, $local_server);	
			}
		else
			{
			#print "list_services_on_server: no entry found for server $local_server \n";
			}
		}
	$size_of_return_list = @return_list;
	if ($size_of_return_list == 0)
		{
		return (-1);
		}
	else
		{
		return (@return_list);
		}
	}

sub list_services_on_vip
	{
	#"which services are using this VIP?"
	#print "list_services_on_vip: looking for services on vip $_[0] \n";
	my @ra;
	my @local_servers;
	my $local_server;
	my @return_list;
	my $size_of_return_list;
	
	my @local_services;
	my $local_service;
	my @local_params;
	my $local_param;
	
	@local_servers = sort keys %servers_and_services;
	for $local_server (@local_servers)
		{
		#print "list_services_on_vip: querying local_server $local_server \n";
		@local_services = sort keys %{$servers_and_services{$local_server}};
		#print "list_services_on_vip: services for $_[0]: @local_services \n";
		for $local_service (@local_services)
			{
			#print "list_services_on_vip: querying $local_service on $local_server \n";
			#print "list_services_on_vip: @{$servers_and_services{$local_server}->{$local_service}} \n";
			@local_params = @{$servers_and_services{$local_server}->{$local_service}} ;
			if ($local_params[3] eq $_[0])
				{
				#print "list_services_on_vip: found $_[0]:$local_service on $local_server \n";
				push (@return_list, $local_service);
				}
			}
		}

	@return_list = find_uniq(@return_list);

	$size_of_return_list = @return_list;
	if ($size_of_return_list == 0)
		{
		return (-1);
		}
	else
		{
		return (@return_list);
		}
	}

sub store_service
	{
	#How services are stored:
	#You need to store a line like 
	#VIP, server_name, service, proto, scheduler, weight, status.
	#eg 192.168.1.110, RS1, ftp, tcp, rr, 1, UP.
	#
	#I put a bit of thought into the format for storing services data, 
	#so that it can be retrieved easily and would reflect the underlying way LVS works.
	#Each server can have many services, each service on a server has one entry.
	#The scheme I have is 
	#on each server: 
	#hash with 
	#	key=service, 
	#	value=reference to array of proto, scheduler, weight, status, VIP
	#for the LVS
	#hash with 
	#	key=server_name
	#	value=reference to above hash.
	#
	#Doing it this way makes it easy to retrieve all services on a server.
	#However retreiving all servers with a particular service is a mess.
	#You have to descend through the layers and then go back and check the servers.
	#
	#I now realise that it doesn't make any difference how I'd stored the data.
	#I just need a scheme that I can write routines to search the data.
	#An array of arrays would have done.
	#

	#print "store_service: @_\n";
	##print "store_service: $_[0] $_[1] $_[2] $_[3] $_[4] $_[5]\n";
	##params will be array = RS1 ftp t rr 1 192.168.1.110 UP (and any other values we add later)

	%{$servers_and_services{$_[0]}}->{$_[1]} = ["$_[2]","$_[3]","$_[4]","$_[5]","$_[6]"] ;
	#print_hash($servers_and_services{$_[0]});	#show services for server $_[0]
	#print_hash(\%servers_and_services);		#show services for all servers
	#show_services(\%servers_and_services);		#used to send reference to hash
	#show_services;
	} #store_service

sub delete_service
	{
	my @keys;
	my @values;
	my $size_of;
	#print "delete_service: @_\n";
	#params will be array = RS1 ftp 
	#print "before delete \n";
	#print "services on $_[0] \n";
	#print_hash($servers_and_services{$_[0]});	#show services for server $_[0]
	#@keys = keys %servers_and_services; 
	#print "keys @keys \n";
	#@values = values(%{$servers_and_services{$_[0]}});
	#print "values @values \n";
	#print "delete_service: all services \n"
	#print_hash(\%servers_and_services);		#show services for all servers

	delete $servers_and_services{$_[0]}->{$_[1]};

	#we're deleting key/value pairs from the 2nd level of the hash
	#when we've deleted the last service from server,
	#then we'll have to delete the server too.
	#print "after delete \n";
	#print "delete_service: services on $_[0] \n";
	#print_hash($servers_and_services{$_[0]});	#show services for server $_[0]
	#@keys = keys %servers_and_services; 
	#print "keys @keys \n";
	@values = values(%{$servers_and_services{$_[0]}});
	$size_of = @values;	#array in scalar context gives size
	#print "values $size_of @values \n";
	if ($size_of == 0)
		{
		#print "deleting entry for $_[0] \n";
		delete $servers_and_services{$_[0]};
		#@keys = keys %servers_and_services; 
		#print "keys after removing $_[0]: @keys \n";
		#print "hash after removing $_[0] \n";
		#print_hash($servers_and_services{$_[0]});	#show services for server $_[0]
		}
	#print "delete_service: all services \n"
	#print_hash(\%servers_and_services);		#show services for all servers
	#show_services(\%servers_and_services);		#used to send reference to hash
	#show_services;

	#print "delete_service: end \n";
	}

sub print_service_lines
	{
	#0.9.4, support for the early (<2.2.14) style persistence has been dropped.
	#Persistence is supported only through @persistent_services.
	#i.e., there is no -p etc in the SERVICE line in the conf file.
	#
	#A scheduler must be set. 
	#This is easier than writing code to handle variable syntax,
	#where the scheduler may be missing.
	#
	#SERVICE=t telnet rr 192.168.1.11,1 192.168.1.12,2
	#SERVICE=t https rr 192.168.1.11 192.168.1.12 
	#
	#print "print_service_lines: "; foreach $element (@params) { print "$element "; } print "\n";
	shift @params; #@param[0] is "SERVICE"
	($proto, $service, $scheduler) = @params; 
	shift @params; #shift out 2 parameters
	shift @params;

	#parse protocol
	if ($proto eq 't'){$protocol = "tcp";} 
	elsif ($proto eq 'u'){$protocol = "udp";} 
	#else {print "error line $line_counter: unknown protocol $proto \n";}
	else {print "print_service_lines error: unknown protocol $proto \n";}

	($service_name, $port) = parse_service($service);
	#print "print_service_lines: service_name $service_name, port $port \n";
	if ($port eq 0)
		{
		die "print_service_lines: port $port not allowed in SERVICE line. \n";
		}

	print RC_FILE "	#\n";
	print RC_FILE "	#setup servers for $service \n";
	print RC_FILE "	#echo \"print_service_lines: service $service_name \" \n";

	#print "print_service_lines: set_monitor \n";
	set_monitor;

	#handle scheduler,
	#string will be one of @schedulers=rr|wrr|lc|wlc|dh|sh|lblc|lblcr
	my $scheduler_known = "N";
	my $scheduler_item; 
	foreach $scheduler_item (@schedulers)
		{
		if ($scheduler eq $scheduler_item)
			{
			#print "print_service_lines: scheduler $scheduler \n";
			$scheduler_known = "Y";
			#can shift out @params now
			shift @params;
			last;
			}
		#print "print_service_lines: scheduler_item $scheduler_item \n";
		} # $scheduler_item

	$persistence_found{$vip} = "N"; 
	foreach $persistent_service (@persistent_services)
		{
		print RC_FILE "	#echo \"print_service_lines: does service $service_name match the persistent service $persistent_service ? \" \n";
		if ($persistent_service eq $service_name) 
			{
			print RC_FILE "	#echo \"print_service_lines: service $service on forwarding type $lvs_type matches a service in @persistent_services\" \n";
			#ftp persistent on VS_DR, VS-Tun.
			#ftp not persistent if ftp on VS-NAT 
			if ($service eq "ftp")
				{
				#print RC_FILE "	echo \"print_service_lines: ftp service $service on forwarding type $lvs_type \" \n";
				if ($lvs_type =~ /nat/) 
					{
					print RC_FILE "	echo \"print_service_lines: service $service on forwarding type $lvs_type non persistent\" \n";
					}
				else
					{
					print RC_FILE "	echo \"print_service_lines: service $service on forwarding type $lvs_type persistent\" \n";
					$persistence_found{$vip} = "Y"; 
					}
				}
			else
				{
				print RC_FILE "	#echo \"print_service_lines: service $service on forwarding type $lvs_type persistent\" \n";
				$persistence_found{$vip} = "Y"; 
				
				}

			if ($persistence_found{$vip} eq "Y") 
				{
				print RC_FILE "	#echo \"print_service_lines: service $service_name for forwarding type $lvs_type will be set persistent \" \n";
				}
			}
		#else
		#	{
		#	print RC_FILE "	echo \"print_service_lines: service $service_name does not match persistent_service $persistent_service \" \n";
		#	}
		} #foreach $persistent_service

	#install service with ipvsadm
	if ($persistence_found{$vip} eq "N")
		{
		print RC_FILE "	echo \"print_service_lines: adding non-persistent service $vip:$service_name\" \n";
		print RC_FILE "	\$IPVSADM -A -$proto \${VIP}:$service_name  -s $scheduler \n";
		}
	else
		{
		print RC_FILE "	echo \"print_service_lines: adding persistent service $vip:$service_name \" \n";
		if ($persistence_set{$vip} eq "N")
			{
			#hasn't been setup yet
			#
			#this code is designed to prevent multiple set up of persistent service.
			#eg if ftp and https is running on the same VIP.
			#If you have already setup persistent service on the VIP, 
			#on second and subsequent installation of persistent service
			#you get a warning "setsockopt failed: File exists".
			#This is not a problem as the LVS gets setup properly,
			#but I'm trying to remove messages that might make people
			#call up the help desk with a "problem". ;-\
			#
			print RC_FILE "	echo \"print_service_lines: add persistent service for \${VIP} \" \n";
			print RC_FILE "	\$IPVSADM -A -$proto \${VIP} -p \$PERSISTENCE_TIMEOUT -s $scheduler \n";
			}
		else
			{
			#has already been setup, do nothing
			print RC_FILE "	echo \"print_service_lines: persistence already set for $vip \" \n";
			}
		}

	while ($server=$params[0])
		{
		my ($server_name);
		shift @params;
		#parse server
		#server will be ip[,weight] for tun|dr, giving server_ipport as just an ip 
		#server will be ip:port[,weight] for nat, giving server_ipport as ip:port
		($server_ipport, $weight) = split /,/, $server;
		if ($weight eq ''){$weight = 1;}
		if ($initial_state eq "OFF")
			{$weight_lvsadm = 0;}
		else
			{$weight_lvsadm = $weight;}
		#print "server ip:port $server_ipport \n";	
		($server_ip, $server_port) = split /:/, $server_ipport;

		#check that we have target port information for the realserver.
		if ($lvs_type =~ /tun|dr/)
			{
			#print "port $server_port \n";
			if ($server_port ne "")
				{
				print "Notice: SERVICE entry $server_ipport - setting port not allowed for lvs-$lvs_type. port $server_port ignored.\n";
				}
			}
		elsif ($lvs_type =~ /nat/)
			{
			if (($server_port eq "") || ($server_ip eq ""))
				{
				print "Error: SERVICE entry $server_ipport requires an ip:port for lvs-$lvs_type \n";
				$input_errors++;
				}
			}


		($server_name,$server_ip)=parse_hostname($server_ip);
		#add IP of server to @servers
		#print "adding $server_ip to list of servers \n";
		push (@servers, $server_ip);
		#add server names to @server_names
		push (@server_names, $server_name);

		#accumulate the number of servers out there
		#use later as a test to see if I'm the director or the server
		#don't count 127.0.0.1, all machines have it.
		#
		unless ($server_ip eq "127.0.0.1") {$server_counter++};
		#print "server_counter $server_counter\n";
		
		print RC_FILE "	#echo \"adding service $service_name to realserver $server_name using connection type $lvs_type weight $weight_lvsadm\"\n";

		if ($persistence_found{$vip} eq "Y")
			{
			if ($persistence_set{$vip} eq "N")
				{
				print RC_FILE "	\$IPVSADM -a -$proto \${VIP} -r ${server_name} \$CONNECTION_TYPE -w $weight_lvsadm\n";
				}
			else
				{
				print RC_FILE "	echo \"no need to install persistent service for server ${server_name} \"\n";
				
				}
			}
		else
			{
			if ($lvs_type eq "nat")
				{
				print RC_FILE "	\$IPVSADM -a -$proto \${VIP}:$service_name -r ${server_name}:${server_port} \$CONNECTION_TYPE -w $weight_lvsadm\n";
				}
			else
				{
				print RC_FILE "	\$IPVSADM -a -$proto \${VIP}:$service_name -r ${server_name} \$CONNECTION_TYPE -w $weight_lvsadm\n";
				}
			}

		print RC_FILE "	echo \"checking realserver $server_name reachable from director - \"\n";
		print RC_FILE "	\$FPING $server_name \n";

		if ($initial_state eq "OFF")
			{
			#if start with initial state off, 
			#then load ipvsadm commands above 
			#(to test that they can be loaded at run time)	
			#then delete the specific entry again
			#this way the scheduling is still on
			#if instead did ipvsadm -C, then the scheduling would be deleted too
			print RC_FILE "	echo \"director starting in \$INITIAL_STATE state.\" \n";
			print RC_FILE "	echo \"clearing ipvsadm entry. \" \n";

			if ($persistence_found{$vip} eq "Y")
				{
				print RC_FILE "	\$IPVSADM -d -$proto \${VIP} -r $server_name \n";
				}
			else
				{
				print RC_FILE "	\$IPVSADM -d -$proto \${VIP}:$service_name -r $server_name \n";
				}

			print RC_FILE "	echo \"\"\n";
			}
		else
			{
			print RC_FILE "	echo \"director starting in \$INITIAL_STATE state \" \n";
			}

		if ($lvs_type =~ /tun|dr/)
			{ 
			#print "director tun|dr \n";
			}
		elsif ($lvs_type eq "nat")
			{
			if ($service_name eq "ftp")
				{
				print RC_FILE "	echo \"loading ftp module \" \n";
				print RC_FILE "	load_function ftp\n";
				}

			}
		else
			{
			print "error: unknown LVS_type $lvs_type \n";
			} # $lvs_type

		#update %servers
		#print "print_service_line: calling store_service $server_name, $service_name, $proto, $scheduler, $weight,$vip,$up \n";
		store_service($server_name, $service_name, $proto, $scheduler, $weight, $vip, $initial_state);
		store_redirect_line;
		write_mon_hostgroup;
		} # while more servers:ports on the input line

	#have done all servers for this service.
	#If have set persistence for one service (eg ftp), 
	#then on the next pass, if you're setting another persistent service (like https)
	#then you won't need to add https on that round 
	#you'll get a "Service already exists" for the -A entry (the service)
	#you'll get a "Destination already exists" for the -a entry (the realserver)

	if ($persistence_found{$vip} eq "Y")
		{
		print RC_FILE "	echo \"print_service_lines: recording that persistence has been installed after installation of service $service_name \"\n";
		$persistence_set{$vip} = "Y";
		}

	#print RC_FILE "\n";
	} # sub print_service_lines

sub install_print_ipvsadm_lines
	{
	#print "installing install_print_ipvsadm_lines \n";
	print RC_FILE "print_ipvsadm_lines() {\n";
	foreach $line_reference (@service_lines)
		{
		#print "main: ";
		foreach $element (@$line_reference)
			{
			#print "$element ";
			push (@params, $element);
			}
		#print "\n";
		print_service_lines;
		}

	#show_services;
	#sort_ipvsadm("by_servername");
	#print "\n";
	#sort_ipvsadm("by_serverip");
	#print "\n";
	#sort_ipvsadm("by_servicename");
	#print "\n";
	#sort_ipvsadm("by_proto_service");
	#print "\n";
	#sort_ipvsadm("by_servicenumber");
	#print "\n";
	#sort_ipvsadm("by_installorder");
	#print "\n";
	#delete_service("RS1","ftp");
	#sort_ipvsadm("by_installorder");
	#print "\n";
	#delete_service("RS1","telnet");
	#sort_ipvsadm("by_installorder");
	#print "\n";
	my @foo;
	my $size_of_foo;
	#my $local_service = "ftp";
	#@foo = list_services_on_server("$local_service");
	#if (@foo[0] != -1)
	#	{
	#	$size_of_foo = @foo; 	#array in scalar context
	#	print "install_print_ipvsadm_lines: servers with $local_service, @foo $size_of_foo entries\n" ;
	#	}
	#my $local_service = "zork";
	#@foo = list_services_on_server("$local_service");
	#if (@foo[0] != -1)
	#	{
	#	$size_of_foo = @foo; 	#array in scalar context
	#	print "install_print_ipvsadm_lines: servers with $local_service, @foo $size_of_foo entries\n" ;
	#	}

	#my $local_server = "RS1";
	#@foo = list_services_on_server("$local_server");
	#if (@foo[0] != -1)
	#	{
	#	$size_of_foo = @foo; 	#array in scalar context
	#	print "install_print_ipvsadm_lines: $local_server has @foo $size_of_foo entries\n" ;
	#	}

	#my $local_server = "snagglepuss";
	#@foo = list_services_on_server("$local_server");
	#if (@foo[0] != -1)
	#	{
	#	$size_of_foo = @foo; 	#array in scalar context
	#	print "install_print_ipvsadm_lines: local_server has @foo $size_of_foo entries\n" ;
	#	}

	#@foo = list_services_on_vip($vip);	
	#if (@foo[0] != -1)
	#	{
	#	$size_of_foo = @foo; 	#array in scalar context
	#	print "install_print_ipvsadm_lines: vip $vip has $size_of_foo services, @foo \n" ;
	#	}
		
	print RC_FILE  "	}\n";
	print RC_FILE  "\n";
	} #install_print_ipvsadm_lines

sub print_director_ntp_file
	{
	print DIRECTOR_NTP_FILE "#director /etc/ntp.conf \n";
	print DIRECTOR_NTP_FILE "#pick (uncomment) a server near you (this one is in eastern USA) \n";
	print DIRECTOR_NTP_FILE "#server louie.udel.edu \n";
	print DIRECTOR_NTP_FILE "driftfile /etc/ntp.drift \n";
	print DIRECTOR_NTP_FILE "#logging is verbose. \n";
	print DIRECTOR_NTP_FILE "#This would seem to be the quietest setting. \n";
	print DIRECTOR_NTP_FILE "#Note: logconfig must have no blank between = and all. \n";
	print DIRECTOR_NTP_FILE "#logconfig =all\n";
	print DIRECTOR_NTP_FILE "logfile /var/adm/ntplog \n";
	print DIRECTOR_NTP_FILE "disable auth \n";
	print DIRECTOR_NTP_FILE "server 127.127.1.0      	#local CMOS clock \n";
	print DIRECTOR_NTP_FILE "#tell everyone we're a stratum 3 ntp server \n";
	print DIRECTOR_NTP_FILE "fudge 127.127.1.0 stratum 3 \n";
	}

sub print_realserver_ntp_file
	{
	my @uniq_server_names;
	@uniq_server_names = find_uniq(@server_names);
	print REALSERVER_NTP_FILE "#realserver /etc/ntp.conf \n";
	print REALSERVER_NTP_FILE "driftfile /etc/ntp.drift \n";
	print REALSERVER_NTP_FILE "logfile /var/adm/ntplog \n";
	print REALSERVER_NTP_FILE "disable auth \n";
	print REALSERVER_NTP_FILE "server 127.127.1.0 #local clock \n";
	print REALSERVER_NTP_FILE "#tell everyone we're a stratum 4 ntp server \n";
	print REALSERVER_NTP_FILE "#we'll defer to the director which is stratum 3 \n";
	print REALSERVER_NTP_FILE "fudge 127.127.1.0 stratum 4 \n";
	($dip_name, $dip) = parse_hostname($dip);
	print REALSERVER_NTP_FILE "server $dip_name \n";
	foreach $server_name (@uniq_server_names)	#$server_name may be an IP
		{
		($server_name,$server_ip) = parse_hostname($server_name);
		#loop through IPs of known realservers comparing them to the IP of this machine.
		#don't test for 127.0.0.1
		if ($server_name eq "localhost") {next;}
		#this code will print a $server_name for each of its services
		#need to fix this sometime
		print DIRECTOR_NTP_FILE "peer $server_name \n";
		print REALSERVER_NTP_FILE "peer $server_name \n";
		}
	}

sub copy_and_execute_rc_files
	{
	my $cmd;
	my $arguments;
	my $return_code;
	my %seen = ();
	my @uniq_server_names = ();
	my $item;
	print "\n";
	if ( -e "$rc_ntp_home"."ntp.conf")
		{
		$cmd = "/bin/rm $rc_ntp_home"."ntp.conf";
		#print "$cmd \n";
		system ("$cmd");
		}
	$cmd = "ln -s $rc_ntp_home"."$director_ntp_file $rc_ntp_home"."ntp.conf";
	#print "ntp command:$cmd \n";
	system ("$cmd");
	if ( -e "$rc_lvs_home"."rc.lvs")
		{
		$cmd = "/bin/rm $rc_lvs_home"."rc.lvs";
		#print "$cmd \n";
		system ("$cmd");
		}
	$cmd = "ln -s $rc_lvs_home"."$rc_file $rc_lvs_home"."rc.lvs";
	#print "$cmd \n";
	system ("$cmd");
	print "setup director \n";
	print "running rc.lvs on director \n";
	$cmd = "$rc_lvs_home"."$rc_file >/dev/null";
	#print "$cmd \n";
	$return_code = system ("$cmd");
	if ($return_code ne 0)
		{
		print "\n";
		print "rc.lvs failed on director. \n";
		print "exiting lvs setup.\n";
		}
	else
		{
		print " \n";
		print "setup real-servers \n";
		#array @server_names has the server_name 
		#for each time a service was added with ipvsadm.
		#remove redundant entries.
		#Ram book, Sec 4.6
		foreach $item (@server_names)
			{
			unless ($seen{$item})
				{
				#if we get here, we haven't seen it before
				$seen{$item} = 1;
				push (@uniq_server_names, $item);
				}
			}
		
		foreach $server_name (@uniq_server_names)	
			{
			#$server_name may be an IP, find name too.
			($server_name,$server_ip) = parse_hostname($server_name);
			#loop through IPs of known realservers comparing them to the IP of this machine.
			#don't test for 127.0.0.1
			if ($server_name eq "localhost") {next;}
			#this code will print a $server_name for each of its services
			print "\n";
			print "setting up $server_name \n";
			print "copying files \n";
			#don't copy mon files to real-server, mon only runs on director

			#rc.lvs
			#verbose
			$cmd = "/bin/scp -v "."$rc_lvs_home"."$rc_file"." $server_name".":"."$rc_lvs_home"."rc.lvs";
			#not so verbose
			$cmd = "/bin/scp "."$rc_lvs_home"."$rc_file"." $server_name".":"."$rc_lvs_home"."rc.lvs";
			#print "command=$cmd \n";
			system ($cmd);
	
			#ntpd	
			$cmd = "/bin/scp "."$rc_ntp_home"."$realserver_ntp_file"." $server_name".":"."$rc_ntp_home"."ntp.conf";
			#print "command=$cmd \n";
			system ($cmd);
		
			#run rc.lvs
			print "running rc.lvs on $server_name \n";
			#$cmd = "/usr/local/bin/ssh $server_name '. /etc/rc.d/rc.lvs' >/dev/null";
			$cmd = "/bin/ssh $server_name '. /etc/rc.d/rc.lvs' >/dev/null";
			#print "$cmd \n";
			system ($cmd);
	
#			#run multiscan
#			if ( -x "/usr/local/bin/multiscan" )
#				{
#				print "ports visible on $server_name from director. \n";
#				print "(should be none) \n";
#				$cmd = "/usr/local/bin/multiscan -t $server_name";
#				#print "$cmd \n";
#				system ($cmd); 
#				print "ports listening on $vip on $server_name. \n";
#				print "(should only be lvs services.) \n";
#				#$cmd = "/usr/local/bin/ssh $server_name '/usr/local/bin/multiscan -t $vip'";
#				$cmd = "$SSH $server_name '/usr/local/bin/multiscan -t $vip'";
#				#print "$cmd \n";
#				system ($cmd); 
#				}
#			else
#				{
#				print " \n";
#				print "/usr/local/bin/multiscan not found \n";
#				print "will not port scan $server_name \n";
#				}
			print "\n";
			}#foreach $server_name
		}#succeeds on director
	}#copy_and_execute_rc_files

sub do_for_director_or_realserver
	{
	print RC_FILE "	#Determine if I'm a realserver, director (or fail)\n";
	print RC_FILE "	#Am I a server?\n";
	print RC_FILE "	#The IPs listed by ifconfig are grep'ed against the list of server IPs.\n";
	print RC_FILE "	#If there are no matches (== 0), then I'm not a server.\n";
	print RC_FILE "	#(Note the blank after the IP.\n"; 
	print RC_FILE "	#Without the blank, \"192.168.1.1\" matches \"192.168.1.1x\", but \"192.168.1.1 \" will not.\n";
	print RC_FILE "	#a similar problem exists at the front of the IP, but there's no easy way to handle it.\n";
	print RC_FILE "	#In linux the previous char is a blank or \'\', in solaris it's \'#\' \n";
	print RC_FILE "	#If I'm not a server, then maybe I'm the director.\n";
	print RC_FILE "	#If I'm the director, then ipvsadm will be on this machine.\n";
	print RC_FILE "	#If this fails, give up, exit and cry for help\n";
	print RC_FILE "	\n";
	print RC_FILE "	if \n";
	while ($server_ip = $servers[0])	#$server_ip may be a $server_name
		{
		($server_name,$server_ip) = parse_hostname($server_ip);
		#loop through IPs of known realservers comparing them to the IP of this machine.
		shift @servers;
		#don't test for 127.0.0.1, all machines have this, and they weren't counted with $server_counter
		if ($server_ip eq "127.0.0.1") {next;}
		$server_counter--;
		#print "server_counter $server_counter $server_ip\n";
		#print RC_FILE "	[ `\$IFCONFIG $server_net_device | \$GREP -c \"[ #]$server_ip \"` = \"0\" ] ";
		print RC_FILE "	[ `\$IFCONFIG $server_net_device | \$GREP -c \"$server_ip \"` = \"0\" ] ";
		#are we at the last server?
		if ($server_counter > 0) 
			{
			#not last server
			print RC_FILE "	 && \\\n";
			}
	#	else	
	#		{
	#		#last server
	#		print RC_FILE "	 \n";
	#		}
		} #while more servers
	print RC_FILE "	\n";
	print RC_FILE "	then \n";
	print RC_FILE "		{\n";
	print RC_FILE "		#used to test for ipvsadm \n"; 
	print RC_FILE "		#I'm the director, then \$IPVSADM should be on this machine\n";
	print RC_FILE "		#if [ -x \$IPVSADM ] \n";
	print RC_FILE "		#now test for DIP \n"; 
	print RC_FILE "		#note blank after DIP \n";
	print RC_FILE "		if [ `\$IFCONFIG -a | \$GREP -c \"\$DIP \"` = \"1\" ] \n";
	print RC_FILE "		then\n";
	print RC_FILE "			{\n";
	print RC_FILE "			WHOAMI=\"DIRECTOR\" \n";
	print RC_FILE "			setup_director\n";
	print_director_ntp_file;
	print RC_FILE "			}\n";
	print RC_FILE "		else\n";
	print RC_FILE "			{\n";
	print RC_FILE "			WHOAMI=\"UNKNOWN\" \n";
	print RC_FILE "			echo \"error: not a server, not a director\"\n";
	print RC_FILE "			echo \"I'm not a server machine - \"\n";
	print RC_FILE "			echo \"this machine doesn\'t have any of the IPs listed in the SERVICE lines.\"\n";
	#print RC_FILE "			echo \"I'm not a director - I can't execute \$IPVSADM\"\n";
	print RC_FILE "			echo \"I'm not a director - I don't have the DIP \$DIP\"\n";
	print RC_FILE "			echo \"bummer - exiting.\"\n";
	print RC_FILE "			#return error in case this script is executed by another script\n";
	print RC_FILE "			exit 1\n";
	print RC_FILE "			}\n";
	print RC_FILE "		fi\n";
	print RC_FILE "		}\n";
	print RC_FILE "	else \n";
	print RC_FILE "		{\n";
	print RC_FILE "		WHOAMI=\"REALSERVER\" \n";
	print RC_FILE "		whats_my_ip\n";
	print RC_FILE "		setup_realserver\n";
	print_realserver_ntp_file;
	#figure out devices inside setup_core_filter_rules
	print RC_FILE "		FILTERED_DEVICE=\$SERVER_NET_DEVICE \n";
	print RC_FILE "		FILTERED_NETMASK_CLASSLESS=\$VIP_NETMASK_CLASSLESS \n";
	print RC_FILE "		FILTERED_IP=\$VIP \n";
	if ($filters_on eq "y")
		{
		print RC_FILE "		setup_core_filter_rules\n";
		if ($poke_holes eq "y")
			{
			print RC_FILE "		poke_holes \n";
			}
		add_service_rules; 
		print RC_FILE "		show_filter_rules \n";
		}
	else
		{
		print RC_FILE "		echo \"not adding filter rules.\"\n";
		}
	print RC_FILE "		}\n";
	print RC_FILE "	fi\n";
	}

sub test_same_network
	{
	#tests if an IP is in the network
	#eg 192.168.1.110 is     in the same network as 192.168.1.0/255.255.255.0
	#eg 192.168.2.110 is NOT in the same network as 192.168.1.0/255.255.255.0
	#call with 3 arguements
	#1. IP of interest
	#2. network
	#3. netmask (in dotquad notation).
	#
	#returns Y|N
	
	my ($ip_number, $network_number, $netmask_number, $masked_ip_number);
	$ip_number=dotquad_to_number($_[0]);
	$network_number=dotquad_to_number($_[1]);
	$netmask_number=dotquad_to_number($_[2]);
	$masked_ip_number=$ip_number&$netmask_number;
	#print "ip_number=$ip_number, network_number=$network_number, netmask_number=$netmask_number, masked_ip_number=$masked_ip_number\n";
	if ($masked_ip_number eq $network_number)
		{
		#print "test_same_network: in same network \n";
		return ("Y");
		} 
	else
		{
		#print "test_same_network: NOT in same network \n";
		return ("N");
		}
	} #test_same_network

sub find_short_device
	{
	$_[0] =~ /([a-z0-9]+):?[0-9]?/;
	return ($1);
	}


sub read_conf_file
{
	#improved version by Jon Molin, Jon.Molin@refeger.se, Feb 2002
	#checks the number of parameters for each input line
	#and gives error message if not enough input data.
	#
	#print "reading conf_file $conf_file \n";
	my $conf_errors = 0;
	while (<CONF_FILE>)
	{
	$line_counter++;
	chomp;
	#ignore comments (lines starting with "#')and blank lines
	s/\#.+$//;
	#skip empty lines
	next if $_ =~ /^#|^\s*$/;
	#split on '=' or white space (the '=' in the conf file is for readability only)	
	#@params = split /=|\s+/;
	@params = split /=|\s+/,$_;
	#useful print for debug	
	#print "$params[0] \n";
	my $name = $params[0];
	if ($params[0] eq "LVSCONF_FORMAT")
	{
		$lvsconf_format = '';
		shift @params;
		$unknown_format = $params[0]; #save it incase LVSCONF_FORMAT is unknown.
		foreach $known_format (@formats_ok)
		{
		if ($unknown_format eq $known_format)
		{
			$lvsconf_format_is_ok = "Y";
			$lvsconf_format = $known_format;
			#print "$known_format \n";
			#print "LVSCONF_FORMAT = $lvsconf_format \n";
			last;
		}
		}
		if (!$unknown_format || $lvsconf_format_is_ok ne 'Y')
		{
		print STDERR "unknown format for LVSCONF_FORMAT $unknown_format\n";
		$conf_errors += 1;
		}
		shift @params;
	}
	elsif ($params[0] eq "LVS_TYPE")
	{
		$lvs_type = '';
		shift @params;
		if (uc($params[0]) =~/TUN/) 
		{ 
		$lvs_type = "tun"; 
		$connection_type= '$TUN';
		$connection_parameter='-i';
		}
		elsif (uc($params[0]) =~/NAT/) 
		{ 
		$lvs_type = "nat"; 
		$connection_type= '$NAT';
		$connection_parameter= '-m';
		}
		elsif (uc($params[0]) =~/DR/) 
		{ 
		$lvs_type = "dr"; 
		$connection_type= '$DR';
		$connection_parameter= '-g';
		}
		else
		{
		print STDERR "unknown LVS_TYPE $lvs_type\n";
		$conf_errors += 1;
		}
		#print "LVS_TYPE: $lvs_type \n";
		#print_headers;
		shift @params;
	}
	elsif ($params[0] eq "INITIAL_STATE")
	{
		shift @params;
		if (uc($params[0]) =~/OFF/)
		{
		$initial_state = "OFF";
		}
		elsif (uc($params[0]) =~/ON/)
		{
		$initial_state = "ON";
		}
		else
		{
		print STDERR "unknown INITIAL_STATE $params[0]\n";
		$conf_errors += 1;
		}
		shift @params;
	}
	elsif ($params[0] eq "CLEAR_IPVS_TABLES")
	{
		shift @params;
		if (uc($params[0]) =~/YES|Y/)
		{
		$clear_ipvs_tables = "YES";
		}
		elsif (uc($params[0]) =~ /NO|N/)
		{
		$clear_ipvs_tables = "NO";
		}
		else
		{
		print STDERR "unknown CLEAR_IPVS_TABLES $params[0]";
		$conf_errors += 1;
		}
		shift @params;
	}
	elsif ($params[0] eq "VIP") 
	{
		shift @params;
		($director_vip_device, $vip, $vip_netmask, $vip_broadcast) = @params;
		#vip could be a hostname;
		($server_name,$vip) = parse_hostname($vip);
		#director_vip_device could be an alias eg eth0:110
		#find actual device for VIP
		#$director_vip_device =~ /([a-z]+):?[0-9]?/;
		#$short_director_vip_device = $1;
		$short_director_vip_device = find_short_device($director_vip_device);
		#print "read_conf_file: vip info $director_vip_device $short_director_vip_device $vip $vip_netmask $vip_broadcast \n";
		if (!($director_vip_device && $vip && $vip_netmask && $vip_broadcast))
		{
		$conf_errors += 1;
		print STDERR "All VIP values not set: dev=$director_vip_device, ";
		print STDERR "ip=$vip, netmask=$vip_netmask, broadcast=$vip_broadcast\n";
		}
		#initialise a few things
		#print "read_conf_file: initialising persistence set for vip $vip \n";
		$persistence_set{$vip} = "N";
		shift @params;
		shift @params;
		shift @params;
		shift @params;
	}
	elsif ($params[0] eq "DIP") 
	{
		shift @params;
		($dip_device, $dip, $realserver_network, $realserver_netmask, $realserver_broadcast) = @params;
		#DIP device could be an alias eg eth0:110
		#find actual device for DIP
		#find everything infront of ':' or the whole thing if no ':'
		#eth0->eth0, eth0:1->eth0
		#$dip_device =~ /(.*?):*.*/;
		#$short_dip_device = $1;
		$short_dip_device = find_short_device($dip_device);
		#print "short_dip_device $short_dip_device \n";
		#DIP could be a hostname
		($server_name,$dip) = parse_hostname($dip);
		if (!($dip_device, $dip, $realserver_network, $realserver_netmask, $realserver_broadcast))
		{
			print STDERR "Not all DIP values set: dev=$dip_device, ip=$dip,";
			print STDERR " network=$realserver_network, ";
			print STDERR "netmask=$realserver_netmask,";
			print STDERR "broadcast=$realserver_broadcast, ";
			print STDERR "server_name=$server_name\n";
			$conf_errors += 1;
		}
		#print "read_conf_file: dip $dip, server_name $server_name \n"; 
		#my $number;
		#$number = dotquad_to_number($realserver_network);
		#print "realserver_network $number \n"; 
		shift @params;
		shift @params;
		shift @params;
		shift @params;
		shift @params;
	}
	elsif ($params[0] eq "DIRECTOR_GW")
	{
		shift @params;
		$director_gw = $params[0];
		#director_gw could be a hostname
		($server_name, $director_gw) = parse_hostname($director_gw);
		#print "director_gw $director_gw \n";
		if (!$director_gw)
		{
			print STDERR "DIRECTOR_GW needs value\n";
			$conf_errors += 1;
		}
		shift @params; #empty
	}
	elsif ($params[0] eq "SERVER_VIP_DEVICE")
	{
		shift @params;
		$server_vip_device= $params[0];
		#$server_vip_device =~ /(.*?):*.*/;
		#$short_server_vip_device = $1;
		$short_server_vip_device = find_short_device($server_vip_device);
		if (!$server_vip_device)
		{
			print STDERR "SERVER_VIP_DEVICE needs value\n";
			$conf_errors += 1;
		}
		#print "read_conf_file: server vip device info $server_vip_device $short_server_vip_device \n";
		shift @params;
	}
	elsif ($params[0] eq "SERVER_NET_DEVICE")
	{
		shift @params;
		$server_net_device = $params[0]; #the ethernet device on the servers 
		if (!$server_net_device)
		{
		print STDERR "SERVER_NET_DEVICE needs value\n";
		$conf_errors += 1;
		}
		#that has the regular ethernet connection 	
		#for the real IP (usually eth0, le0)
		shift @params;
	}
	elsif ($params[0] eq "SERVER_GW")
	{
		shift @params;
		$server_gw = $params[0];
		#server_gw could be a hostname
		($server_name, $server_gw) = parse_hostname($server_gw);
		#print "server_gw $server_gw \n";
		if (!$server_gw)
		{
		print STDERR "SERVER_GW needs value\n";
		$conf_errors += 1;
		}
		shift @params;	#empty
	}	
	elsif ($params[0] eq "SERVICE")
	{
		#Initially I read in a "SERVICE" line one line at a time 
		#and then wrote out the rc.lvs lines immediately.
		#Now I store all the input till EOF(conf_file) 
		#and then output it
		#
		add_service_line;
		shift @params while (@params);
	} # $params[0]=SERVICE

	#check_conf_file_input_line;
	if ($#params >= 0)
	{
		print STDERR "$name has too many inputs".$#params."(@params)\n";
		$conf_errors += 1;
	}
		
	} # while CONF_FILE
	if ($conf_errors)
	{
	print STDERR "exiting due to error cfg values: $conf_errors errors.\n";
	exit -1;
	}

}

sub old_read_conf_file
	{
	#print "reading conf_file $conf_file \n";
	while (<CONF_FILE>)
		{
		$line_counter++;
		chomp;
		#ignore comments (lines starting with "#')and blank lines
		next if $_ =~ /^#|^\s*$/;
		#ignore anything after a '#'
		if ($_ =~ /#/) 
			{ 
			#need non-greedy match in case have line like
			#      #    domain             53/tcp          nameserver dns  # name-domain server
			#which is a commented comment
			#you want only the stuff before the first comment symbol
			$_ =~ /(.*?)#.*/ ; 
			$line = $1;
			}
		else
			{
			$line = $_;
			}
		#any lines blank now?
		next if $line =~ /^\s*$/;
		#print "non commented line $line \n";
		#split on '=' or white space (the '=' in the conf file is for readability only)	
		#@params = split /=|\s+/;
		@params = split /=|\s+/,$line;
		#useful print for debug	
		#print "$params[0] \n";

		if ($params[0] eq "LVSCONF_FORMAT")
			{
			$lvsconf_format = '';
			shift @params;
			$unknown_format = $params[0]; #save it incase LVSCONF_FORMAT is unknown.
			foreach $known_format (@formats_ok)
				{
				if ($params[0] eq $known_format)
					{
					$lvsconf_format_is_ok = "Y";
					$lvsconf_format = $params[0];
					#print "$known_format \n";
					#print "LVSCONF_FORMAT = $lvsconf_format \n";
					last;
					}
				}
			shift @params;
			}
	
		if ($params[0] eq "LVS_TYPE")
			{
			$lvs_type = '';
			shift @params;
			if (uc($params[0]) =~/TUN/) 
				{ 
				$lvs_type = "tun"; 
				$connection_type= '$TUN';
				$connection_parameter='-i';
				}
			if (uc($params[0]) =~/NAT/) 
				{ 
				$lvs_type = "nat"; 
				$connection_type= '$NAT';
				$connection_parameter= '-m';
				}
			if (uc($params[0]) =~/DR/) 
				{ 
				$lvs_type = "dr"; 
				$connection_type= '$DR';
				$connection_parameter= '-g';
				}
			($lvs_type =~ /tun|nat|dr/) || die "unknown LVS_TYPE $lvs_type ";
			#print "LVS_TYPE: $lvs_type \n";
			#print_headers;
			shift @params;
			}
	
		if ($params[0] eq "INITIAL_STATE")
			{
			shift @params;
			if (uc($params[0]) =~/OFF/)
				{
				$initial_state = "OFF";
				}
			elsif (uc($params[0]) =~/ON/)
				{
				$initial_state = "ON";
				}
			else
				{
				die "unknown INITIAL_STATE $params[0]";
				}
			shift @params;
			}

		if ($params[0] eq "CLEAR_IPVS_TABLES")
			{
			shift @params;
			if (uc($params[0]) =~/YES|Y/)
				{
				$clear_ipvs_tables = "YES";
				}
			elsif (uc($params[0]) =~ /NO|N/)
				{
				$clear_ipvs_tables = "NO";
				}
			else
				{
				die "unknown CLEAR_IPVS_TABLES $params[0]";
				}
			shift @params;
			}
	
		if ($params[0] eq "VIP") 
			{
			shift @params;
			($director_vip_device, $vip, $vip_netmask, $vip_broadcast) = @params;
			#vip could be a hostname;
			($server_name,$vip) = parse_hostname($vip);
			#director_vip_device could be an alias eg eth0:110
			#find actual device for VIP
			#$director_vip_device =~ /([a-z]+):?[0-9]?/;
			#$short_director_vip_device = $1;
			$short_director_vip_device = find_short_device($director_vip_device);
			#print "read_conf_file: vip info $director_vip_device $short_director_vip_device $vip $vip_netmask $vip_broadcast \n";
			shift @params;
			shift @params;
			shift @params;
			shift @params;
			}
	
		if ($params[0] eq "DIP") 
			{
			shift @params;
			($dip_device, $dip, $realserver_network, $realserver_netmask, $realserver_broadcast) = @params;
			#DIP device could be an alias eg eth0:110
			#find actual device for DIP
			#find everything infront of ':' or the whole thing if no ':'
			#eth0->eth0, eth0:1->eth0
			#$dip_device =~ /(.*?):*.*/;
			#$short_dip_device = $1;
			$short_dip_device = find_short_device($dip_device);
			#print "short_dip_device $short_dip_device \n";
			#DIP could be a hostname
			($server_name,$dip) = parse_hostname($dip);
			#print "read_conf_file: dip $dip, server_name $server_name \n"; 
			#my $number;
			#$number = dotquad_to_number($realserver_network);
			#print "realserver_network $number \n"; 
			shift @params;
			shift @params;
			shift @params;
			shift @params;
			shift @params;
			}

		if ($params[0] eq "DIRECTOR_GW")
			{
			shift @params;
			$director_gw = $params[0];
			#director_gw could be a hostname
			($server_name, $director_gw) = parse_hostname($director_gw);
			#print "director_gw $director_gw \n";
			shift @params; #empty
			}
	
		if ($params[0] eq "SERVER_VIP_DEVICE")
			{
			shift @params;
			$server_vip_device= $params[0];
			#$server_vip_device =~ /(.*?):*.*/;
			#$short_server_vip_device = $1;
			$short_server_vip_device = find_short_device($server_vip_device);
			#print "read_conf_file: server vip device info $server_vip_device $short_server_vip_device \n";
			shift @params;
			}
	
		if ($params[0] eq "SERVER_NET_DEVICE")
			{
			shift @params;
			$server_net_device = $params[0]; #the ethernet device on the servers 
						#that has the regular ethernet connection 	
						#for the real IP (usually eth0, le0)
			shift @params;
			}

		if ($params[0] eq "SERVER_GW")
			{
			shift @params;
			$server_gw = $params[0];
			#server_gw could be a hostname
			($server_name, $server_gw) = parse_hostname($server_gw);
			#print "server_gw $server_gw \n";
			shift @params;	#empty
			}
	
		if ($params[0] eq "SERVICE")
			{
			#Initially I read in a "SERVICE" line one line at a time 
			#and then wrote out the rc.lvs lines immediately.
			#Now I store all the input till EOF(conf_file) 
			#and then output it
			#
			add_service_line;
			} # $params[0]=SERVICE

		check_conf_file_input_line;
	
		} # while CONF_FILE
	}

sub check_kernel_version
	{
	#changes can be associated with the kernel, or with ipvsadm.
	#It's a bit hard to keep track of both the kernel and ipvsadm.
	#For the moment, just keep track of the kernel.
	#are we running this on a kernel version that has been tested with configure?
	$uname_r = `uname -r`;
	chomp ($uname_r);
	#print "uname_r $uname_r \n";
	#wierd names. 
	#patched kernels have names like 2.2.15pre9, 2.3.99-pre8 or 2.4.0-test1
	#I was hoping to match on the pre until the -pre and -test format came along.
	#Now I just look for non-digits.
	#originally I decided that a 2.2.15pre9 kernel should be treated as a 2.2.14
	#then 2.4.0-test came along. I can't treat it as a "-1". Will treat "0" as a special case
	#the kernel could be 2.2.15pre5 -rev it down to 2.2.14
	#test cases
	#$uname_r = "2.2.1test-9";
	#$uname_r = "2.2.1-test9";
	#$uname_r = "2.2.1";
	$uname_r =~ /(\d)\.(\d)\.(\d{1,3})[\-A-Za-z]*(\d{0,3})/ ;
	#print "kernel version $1 $2 $3 $4\n";
	$kernel_series ="$1.$2";
	#print "kernel_series $kernel_series \n";
	if ($4 eq "")
		{
		#print "standard release \n";
		#print "kernel version $1.$2.$3\n";
		}
	else	
		{
		#print "pre or test release \n";
		#print "$1.$2.$3pre$4;\n";
		$minor_rev = $3 - 1;
		if ($minor_rev eq -1) {$minor_rev = 0;}
		print "kernel $uname_r is being treated as ";
		$uname_r = "$1.$2.$minor_rev";
		print "$uname_r\n";
		}
	#print "uname $uname_r \n";
	
	#check if this is a known $uname_r
	foreach $uname_r_item (@uname_r_ok)
		{
		if ($uname_r eq $uname_r_item)
			{
			$uname_r_known = "Y";
			last;
			}
		#print "$uname_r_item \n";
		}
	if ($uname_r_known eq "N")
		{
		print "warning: $0 has not been tested with kernel version $uname_r \n";
		}
	}	

sub initialise_variables_for_reading_conf_file
	{
	#when configure.pl becomes lvsd, 
	#some variables will have to be initialised 
	#when a new conf_file is read in.
	$conf_file = "";
	%servers_and_services = {};
	%services_and_params = {};
	}

sub write_director_setup
	{
	print RC_FILE "\n";
	print RC_FILE "setup_director() { \n";
	print RC_FILE "	echo \"LVS director \"\n";
	print RC_FILE "	echo \"\"\n";
	print RC_FILE "	load_function ip_vs \n";
	print RC_FILE "	clear_ip_tables_chains \n";
	print RC_FILE "	#make sure you set ip_forward before you add the VIP to the director \n";
	print RC_FILE "	#because if the VIP is by TP, then TP will set forwarding on. \n";
	print RC_FILE "	#These settings are just to close down anything not needed for LVS, \n";
	print RC_FILE "	#just to be safe. \n";
	print RC_FILE "	#If you do this in the reverse order, \n";
	print RC_FILE "	#then you'll override the settings needed for TP \n";
	print RC_FILE "	set_director_ip_forwarding \n";
	print RC_FILE "	turn_off_icmp_redirects \n";
	print RC_FILE "	echo \"\"\n";
	print RC_FILE "	NEW_VIP_DEVICE=\$DIRECTOR_VIP_DEVICE \n";
	print RC_FILE "	remove_old_vip \n";
	print RC_FILE "	NEW_VIP_DEVICE=\"\" \n";
	print RC_FILE " \n";
	if ($director_vip_device eq "TP")
		{
		#print "setting up TP for VIP director \n";
		setup_director_TP;
		}
	else	{
		#print "setting up network device for VIP on director \n";
		print RC_FILE "	setup_director_vip \n";
		}
	print RC_FILE "	setup_ipvsadm_table \n";
	#Aug 2001, the order of the next two lines was exchanged
	print RC_FILE "	from_director_check_target_of_server_gw \n";
	print RC_FILE "	echo \"DIRECTOR_GW=\$DIRECTOR_GW\" \n";
	print RC_FILE "	install_director_gw \n";
	print RC_FILE "	echo \"DIRECTOR_GW=\$DIRECTOR_GW\" \n";
	#write masquerading rules for 2.0.x, 2.2.x if VS-NAT
	setup_director_masq_table;
	#setup devices inside setup_core_filter_rules
	print RC_FILE "	FILTERED_DEVICE=\$SHORT_DIRECTOR_VIP_DEVICE \n";
	print RC_FILE "	FILTERED_NETMASK_CLASSLESS=\$VIP_NETMASK_CLASSLESS \n";
	print RC_FILE "	FILTERED_IP=\$VIP \n";
	if ($filters_on eq "y")
		{
		print RC_FILE "	setup_core_filter_rules \n";
		if ($poke_holes eq "y")
			{
			print RC_FILE "		poke_holes \n";
			}
		add_service_rules; 
		print RC_FILE "	show_filter_rules \n";
		}
	else
		{
		print RC_FILE "		echo \"not adding filter rules.\"\n";
		}
        print RC_FILE "	restart_mon \n";
        print RC_FILE "	restart_ntpd \n";
        print RC_FILE "	echo \"\"\n";
        print RC_FILE " } #setup_director\n";
        print RC_FILE "\n";
	}

sub write_main
	{
	my $module;
	#this becomes main: in the output rc.lvs shell file
	print RC_FILE "#-----------------------------------------\n";
	print RC_FILE "#main: \n";
	print RC_FILE "#configure_lvs.pl \n";
	print RC_FILE "VERSION=\"$configure_version\" \n";
	print RC_FILE "DATE=\"$configure_date\" \n";
	print RC_FILE "#rc.lvs derived from conf file $conf_file \n";
	print RC_FILE " \n";
	print RC_FILE "#Users will have at least this PATH \n"; 
	print RC_FILE "#make sure we can run with the minimum expected PATH.\n"; 
	print RC_FILE "PATH=\"/bin;/usr/bin\" \n"; 
	print RC_FILE "#did we get any command line params? \n";
	print RC_FILE "#allowed start|stop|delete|restart|\'\'(ie no params) \n";
	print RC_FILE "ACTION=\$1 \n"; 
	print RC_FILE "case \"\$ACTION\" in \n";
	print RC_FILE "restart) \n";
        print RC_FILE "	\$0 stop \n";
        print RC_FILE "	\$0 start \n";
        print RC_FILE "	EXITCODE=0 \n";
        print RC_FILE "	;; \n";
        print RC_FILE "	\n";

	print RC_FILE "delete) \n";
        print RC_FILE "	echo \"\" \n";
        print RC_FILE "	echo \"current entries in ipvsadm table. \" \n";
        print RC_FILE "	\$IPVSADM  \n";
        print RC_FILE "	echo \"deleting all entries in ipvsadm table. \" \n";
        print RC_FILE "	\$IPVSADM -C \n";
        print RC_FILE "	;; \n";
        print RC_FILE "	\n";

	print RC_FILE "stop) \n";
        print RC_FILE "	echo \"\" \n";
        print RC_FILE "	echo \"setting connection weights to 0 (i.e. stopping new connections)\" \n";
        print RC_FILE "	echo \"and waiting for connections to drop. \" \n";
        print RC_FILE "	echo \"This command is not activated yet (ie it's a null command). \" \n";
        print RC_FILE "	;; \n";
        print RC_FILE "	\n";

	print RC_FILE "*|start) \n";
	#initialise a few things
	print RC_FILE "	NUM_ERRORS=0 \n";
	print RC_FILE "	NUM_WARNINGS=0 \n";
	print RC_FILE "	NAT=\"-m\" \n";
	print RC_FILE "	DR=\"-g\" \n";
	print RC_FILE "	TUN=\"-i\" \n";

	#hide VIP on real-servers?
	#from "User configurable" variables
	$hidden=uc($hidden);
	print RC_FILE "	HIDDEN=$hidden \n";
	$router_forwards=uc($router_forwards);
	print RC_FILE "	ROUTER_FORWARDS=$router_forwards \n";
	print RC_FILE "	PERSISTENCE_TIMEOUT=360 \n";
	if ($router_forwards eq "")
                {
                print "Notice: you haven't set the variable \$router_forwards \n";
                print "in the user configurable section. \n";
                print "When you run the rc.lvs script on a real-server \n";
                print "(after running it on the director first), \n";
                print "the outwill tell you whether to set \$router_forwards to Y|N.\n";
                print "More information is available with perdoc in the configure documentation.\n";
                }

	print RC_FILE "	find_standard_executables\n";
	print RC_FILE "	hello_notice\n";
	print RC_FILE "	find_System_map\n";

	#assign input params (in case need to use them latter)
	#general stuff
	print RC_FILE "	LVSCONF_FORMAT=\"$lvsconf_format\" \n";
	print RC_FILE "	INITIAL_STATE=\"$initial_state\" \n";
	print RC_FILE "	CLEAR_IPVS_TABLES=\"$clear_ipvs_tables\" \n";
	#originally I determined this in the perl code, which is run on the director
	#giving me the kernel_series for the director, 
	#which is not what I wanted.
	#I wanted the kernel_series for the machine the shell script code is running on.
	#now is handled by LINUX_KERNEL_SERIES.
	#print RC_FILE "	KERNEL_SERIES=\"$kernel_series\" \n";
	print RC_FILE "	if [ \$LINUX_KERNEL_SERIES = \"2.0\" -o \$LINUX_KERNEL_SERIES = \"2.2\" -o \$LINUX_KERNEL_SERIES = \"2.4\" ] \n";
	print RC_FILE "	then \n";
	print RC_FILE "		echo \"\" \n";
	print RC_FILE "	else \n";
	print RC_FILE "		echo \"Error: kernel series \$LINUX_KERNEL_SERIES not handled by this code \" \n";
	print RC_FILE "		echo \"you may get errors depending what you ask it to do. \" \n";
	print RC_FILE "		echo \"supported are Linux 2.2.x and 2.4.x \" \n";
	print RC_FILE "		((NUM_ERRORS++)) \n";
	print RC_FILE "	fi \n";
	print RC_FILE "	find_kernel_dependant_executables \n";
	print RC_FILE "	find_module_executables \n";

	#LVS_TYPE
	print RC_FILE "	LVS_TYPE=\"vs-$lvs_type\" \n";
	print RC_FILE "	CONNECTION_TYPE=\"$connection_type\" \n";
	print RC_FILE "	CONNECTION_PARAMETER=\"$connection_parameter\" \n";

	#VIP,DIRECTOR
	print RC_FILE "	VIP=\"$vip\" \n";
	print RC_FILE "	DIRECTOR_VIP_DEVICE=\"$director_vip_device\" \n";
	print RC_FILE "	SHORT_DIRECTOR_VIP_DEVICE=\"$short_director_vip_device\" \n";
	print RC_FILE "	VIP_NETMASK=\"$vip_netmask\" \n";
	my $vip_netmask_classless = dotquad_to_classless($vip_netmask);
	print RC_FILE "	VIP_NETMASK_CLASSLESS=\"$vip_netmask_classless\" \n";
	print RC_FILE "	VIP_BROADCAST=\"$vip_broadcast\" \n";
	print RC_FILE "	DIP=\"$dip\" \n";
	print RC_FILE "	DIP_DEVICE=\"$dip_device\" \n";
	print RC_FILE "	SHORT_DIP_DEVICE=\"$short_dip_device\" \n";
	print RC_FILE "	REALSERVER_NETWORK=\"$realserver_network\" \n";
	print RC_FILE "	REALSERVER_NETMASK=\"$realserver_netmask\" \n";
	my $realserver_netmask_classless = dotquad_to_classless($realserver_netmask);
	print RC_FILE "	REALSERVER_NETMASK_CLASSLESS=\"$realserver_netmask_classless\" \n";
	print RC_FILE "	DIRECTOR_GW=\"$director_gw\" \n";
	#NICs on director
	print RC_FILE "	if [ \"\$SHORT_DIRECTOR_VIP_DEVICE\" = \"\$SHORT_DIP_DEVICE\" ] \n";
	print RC_FILE "	then \n";
	print RC_FILE "		NUMBER_DIRECTOR_NICS=1\n";
	print RC_FILE "	else \n";
	print RC_FILE "		NUMBER_DIRECTOR_NICS=2\n";
	print RC_FILE "	fi \n";
	print RC_FILE "	echo \"number nics on director \$NUMBER_DIRECTOR_NICS\" \n";

	#services are arrays, don't output them here

	#real-server
	print RC_FILE "	SERVER_VIP_DEVICE=\"$server_vip_device\" \n";
	print RC_FILE "	SHORT_SERVER_VIP_DEVICE=\"$short_server_vip_device\" \n";
	print RC_FILE "	IS_SERVER_VIP_DEVICE_LOCAL=\"$is_server_vip_device_local\" \n";
	print RC_FILE "	SERVER_NET_DEVICE=\"$server_net_device\" \n";
	print RC_FILE "	SERVER_GW=\"$server_gw\" \n";

	#test if VIP is in same network as real-servers.
	#real-server setup pings VIP from real-server. 
	#need to know expected result.
	if (test_same_network($vip, $realserver_network, $realserver_netmask) eq "Y")
		{
		print RC_FILE "	VIP_RIP_SAME_NETWORK=\"Y\" \n";
		}
	else
		{
		print RC_FILE "	VIP_RIP_SAME_NETWORK=\"N\" \n";
		}

	if (test_same_network($director_gw, $realserver_network, $realserver_netmask) eq "Y")
		{
		print RC_FILE "	DIRGW_RIP_SAME_NETWORK=\"Y\" \n";
		}
	else
		{
		print RC_FILE "	DIRGW_RIP_SAME_NETWORK=\"N\" \n";
		}
	print RC_FILE "	\n";

	#look for device driver modules from user configurable @network_driver_modules
	print RC_FILE "	ETH_DEVICE_MODULES=\""; #note no <cr>
	foreach $module (@network_driver_modules)
		{
		print RC_FILE "$module "; #note no <cr>
		}
	print RC_FILE "\"\n";	#here's the <cr>
	#did we use iproute2 tools?
	print RC_FILE "	IPROUTE2_USED=\"N\" \n";

	#the main function of main
	do_for_director_or_realserver;

	print RC_FILE "	\n";
	#are the output directories at their default location?
	if ($rc_lvs_home eq "./")
		{
        	print RC_FILE "	echo \"The location of the output file rc.lvs is the default = ./\" \n";
        	print RC_FILE "	echo \'You can change this (and other output files) by editing the variables \$rc_lvs_home, \$rc_mon_home, \$rc_ntp_home \' \n";
        	print RC_FILE "	echo \" \" \n";
		}
	else
		{
		print RC_FILE "\n";
		#print RC_FILE "	echo \"the directories for the output file has been changed by the user from the default location of ./ \" \n";
		}

	print RC_FILE "	if [ \"\$IPROUTE2_USED\" = \"Y\" ] \n";
	print RC_FILE "	then \n";
	print RC_FILE "		echo \"Some iproute2 commands have been executed.\" \n";
	print RC_FILE "		echo \"Use the ip tools to inspect your routing and network devices.\" \n";
	print RC_FILE "		echo \"The output of ifconfig and netstat may not be valid.\" \n";
	print RC_FILE "	fi \n";

	print RC_FILE "	report_errors \n";
        print RC_FILE "	;; \n";
	#print RC_FILE " \n";
	print RC_FILE "esac \n";
	print RC_FILE " \n";
	print RC_FILE "#rc.lvs_${lvs_type}----------------------\n";
	} # write_main

#--------------------------------------------------
#perl main:

if (@ARGV == 0)
	{
	print "configure_lvs.pl v$configure_version $configure_date\n";
	print "usage: ./configure_lvs.pl conf_file [-i]\n";
	exit;
	}

initialise_variables_for_reading_conf_file;
$conf_file = shift(@ARGV);
#print "conf_file: $conf_file\n";
open (CONF_FILE, "<$conf_file")|| die "Can't read $conf_file $!\n";

if (@ARGV)
	{
	#something left in @ARGV
	$command_line_argument = shift(@ARGV);
	#print "command_line_argument $command_line_argument \n";
	}

check_kernel_version;	#looks at kernel version on director.
			#kernel version on realserver 
			#determined in detect_realserver_os when shell script is run
is_kernel_too_old;	#needs $uname_r from check_kernel_version

#get input
read_conf_file;
print_verify_inputs;

#start output files and write stuff at the top of them
create_directories;
open_files;
print_headers;

#write subroutines, 
#can't do this till have opened file(s)
install_print_ipvsadm_lines;
install_print_redirect_services;

#setup director and mon 
write_director_setup;
print_close_mon_file;

#main
write_main;

close DIRECTOR_NTP_FILE;
close REALSERVER_NTP_FILE;
close MON_FILE;
close RC_FILE;
#make rc.lvs executable
#umask must be octal, ie have a leading 0, eg 0755
my @cannot;
@cannot = grep {not chmod 0755, $_} "$rc_lvs_home"."$rc_file";
die "$0: could not chmod @cannot\n" if @cannot;

if ($command_line_argument eq "-i")
	{
	#copy output files to real-servers, execute rc.lvs
	copy_and_execute_rc_files;
	}
else
	{
	#print "rc.lvs file not executed on real-servers \n";
	}

#configure_lvs-0.9.4.pl---------------------------------
